/* ******************************************************************
 * Function name : str_convert(string)
 *
 * Description  : This function is used to convert lower
 *                 case alpha character  to upper case and
 *                 upper case alpha characters to lower case.
 *
 * Variables      : string  -  This parameter is a character
 *                             pointer to the string that is to be
 *                             modified.
 *
 * Example       : strcpy(str,"ABC");
 *                 output = str_convert(str);    returns the string abc
 *                 strcpy(str,"abC");
 *                 output = str_convert(str);    returns the string ABc
 *
 * Rules         : 1) The function must be passed a character pointer
 *                    containing the address of the character string to
 *                    be converted.
 *
 *                 2) This function does not use a return value. Therefore
 *                    it may be defined using the "void" data type if
 *                    it is supported by your compiler.
 *
 *
 */
str_convert(string)
 char string[];
{ int count;
  count = 0;
  while ( string[count] != '\0' )
     { if ( string[count] >= 'A' && string[count] <= 'Z' )
	string[count] = string[count] + 'a' - 'A';
       else
	 if ( string[count] >= 'a' && string[count] <= 'z' )
	    string[count] = string[count] - 'a' + 'A';
       count++;
     }
}

/* ******************************************************************
 * Function name : str_count(string)
 *
 * Description   : This function counts the number of characters
 *                 contained within a specified character string.
 *
 * Variables      : string  -  This parameter is a character
 *                             pointer to the string that is to be
 *                             counted.
 *
 *
 * Example       : str_convert("abcd");       returns a 4
 *                 str_convert("ab");         returns a 2
 *
 * Rules         : 1) The function must be passed a character pointer
 *                    containing the address of the character string
 *                    being analyzed.
 *
 *                 2) The return value must be passed to an integer
 *                    variable.
 *
 *                 3) The string being analyzed must be followed
 *                    by a null terminator "\0" value.
 *
 *
 */
str_count(string,sub_string)
 char string[];
 char sub_string;
{ int count, index;
  count = index = 0;
  while ( string[count] != '\0' )
    {   if ( string[count++] == sub_string ) index++;
    }
  return(index);
}

/* ******************************************************************
 * Function name : str_lindex(string, sub )
 *
 *  Description   : This function returns an integer value, indicating
 *                  the first location of the specified sub-string
 *                  character within the string being analyzed.
 *
 * Variables      : string  -  This parameter is a character
 *                             pointer to the string that is to be
 *                             counted.
 *
 *                  sub     - This parameter contains the character
 *                            being searched.
 *
 * Example       : output = str_convert("abcde",'c');  returns a 3
 *                 output = str_convert("abcd",'d');   returns a 4
 *                 output = str_convert("abcd",'b');   returns a 2
 *
 * Rules         : 1) The function must be passed a character pointer
 *                    containing the address of the character string
 *                    being analyzed.
 *
 *                 2) The return value of this funtion must be passed
 *                    to an integer variable.
 *
 *                 3) The string being analyzed must be followed
 *                    by a null terminator "\0" value.
 *
 */
str_lindex(string,sub_string)
 char string[];
 char sub_string;
{ int index;
  index = 0;

  while ( string[index] != '\0' && string[index] != sub_string)
    ++index;

  return(index);
}

/* ******************************************************************
 * Function name : str_delete(string, start, no_char)
 *
 * Description   : Thi functio deletes a specified group of characters
 *                 from within a character string as defined by the
 *                 second and third parameter.
 *
 * Variables      : string  -  This parameter is a character
 *                             pointer to the string that is t be
 *                             counted.
 *
 *                 start    -  This paramater specifies the location
 *                             of the first character to be deleted.
 *
 *                 no_char  -  This parameter specifies how many
 *                             characters should be deleted.
 *
 * Example       : str_convert("abcdefg",3,2);       returns "abdeg"
 *                 str_convert("abcdefg",2,5);       returns "ag"
 *
 * Rules         : 1) The function must be passed a character pointer
 *                    containing the address of the character string
 *                    being analyzed.
 *
 *                 2) The second and third parameters must be integers.
 *
 *                 3) The string being analyzed must be followed
 *                    by a null terminator "\0" value.
 *
 */
str_delete(string,start,no_char)
 char string[];
 int start, no_char;
{ int index1, index2;
  index1 = index2 = 0;
  index2 = start + no_char;
  for ( index1 = start; string[index1] = string[index2]; index1++)
    index2++;
}

/* ******************************************************************
 * Function name : str_lower(string)
 *
 * Description   : This function is used to convert all upper case
 *                 characters within the passed string to lower case.
 *
 * Variables      : string  -  This parameter is a character
 *                             pointer to the string that is to be
 *                             modified.
 *
 * Example       : strcpy(str,"ABC");
 *                 output = str_convert(str);    returns the string abc
 *                 strcpy(str,"Abcde");
 *                 output = str_convert(str);    returns the string abcde
 *
 * Rules         : 1) The function must be passed a character pointer
 *                    containing the address of the character string to
 *                    be converted.
 *
 *                 2) This function does not use a return value. Therefore
 *                    it may be defined using the "void" data type if
 *                    it is supported by your compiler.
 *
 */
str_lower(string)
 char string[];
{ int count;
  count = 0;
  while ( string[count] != '\0' )
     { if ( string[count] >= 'A' && string[count] <= 'Z' )
	string[count] = string[count] + 'a' - 'A';
       count++;
     }
}

/* ******************************************************************
 * Function name : str_lpad(string, no_char)
 *
 *  Description   : This function places a spedified number of
 *                  spaces ( " " ) at the beginning of the passed
 *                  character string.
 *
 * Variables      : string  -  This parameter is a character
 *                             pointer to the string that is t be
 *                             modified.
 *
 *                  no_char -  This parameter specifies how many
 *                             characters should be placed at the
 *                             beginning of the passed character string.
 *
 * Example       : str_lpad("ab",2);       returns "  ab"
 *                 str_lpad("ab",5);       returns "     ab"
 *
 * Rules         : 1) The function must be passed a character pointer
 *                    containing the address of the character string
 *                    being analyzed.
 *
 *                 2) The second parameter must be an integer.
 *
 *                 3) The string being modified must be followed
 *                    by a null terminator "\0" value.
 *
 *
 */
str_lpad(string,no_char)
 char *string;
 int no_char;
{ char temp;
  int count;
  count = 0;
  strcpy(temp,"");

  for ( count = 0; count < no_char; count++)
    strcat(temp," ");

  strcat(temp,string);
  strcpy(string,temp);

}

/* ******************************************************************
 * Function name : str_rindex(string)
 *
 *  Description   : This function returns an integer value, indicating
 *                  the last occurrence of the specified sub-string
 *                  character within the string being analyzed.
 *
 * Variables      : string  -  This parameter is a character
 *                             pointer to the string that is to be
 *                             counted.
 *
 *                  sub     - This parameter contains the character
 *                            being searched.
 *
 * Example       : output = str_convert("abcda",'a');  returns a 5
 *                 output = str_convert("abcde",'d');  returns a 4
 *                 output = str_convert("abccd",'c');  returns a 4
 *
 * Rules         : 1) The function must be passed a character pointer
 *                    containing the address of the character string
 *                    to be analyzed.
 *
 *                 2) The return value of this funtion must be passed
 *                    to an integer variable.
 *
 *                 3) The string being analyzed must be followed
 *                    by a null terminator "\0" value.
 *
 */
str_rindex(string,sub_string)
 char string[];
 char sub_string;
{ int index1, index2;
  index1 = index2 = 0;

  while ( string[index1] != '\0')
    { if ( string[index1] == sub_string )  index2 = index1;
      ++index1;
    }

  return(index2);
}

/* ******************************************************************
 * Function name : str_rpad(string, no_char)
 *
 *  Description   : This function places a specified number of
 *                  spaces ( " " ) at the end of the passed
 *                  character string.
 *
 * Variables      : string  -  This parameter is a character
 *                             pointer to the string that is t be
 *                             modified.
 *
 *                  no_char -  This parameter specifies how many
 *                             characters should be placed at the
 *                             end of the passed character string.
 *
 * Example       : str_lpad("ab",2);       returns "ab  "
 *                 str_lpad("ab",5);       returns "ab     "
 *
 * Rules         : 1) The function must be passed a character pointer
 *                    containing the address of the character string
 *                    being analyzed.
 *
 *                 2) The second parameter must be an integer.
 *
 *                 3) The string being modified must be followed
 *                    by a null terminator "\0" value.
 *
 */
str_rpad(string,no_char)
 char *string;
 int no_char;
{ int count;
  count = 0;

  for ( count = 0; count < no_char; count++)
    strcat(string," ");

}

/* ******************************************************************
 * Function name : str_swap(string1, string2)
 *
 *  Description   : This function exchanges the contents of two
 *                  character variable.
 *
 * Variables      : strin1 -  This parameter is a character
 *                             pointer to the first string being
 *                             exchanged
 *
 *                  string2 -  This parameter is a character pointer
 *                             the second string being exchanged.
 *
 * Example       : strcpy(a_string,"AAAAA"
 *                 strcpy(b_string,"BBBBB"
 *                 str_swap(a_string, b_string)
 *                         returns:  a_string is "BBBBB"
 *                                   b_string is "AAAAA"
 *
 * Rules         : 1) The function must be passed a character pointers
 *                    containing the addresses of the character strings
 *                    being swapped.
 *
 *                 2) The character strings must be defined large enough
 *                    to contain each others values.
 *
 *                 3) The string being modified must be followed
 *                    by a null terminator "\0" value.
 *
 */
str_swap(string1, string2)
 char *string1, *string2;
{ char temp;
  strcpy(temp,string1);
  strcpy(string1,string2);
  strcpy(string2,temp);
}

/* ******************************************************************
 * Function name : str_upper(string)
 *
 * Description   : This function is used to convert all lower case
 *                 characters within the passed string to upper case.
 *
 * Variables      : string  -  This parameter is a character
 *                             pointer to the string that is to be
 *                             modified.
 *
 * Example       : strcpy(str,"Abc");
 *                 output = str_convert(str);    returns the string ABC
 *                 strcpy(str,"ABCde");
 *                 output = str_convert(str);    returns the string ABCDE
 *
 * Rules         : 1) The function must be passed a character pointer
 *                    containing the address of the character string to
 *                    be converted.
 *
 *                 2) This function does not use a return value. Therefore
 *                    it may be defined using the "void" data type if
 *                    it is supported by your compiler.
 *
 */
str_upper(string)
 char string[];
{ int count;
  count = 0;
  while ( string[count] != '\0' )
     { if ( string[count] >= 'a' && string[count] <= 'z' )
	 string[count] = string[count] + 'A' - 'a';
       count++;
     }
}
