//==============================================================================================
//
//	Windows Interface Construction Set
//	Version 1.00
//
//	DATEEDT.CPP - Date Edit Class Source File
//	Copyright  1993 by Microdyne Development Technologies.
//	All rights reserved.
//==============================================================================================

#include <owl.h>
#include <dateedt.h>

TDateEdit::TDateEdit (PTWindowsObject AParent, int ResourceId, PTModule AModule)
	: TEdit (AParent, ResourceId, 0, AModule)
{
}

TDateEdit::~TDateEdit ()
{
}

void TDateEdit::WMCommand (RTMessage Msg)
{
	DefWndProc(Msg);
}

void TDateEdit::SetupWindow ()
{
	TEdit::SetupWindow();
}

int TDateEdit::GetLineLength()
{
	TMessage	msg;

	msg.Receiver = HWindow;
	msg.Message  = EM_LINELENGTH;
	msg.WParam   = 0;
	msg.LParam   = 0L;
	msg.Result   = 0L;

	DefWndProc (msg);

	return (int) msg.Result;
}

BOOL TDateEdit::GetLine(LPSTR ATextString, int StrSize)
{
	TMessage	msg;
    int			BytesCopied;
	BOOL		Success;

	if ( StrSize <= 0 )
    	return FALSE;

	if ( (StrSize >= GetLineLength() + 1) )
    	Success = TRUE;
	else
    	Success = FALSE;

	if ( StrSize == 1 )
	{
    	ATextString[0] = '\0';
		return Success;
	}

	msg.Receiver = HWindow;
	msg.Message  = EM_GETLINE;
	msg.WParam   = 0;
	msg.LParam   = (LPARAM) ATextString;
	msg.Result   = 0L;

	DefWndProc (msg);

	BytesCopied = (int) msg.Result;

	if ( BytesCopied != 0 )
	{
		ATextString[BytesCopied] = '\0'; // Windows returns non-null terminated string
		return Success;
	}

	return FALSE;
}

void TDateEdit::GetDate(long & lDays, Rint iMonth, Rint iDay, Rint iYear)
{
	TMessage	msg;
    WORD		wDate[3];

	msg.Receiver = HWindow;
	msg.Message  = DEM_GETDATE;
	msg.WParam   = 0;
	msg.LParam   = (LPARAM) wDate;
	msg.Result   = 0L;

	DefWndProc (msg);

	iMonth = wDate[0];
	iDay   = wDate[1];
	iYear  = wDate[2];

    lDays = msg.Result;
}

void TDateEdit::GetMonth(Rint iMonth)
{
	TMessage	msg;

	msg.Receiver = HWindow;
	msg.Message  = DEM_GETMONTH;
	msg.WParam   = 0;
	msg.LParam   = 0L;
	msg.Result   = 0L;

	DefWndProc (msg);

	iMonth = msg.Result;
}

void TDateEdit::GetDay(Rint iDay)
{
	TMessage	msg;

	msg.Receiver = HWindow;
	msg.Message  = DEM_GETDAY;
	msg.WParam   = 0;
	msg.LParam   = 0L;
	msg.Result   = 0L;

	DefWndProc (msg);

	iDay = msg.Result;
}

void TDateEdit::GetYear(Rint iYear)
{
	TMessage	msg;

	msg.Receiver = HWindow;
	msg.Message  = DEM_GETYEAR;
	msg.WParam   = 0;
	msg.LParam   = 0L;
	msg.Result   = 0L;

	DefWndProc (msg);

	iYear = msg.Result;
}

void TDateEdit::GetEditParams (LPDECPARMS lpep)
{
	TMessage	msg;

	msg.Receiver = HWindow;
	msg.Message  = DEM_GETEDITPARAMS;
	msg.WParam   = 0;
	msg.LParam   = (LPARAM) lpep;
	msg.Result   = 0L;

	DefWndProc (msg);
}

void TDateEdit::SetEditParams (LPDECPARMS lpep)
{
	TMessage	msg;

	msg.Receiver = HWindow;
	msg.Message  = DEM_SETEDITPARAMS;
	msg.WParam   = 0;
	msg.LParam   = (LPARAM) lpep;
	msg.Result   = 0L;

	DefWndProc (msg);
}
