// EDPARSER.CXX - Very Simple Edit control Parser
//               (C) Sturmer Hauss Corp 1991

#define INCL_TIER_TYPES                    // data types like LONG, HWND, HANDLE
#define INCL_TIER_T1OBJECT                 // base class of the TIER1 Library and Window and Overlapped classes 
#define INCL_TIER_WINDOW                   // TIER Window class
   #define INCL_TIER_WINDOWMSGS            // MS-Window compatible message #defines
   #define INCL_TIER_WINDOWSTYLES          // MS-Window compatible window style #defines
      #define INCL_TIER_DIALOG             // Dialog class
         #define INCL_TIER_CONTROLWINDOW   // ControlWindow class
         #define INCL_TIER_EDIT            // EditWindow class
         #define INCL_TIER_EDITMESSAGES    // EditWindow window messages
#define INCL_TIER_MEMORY                   // LocalMemory class




// Module includes .......................................................
#ifdef _TURBOL
extern "C"        {
#define __MSC
#endif // _TURBOL

#ifdef _GLOCK20L
extern "C"        {
#endif // _GLOCK20L

#ifdef _ZTC201L
extern "Pascal"   {
#endif // _ZTC201L

#include <stdio.h>       // sprintf(), fprintf()

#ifdef _GLOCK20L
                  }
#endif // _GLOCK20L

#ifdef _ZTC201L
                  }
#endif // _ZTC201L

#ifdef _TURBOL
                  }
#endif // _TURBOL


#include <tier1.hxx>     // TIER Library .hxx's
#include "edparser.hxx"  // EditParser class



#undef HIWORD
#undef LOWORD

#define HIWORD(l) (((WORD*)&(l))[1])
#define LOWORD(l) (((WORD*)&(l))[0])


#ifdef _TURBOL
extern "C"        {
#endif // _TURBOL

#ifdef _GLOCK20L
extern "C"        {
#endif // _GLOCK20L

#ifdef _ZTC201L
extern "Pascal"   {
#endif // _ZTC201L

// Pick up a Windows Ansi string routine as an extern
extern LPSTR far pascal AnsiLower( LPSTR );

#ifdef _GLOCK20L
                  }
#endif // _GLOCK20L

#ifdef _ZTC201L
                  }
#endif // _ZTC201L

#ifdef _TURBOL
                  }
#endif // _TURBOL





//////////////////////////////////////////////////////////////////////////////
// EditParser class
//////////////////////////////////////////////////////////////////////////////

EditParser :: FAR EditParser( EditWindow FAR *lpEditCntrl, HANDLE hEditCntrlData, int nLocalOrGlobal )
   {
   // Init IVAR's
   lpEdit      = lpEditCntrl;
   hEditData   = hEditCntrlData;
   nMemType    = nLocalOrGlobal;
   }




EditParser :: FAR ~EditParser( void )
   {
   // do nothing!
   }



int EditParser :: FAR Find( LPSTR lpszSearchStr, int nDirection, BOOL bCaseSensitive )
   {

      LocalMemory    lMem;
      GlobalMemory   gMem;
      LPSTR          lpszSource;
      BOOL           bRet;
      LONG           lLen;
      int            nLen;


   if( !lpEdit )
      return( -1 );


   // Initialize IVARS
   nDir  = nDirection;
   bCase = bCaseSensitive;


   // If the user is supplying the data handle for
   // a local edit.
   if( hEditData )
      {
      if( nMemType == DATA_LOCAL )
         {
         nLen = (int)lpEdit->GetTextLen();
         lMem.SetHandle( hEditData );
         lpszSource = (LPSTR)lMem.Lock();
         if( !lpszSource )
            return( -1 );

         bRet = FindText( lpszSource, lpszSearchStr );
         lMem.Unlock();
         return( bRet );
         }
      return( -1 );
      }


   // If this is a local edit allocate some memory
   // for the buffer.
   if( nMemType == DATA_LOCAL )
      {
      nLen = (int)lpEdit->GetTextLen();
      lMem.Allocate( (int)max( nLen+1, 1 ) );
      lpszSource = (LPSTR)gMem.Lock();
      if( !lpszSource )
         return( -1 );
      
      lpEdit->GetText( lpszSource, (WORD)nLen );

      bRet = FindText( lpszSource, lpszSearchStr );

      gMem.Unlock();
      gMem.Free();
      return( bRet );
      }


   // If this is a global edit allocate some memory
   // for the buffer.
   if( nMemType == DATA_GLOBAL )
      {
      lLen = (LONG)lpEdit->GetTextLen();
      gMem.Allocate( (DWORD)max( lLen+1, 1L ) );
      lpszSource = (LPSTR)gMem.Lock();
      if( !lpszSource )
         return( -1 );
      
      lpEdit->GetText( lpszSource, (WORD)lLen );

      bRet = FindText( lpszSource, lpszSearchStr );

      gMem.Unlock();
      gMem.Free();
      return( bRet );
      }


   return( -1 );
   }





BOOL EditParser :: FAR FindText( LPSTR lpszSource, LPSTR lpszSearchStr )
   {
      
      WORD     wStart, wEnd;
      WORD     wLen;
      WORD     i;


   lpEdit->GetSelect( wStart, wEnd );
   wLen  = lpEdit->GetTextLen();

   // Start with the next char in the selected range.   
   lpszSource += wStart + nDir;


   // Compute how many characters are before or after the 
   // current selection.
   if( nDir == SRCH_BACKWARDS )
      {
      i = wStart;
      }
   else
      {
      i = ((wLen - wStart) + 1) - lstrlen( lpszSearchStr );
      }


   // While there are uncompared substrings.
   while( i > 0 )
      {
      wStart += nDir;
      if( Compare( lpszSource, lpszSearchStr ) )
         {
         wEnd = wStart + lstrlen( lpszSearchStr );
         lpEdit->SetSelect( wStart, wEnd );
         return( TRUE );
         }
      i--;
      lpszSource += nDir;
      }   
   
   return( FALSE );
   }







BOOL EditParser :: FAR Compare( LPSTR lpszSource, LPSTR lpszSearchStr )
   {
   if( bCase )
      {
      while( *lpszSearchStr )
         {
         if( *lpszSearchStr++ != *lpszSource++ )
            return( FALSE );
         }
      }   
   else
      {
      ::AnsiLower( lpszSearchStr );
      while( *lpszSearchStr )
         {
         if( *lpszSearchStr++ != (char)::AnsiLower( (LPSTR)(DWORD)(BYTE)*lpszSource++ ) )
            {
            return( FALSE );
            }
         }
      }

   return( TRUE ); 
   }
