#include <stdio.h>
/* ******************************************************************
 * Function name  : comp_off ( compression off )
 * Description    : This function instructs the printer to turn
 *                  compressed mode off.  This function, works in
 *                  conjunction with "comp_on()". "comp_on" causes
 *                  all characters to be printed in compressed 
 *                  format. This smaller type font allows 132 column
 *                  reports to be printed on 80 column paper.
 * Variables      : string    - The string ( octal 22 ) is sent 
 *                              to the screen to instruct the printer
 *                              to enter compressed print format.
 *                  *prn_file - This variable is the file pointer used
 *                              used to open, define and close the file.
 * Example       : comp_off();
 * Rules         : The function is not passed any variables and is 
 *                 called to negate the "comp_on()" function.
 * Special notes : This function is designed to work only on EPSON FX
 *                 printers. If you are using a printer made by another
 *                 manufacturer, you may have to send a different
 *                 control string to the printer. If this is the case,
 *                 you only must change the fprintf() file print line
 *                 function.
 *                 If you will be using many of these special calls 
 *                 in conjunction with report printing, it may 
 *                 be to your advantage to open the printer as a file
 *                 once at the beginning of your report and close it
 *                 after all your printing is complete. If you take this
 *                 plan of action, comment out the file calls used in
 *                 this function.
 */
comp_off()
{ FILE *prn_file, *fopen();
  prn_file = fopen("prn","w");
  fprintf(prn_file,"%c",'\022');
  fprintf(prn_file,"\nthis should not be compressed\n");
  fclose(prn_file);
}
/* ******************************************************************
 * Function name  : comp_on ( compression on )
 * Description    : This function instructs the printer to turn
 *                  compressed mode on.  This function, works in
 *                  conjunction with "comp_off()". "comp_off" turns
 *                  compression mode off. Compression mode prints 
 *                  using a smaller font allowing 132 column
 *                  reports to be printed on 80 column paper.
 * Variables      : string    - The string ( octal 17 ) is sent 
 *                              to the screen to instruct the printer
 *                              to enter compressed print format.
 *                  *prn_file - This variable is the file pointer used
 *                              used to open, define and close the file.
 * Example       : comp_on();
 * Rules         : The function is not passed any variables and is 
 *                 called to initiate compression mode.
 * Special notes : This function is designed to work only on EPSON FX
 *                 printers. If you are using a printer made by another
 *                 manufacturer, you may have to send a different
 *                 control string to the printer. If this is the case,
 *                 you only must change the fprintf() file print line
 *                 function.
 *                 If you will be using many of these special calls 
 *                 in conjunction with report printing, it may 
 *                 be to your advantage to open the printer as a file
 *                 once at the beginning of your report and close it
 *                 after all your printing is complete. If you take this
 *                 plan of action, comment out the file calls used in
 *                 this function.
 */
comp_on()
{ FILE *prn_file, *fopen();
  prn_file = fopen("prn","w");
  fprintf(prn_file,"%c",'\017');
  fprintf(prn_file,"\nthis should be compressed\n");
  fclose(prn_file);
}
/* ******************************************************************
 * Function name  : ds_off ( turn double strike off )
 * Description    : This function instructs the printer to turn
 *                  double strike mode off.  This function, works in
 *                  conjunction with "ds_on()". "ds_on" causes
 *                  all characters to be printed twice ( overstrike ).
 *                  This double striking produces a much darker copy
 *                  because it actually prints the same character twice
 *                  in the same place. 
 * Variables      : string    - A string of ESCAPE ( octal 33 ) followed
 *                              by the ASCII string "H" is sent 
 *                              to the screen to instruct the printer
 *                              to enter double strike mode.
 *                  *prn_file - This variable is the file pointer used
 *                              used to open, define and close the file.
 * Example       : ds_off();
 * Rules         : The function is not passed any variables and is 
 *                 called to negate the "ds_on()" function.
 * Special notes : This function is designed to work only on EPSON FX
 *                 printers. If you are using a printer made by another
 *                 manufacturer, you may have to send a different
 *                 control string to the printer. If this is the case,
 *                 you only must change the fprintf() file print line
 *                 function.
 *                 If you will be using many of these special calls 
 *                 in conjunction with report printing, it may 
 *                 be to your advantage to open the printer as a file
 *                 once at the beginning of your report and close it
 *                 after all your printing is complete. If you take this
 *                 plan of action, comment out the file calls used in
 *                 this function.
 */
ds_off()
{ FILE *prn_file, *fopen();
  prn_file = fopen("prn","w");
  fprintf(prn_file,"%cH",'\033');
  fprintf(prn_file,"\ndouble strike off\n");
  fclose(prn_file);
}
/* ******************************************************************
 * Function name  : ds_on ( double strike on )
 * Description    : This function instructs the printer to turn
 *                  double strike mode on.  This function, works in
 *                  conjunction with "ds_off()". "ds_off" turns
 *                  double strike mode off. Double strike mode 
 *                  prints each character twice. As a result, the 
 *                  printed output is much darker.
 * Variables      : string    - The string ESCAPE ( octal 33 ) 
 *                              followed by the string "G" is sent 
 *                              to the screen to instruct the printer
 *                              to enter double strike mode.
 *                  *prn_file - This variable is the file pointer used
 *                              used to open, define and close the file.
 * Example       : ds_on();
 * Rules         : The function is not passed any variables and is 
 *                 called to initiate double strike mode.
 * Special notes : This function is designed to work only on EPSON FX
 *                 printers. If you are using a printer made by another
 *                 manufacturer, you may have to send a different
 *                 control string to the printer. If this is the case,
 *                 you only must change the fprintf() file print line
 *                 function.
 *                 If you will be using many of these special calls 
 *                 in conjunction with report printing, it may 
 *                 be to your advantage to open the printer as a file
 *                 once at the beginning of your report and close it
 *                 after all your printing is complete. If you take this
 *                 plan of action, comment out the file calls used in
 *                 this function.
 */
ds_on()
{ FILE *prn_file, *fopen();
  prn_file = fopen("prn","w");
  fprintf(prn_file,"%cG",'\033');
  fprintf(prn_file,"\ndouble strike on\n");
  fclose(prn_file);
}
/* ******************************************************************
 * Function name  : emph_off ( turn emphasis off )
 * Description    : This function instructs the printer to turn
 *                  emphasis mode off.  This function, works in
 *                  conjunction with "emph_on()". "emph_on" causes
 *                  all characters to be printed twice, the second 
 *                  strike being slightly to the right of the first.
 *                  This double striking produces a wider and darker copy
 *                  because it actually prints the same character twice.
 * Variables      : string    - A string of ESCAPE ( octal 33 ) followed
 *                              by the ASCII string "F" is sent 
 *                              to the screen to instruct the printer
 *                              to exit emphasis mode.
 *                  *prn_file - This variable is the file pointer used
 *                              used to open, define and close the file.
 * Example       : emph_off();
 * Rules         : The function is not passed any variables and is 
 *                 called to negate the "emph_on()" function.
 * Special notes : This function is designed to work only on EPSON FX
 *                 printers. If you are using a printer made by another
 *                 manufacturer, you may have to send a different
 *                 control string to the printer. If this is the case,
 *                 you only must change the fprintf() file print line
 *                 function.
 *                 If you will be using many of these special calls 
 *                 in conjunction with report printing, it may 
 *                 be to your advantage to open the printer as a file
 *                 once at the beginning of your report and close it
 *                 after all your printing is complete. If you take this
 *                 plan of action, comment out the file calls used in
 *                 this function.
 */
emph_off()
{ FILE *prn_file, *fopen();
  prn_file = fopen("prn","w");
  fprintf(prn_file,"%cF",'\033');
  fprintf(prn_file,"\nemphasis off\n");
  fclose(prn_file);
}
/* ******************************************************************
 * Function name  : emph_on ( turn emphasis on )
 *
 * Description    : This function instructs the printer to turn
 *                  emphasis mode on.  This function, works in
 *                  conjunction with "emph_off()". "emph_on()" causes
 *                  all characters to be printed twice, the second 
 *                  strike being slightly to the right of the first.
 *                  This double striking produces a wider and darker copy
 *                  because it actually prints the same character twice.
 * Variables      : string    - A string of ESCAPE ( octal 33 ) followed
 *                              by the ASCII string "E" is sent 
 *                              to the screen to instruct the printer
 *                              to enter emphasis mode.
 *                  *prn_file - This variable is the file pointer used
 *                              used to open, define and close the file.
 * Example       : emph_on();
 * Rules         : The function is not passed any variables and is 
 *                 called to turn emphasis mode on.
 * Special notes : This function is designed to work only on EPSON FX
 *                 printers. If you are using a printer made by another
 *                 manufacturer, you may have to send a different
 *                 control string to the printer. If this is the case,
 *                 you only must change the fprintf() file print line
 *                 function.
 *                 If you will be using many of these special calls 
 *                 in conjunction with report printing, it may 
 *                 be to your advantage to open the printer as a file
 *                 once at the beginning of your report and close it
 *                 after all your printing is complete. If you take this
 *                 plan of action, comment out the file calls used in
 *                 this function.
 */
emph_on()
{ FILE *prn_file, *fopen();
  prn_file = fopen("prn","w");
  fprintf(prn_file,"%cE",'\033');
  fprintf(prn_file,"\nenphasis on\n");
  fclose(prn_file);
}
/* ******************************************************************
 * Function name  : exp1_off ( single line expanded mode off )
 * Description    : This function instructs the printer to turn single
 *                  line expanded mode off.  This function, works in
 *                  conjunction with "exp1_on()". "exp1_on" causes
 *                  all characters to be printed in expanded 
 *                  format. This larger type font prints letters twice
 *                  as wide as regular printed characters.
 * Variables      : string    - The string ( octal 24 ) is sent 
 *                              to the screen to instruct the printer
 *                              to exit single line expanded 
 *                              print format.
 *                  *prn_file - This variable is the file pointer used
 *                              used to open, define and close the file.
 * Example       : exp1_off();
 * Rules         : The function is not passed any variables and is 
 *                 called to negate the "exp1_on()" function.
 * Special notes : This function is designed to work only on EPSON FX
 *                 printers. If you are using a printer made by another
 *                 manufacturer, you may have to send a different
 *                 control string to the printer. If this is the case,
 *                 you only must change the fprintf() file print line
 *                 function.
 *                 If you will be using many of these special calls 
 *                 in conjunction with report printing, it may 
 *                 be to your advantage to open the printer as a file
 *                 once at the beginning of your report and close it
 *                 after all your printing is complete. If you take this
 *                 plan of action, comment out the file calls used in
 *                 this function.
 */
exp1_off()
{ FILE *prn_file, *fopen();
  prn_file = fopen("prn","w");
  fprintf(prn_file,"%c",'\024');
  fprintf(prn_file,"\nsingle line exanded on\n");
  fclose(prn_file);
}
/* ******************************************************************
 * Function name  : exp1_on ( single line expanded mode on )
 * Description    : This function instructs the printer to turn single
 *                  line expanded mode on.  This function, works in
 *                  conjunction with "exp1_off()". "exp1_on" causes
 *                  all characters to be printed in expanded 
 *                  format. This larger type font prints letters twice
 *                  as wide as regular printed characters.
 * Variables      : string    - The string ( octal 16 ) is sent 
 *                              to the screen to instruct the printer
 *                              to enter single line expanded 
 *                              print format.
 *                  *prn_file - This variable is the file pointer used
 *                              used to open, define and close the file.
 * Example       : exp1_on();
 * Rules         : The function is not passed any variables and is 
 *                 called to turn single line expanded mode on.
 * Special notes : This function is designed to work only on EPSON FX
 *                 printers. If you are using a printer made by another
 *                 manufacturer, you may have to send a different
 *                 control string to the printer. If this is the case,
 *                 you only must change the fprintf() file print line
 *                 function.
 *                 If you will be using many of these special calls 
 *                 in conjunction with report printing, it may 
 *                 be to your advantage to open the printer as a file
 *                 once at the beginning of your report and close it
 *                 after all your printing is complete. If you take this
 *                 plan of action, comment out the file calls used in
 *                 this function.
 */
exp1_on()
{ FILE *prn_file, *fopen();
  prn_file = fopen("prn","w");
  fprintf(prn_file,"%c",'\016');
  fprintf(prn_file,"\nsingle line expanded off\n");
  fclose(prn_file);
}
/* ******************************************************************
 * Function name  : exp_off (  expanded mode off )
 * Description    : This function instructs the printer to turn 
 *                  expanded mode off.  This function, works in
 *                  conjunction with "exp_on()". "exp_on" causes
 *                  all characters to be printed in expanded 
 *                  format. This larger type font prints letters twice
 *                  as wide as regular printed characters.
 * Variables      : string    - The string ESCAPE ( octal 33 ) followed
 *                              by the ASCII characters "W0" is sent 
 *                              to the screen to instruct the printer
 *                              to exit expanded print format.
 *                  *prn_file - This variable is the file pointer used
 *                              used to open, define and close the file.
 * Example       : exp_off();
 * Rules         : The function is not passed any variables and is 
 *                 called to negate the "exp_on()" function.
 * Special notes : This function is designed to work only on EPSON FX
 *                 printers. If you are using a printer made by another
 *                 manufacturer, you may have to send a different
 *                 control string to the printer. If this is the case,
 *                 you only must change the fprintf() file print line
 *                 function.
 *                 If you will be using many of these special calls 
 *                 in conjunction with report printing, it may 
 *                 be to your advantage to open the printer as a file
 *                 once at the beginning of your report and close it
 *                 after all your printing is complete. If you take this
 *                 plan of action, comment out the file calls used in
 *                 this function.
 */
exp_off()
{ FILE *prn_file, *fopen();
  prn_file = fopen("prn","w");
  fprintf(prn_file,"%cW0",'\033');
  fprintf(prn_file,"\ncontinuous expanded off\n");
  fclose(prn_file);
}
/* ******************************************************************
 * Function name  : exp_on (  expanded mode on )
 * Description    : This function instructs the printer to turn
 *                  expanded mode on.  This function, works in
 *                  conjunction with "exp_off()". "exp_on" causes
 *                  all characters to be printed in expanded 
 *                  format. This larger type font prints letters twice
 *                  as wide as regular printed characters.
 * Variables      : string    - The string ESCAPE ( octal 33 ) 
 *                              followed by the ASCII characters "W1"
 *                              is sent to the screen to instruct the
 *                              printer to enter  expanded print format.
 *                  *prn_file - This variable is the file pointer used
 *                              used to open, define and close the file.
 * Example       : exp_on();
 * Rules         : The function is not passed any variables and is 
 *                 called to turn expanded mode on.
 * Special notes : This function is designed to work only on EPSON FX
 *                 printers. If you are using a printer made by another
 *                 manufacturer, you may have to send a different
 *                 control string to the printer. If this is the case,
 *                 you only must change the fprintf() file print line
 *                 function.
 *                 If you will be using many of these special calls 
 *                 in conjunction with report printing, it may 
 *                 be to your advantage to open the printer as a file
 *                 once at the beginning of your report and close it
 *                 after all your printing is complete. If you take this
 *                 plan of action, comment out the file calls used in
 *                 this function.
 */
exp_on()
{ FILE *prn_file, *fopen();
  prn_file = fopen("prn","w");
  fprintf(prn_file,"%cW1",'\033');
  fprintf(prn_file,"\ncontinuous expanded on\n");
  fclose(prn_file);
}
/* ******************************************************************
 * Function name  : ital_off (  italics mode off )
 * Description    : This function instructs the printer to turn 
 *                  italics mode off.  This function, works in
 *                  conjunction with "ital_on()". "ital_on" causes
 *                  all characters to be printed in italics 
 *                  format. This fancy font type prints letters at a
 *                  slight right angle.
 * Variables      : string    - The string ESCAPE ( octal 33 ) followed
 *                              by the ASCII characters "5" is sent 
 *                              to the screen to instruct the printer
 *                              to exit italics print format.
 *                  *prn_file - This variable is the file pointer used
 *                              used to open, define and close the file.
 * Example       : ital_off();
 * Rules         : The function is not passed any variables and is 
 *                 called to negate the "ital_on()" function.
 * Special notes : This function is designed to work only on EPSON FX
 *                 printers. If you are using a printer made by another
 *                 manufacturer, you may have to send a different
 *                 control string to the printer. If this is the case,
 *                 you only must change the fprintf() file print line
 *                 function.
 *                 If you will be using many of these special calls 
 *                 in conjunction with report printing, it may 
 *                 be to your advantage to open the printer as a file
 *                 once at the beginning of your report and close it
 *                 after all your printing is complete. If you take this
 *                 plan of action, comment out the file calls used in
 *                 this function.
 */
ital_off()
{ FILE *prn_file, *fopen();
  prn_file = fopen("prn","w");
  fprintf(prn_file,"%c5",'\033');
  fprintf(prn_file,"\nitalics off\n");
  fclose(prn_file);
}
/* ******************************************************************
 * Function name  : ital_on (  italics mode on )
 * Description    : This function instructs the printer to turn 
 *                  italics mode off.  This function, works in
 *                  conjunction with "ital_off()". "ital_on" causes
 *                  all characters to be printed in italics 
 *                  format. This fancy font type prints letters at a
 *                  slight right angle.
 * Variables      : string    - The string ESCAPE ( octal 33 ) followed
 *                              by the ASCII characters "4" is sent 
 *                              to the screen to instruct the printer
 *                              to exit italics print format.
 *                  *prn_file - This variable is the file pointer used
 *                              used to open, define and close the file.
 * Example       : ital_on();
 * Rules         : The function is not passed any variables and is 
 *                 called to turn italics mode on.
 * Special notes : This function is designed to work only on EPSON FX
 *                 printers. If you are using a printer made by another
 *                 manufacturer, you may have to send a different
 *                 control string to the printer. If this is the case,
 *                 you only must change the fprintf() file print line
 *                 function.
 *                 If you will be using many of these special calls 
 *                 in conjunction with report printing, it may 
 *                 be to your advantage to open the printer as a file
 *                 once at the beginning of your report and close it
 *                 after all your printing is complete. If you take this
 *                 plan of action, comment out the file calls used in
 *                 this function.
 */
ital_on()
{ FILE *prn_file, *fopen();
  prn_file = fopen("prn","w");
  fprintf(prn_file,"%c4",'\033');
  fprintf(prn_file,"\nitalics on\n");
  fclose(prn_file);
}
/* ******************************************************************
 * Function name  : prop_off (  proportional mode off )
 * Description    : This function instructs the printer to turn 
 *                  proportional mode off.  This function, works in
 *                  conjunction with "prop_on()". "prop_on" causes
 *                  all characters to be printed in proportional 
 *                  format. This print mode effects the space between
 *                  printed characters. With proportional printing,
 *                  the letters are printed close together, based on
 *                  the size of the letters being printed.
 * Variables      : string    - The string ESCAPE ( octal 33 ) followed
 *                              by the ASCII characters "p0" is sent 
 *                              to the screen to instruct the printer
 *                              to exit proportional print format.
 *                  *prn_file - This variable is the file pointer used
 *                              used to open, define and close the file.
 * Example       : prop_off();
 * Rules         : The function is not passed any variables and is 
 *                 called to negate the "prop_on()" function.
 * Special notes : This function is designed to work only on EPSON FX
 *                 printers. If you are using a printer made by another
 *                 manufacturer, you may have to send a different
 *                 control string to the printer. If this is the case,
 *                 you only must change the fprintf() file print line
 *                 function.
 *                 If you will be using many of these special calls 
 *                 in conjunction with report printing, it may 
 *                 be to your advantage to open the printer as a file
 *                 once at the beginning of your report and close it
 *                 after all your printing is complete. If you take this
 *                 plan of action, comment out the file calls used in
 *                 this function.
 */
prop_off()
{ FILE *prn_file, *fopen();
  prn_file = fopen("prn","w");
  fprintf(prn_file,"%cp0",'\033');
  fprintf(prn_file,"\nproportional off\n");
  fclose(prn_file);
}
/* ******************************************************************
 * Function name  : prop_on (  proportional mode on )
 *
 * Description    : This function instructs the printer to turn 
 *                  proportional mode on.  This function, works in
 *                  conjunction with "prop_off()". "prop_on" causes
 *                  all characters to be printed in proportional 
 *                  format. This print mode effects the space between
 *                  printed characters. With proportional printing,
 *                  the letters are printed close together, based on
 *                  the size of the letters being printed.
 *
 * Variables      : string    - The string ESCAPE ( octal 33 ) followed
 *                              by the ASCII characters "p1" is sent 
 *                              to the screen to instruct the printer
 *                              to exit proportional print format.
 *
 *                  *prn_file - This variable is the file pointer used
 *                              used to open, define and close the file.
 *
 * Example       : prop_on();
 *
 * Rules         : The function is not passed any variables and is 
 *                 called to enter proportional mode.
 *
 * Special notes : This function is designed to work only on EPSON FX
 *                 printers. If you are using a printer made by another
 *                 manufacturer, you may have to send a different
 *                 control string to the printer. If this is the case,
 *                 you only must change the fprintf() file print line
 *                 function.
 *
 *                 If you will be using many of these special calls 
 *                 in conjunction with report printing, it may 
 *                 be to your advantage to open the printer as a file
 *                 once at the beginning of your report and close it
 *                 after all your printing is complete. If you take this
 *                 plan of action, comment out the file calls used in
 *                 this function.
 *
 *
 */

prop_on()
{ FILE *prn_file, *fopen();
  prn_file = fopen("prn","w");
  fprintf(prn_file,"%cp1",'\033');
  fprintf(prn_file,"\nproportional on\n");
  fclose(prn_file);
}


/* ******************************************************************
 * Function name  : reset (  reset printer )
 *
 * Description    : This function instructs the printer to cancel 
 *                  all programmed settings and return to the settings
 *                  that where in place when the printer was first 
 *                  turned on. These settings will be a combination
 *                  of the manufacturers defaults and the position of
 *                  the printer dip switches.
 *
 * Variables      : string    - The string ESCAPE ( octal 33 ) followed
 *                              by the ASCII character "@" is sent 
 *                              to the screen to instruct the printer
 *                              to reset settings.
 *
 *                  *prn_file - This variable is the file pointer used
 *                              used to open, define and close the file.
 *
 * Example       : reset();
 *
 * Rules         : The function is not passed any variables and is 
 *                 called to reset the printer.
 *
 * Special notes : This function is designed to work only on EPSON FX
 *                 printers. If you are using a printer made by another
 *                 manufacturer, you may have to send a different
 *                 control string to the printer. If this is the case,
 *                 you only must change the fprintf() file print line
 *                 function.
 *
 *                 If you will be using many of these special calls 
 *                 in conjunction with report printing, it may 
 *                 be to your advantage to open the printer as a file
 *                 once at the beginning of your report and close it
 *                 after all your printing is complete. If you take this
 *                 plan of action, comment out the file calls used in
 *                 this function.
 *
 *
 */

reset()
{ FILE *prn_file, *fopen();
  prn_file = fopen("prn","w");
  fprintf(prn_file,"%c@",'\033');
  fprintf(prn_file,"\nprinter reset\n");
  fclose(prn_file);
}


/* ******************************************************************
 * Function name  : sub_off (  subscript mode off )
 *
 * Description    : This function instructs the printer to turn 
 *                  subscript mode off.  This function, works in
 *                  conjunction with "sub_on()". "sub_on" causes
 *                  all characters to be printed in subscript 
 *                  format. This type font prints letters in a 
 *                  footnote type fassion, being lower and shorter
 *                  than the regular printed characters.
 *
 * Variables      : string    - The string ESCAPE ( octal 33 ) followed
 *                              by the ASCII characters "T" is sent 
 *                              to the screen to instruct the printer
 *                              to exit subscript print format.
 *
 *                  *prn_file - This variable is the file pointer used
 *                              used to open, define and close the file.
 *
 * Example       : sub_off();
 *
 * Rules         : The function is not passed any variables and is 
 *                 called to negate the "sub_on()" function.
 *
 * Special notes : This function is designed to work only on EPSON FX
 *                 printers. If you are using a printer made by another
 *                 manufacturer, you may have to send a different
 *                 control string to the printer. If this is the case,
 *                 you only must change the fprintf() file print line
 *                 function.
 *
 *                 If you will be using many of these special calls 
 *                 in conjunction with report printing, it may 
 *                 be to your advantage to open the printer as a file
 *                 once at the beginning of your report and close it
 *                 after all your printing is complete. If you take this
 *                 plan of action, comment out the file calls used in
 *                 this function.
 *
 *
 */

sub_off()
{ FILE *prn_file, *fopen();
  prn_file = fopen("prn","w");
  fprintf(prn_file,"%cT",'\033');
  fprintf(prn_file,"\nsubscript off\n");
  fclose(prn_file);
}


/* ******************************************************************
 * Function name  : sub_on (  subscript mode on )
 *
 * Description    : This function instructs the printer to turn 
 *                  subscript mode on.  This function, works in
 *                  conjunction with "sub_off()". "sub_on" causes
 *                  all characters to be printed in subscript 
 *                  format. This type font prints letters in a 
 *                  footnote type fassion, being lower and shorter
 *                  than the regular printed characters.
 *
 * Variables      : string    - The string ESCAPE ( octal 33 ) followed
 *                              by the ASCII characters "S1" is sent 
 *                              to the screen to instruct the printer
 *                              to exit subscript print format.
 *
 *                  *prn_file - This variable is the file pointer used
 *                              used to open, define and close the file.
 *
 * Example       : sub_on();
 *
 * Rules         : The function is not passed any variables and is 
 *                 called to enter subscript mode.
 *
 * Special notes : This function is designed to work only on EPSON FX
 *                 printers. If you are using a printer made by another
 *                 manufacturer, you may have to send a different
 *                 control string to the printer. If this is the case,
 *                 you only must change the fprintf() file print line
 *                 function.
 *
 *                 If you will be using many of these special calls 
 *                 in conjunction with report printing, it may 
 *                 be to your advantage to open the printer as a file
 *                 once at the beginning of your report and close it
 *                 after all your printing is complete. If you take this
 *                 plan of action, comment out the file calls used in
 *                 this function.
 *
 *
 */

sub_on()
{ FILE *prn_file, *fopen();
  prn_file = fopen("prn","w");
  fprintf(prn_file,"%cS1",'\033');
  fprintf(prn_file,"\nsubscript on\n");
  fclose(prn_file);
}


/* ******************************************************************
 * Function name  : super_off (  superscript mode off )
 *
 * Description    : This function instructs the printer to turn 
 *                  superscript mode off.  This function, works in
 *                  conjunction with "super_on()". "super_on" causes
 *                  all characters to be printed in superscript 
 *                  format. This type font prints letters in a 
 *                  footnote type fassion, being lower and shorter
 *                  than the regular printed characters.
 *
 * Variables      : string    - The string ESCAPE ( octal 33 ) followed
 *                              by the ASCII characters "T" is sent 
 *                              to the screen to instruct the printer
 *                              to exit superscript print format.
 *
 *                  *prn_file - This variable is the file pointer used
 *                              used to open, define and close the file.
 *
 * Example       : super_off();
 *
 * Rules         : The function is not passed any variables and is 
 *                 called to negate the "super_on()" function.
 *
 * Special notes : This function is designed to work only on EPSON FX
 *                 printers. If you are using a printer made by another
 *                 manufacturer, you may have to send a different
 *                 control string to the printer. If this is the case,
 *                 you only must change the fprintf() file print line
 *                 function.
 *
 *                 If you will be using many of these special calls 
 *                 in conjunction with report printing, it may 
 *                 be to your advantage to open the printer as a file
 *                 once at the beginning of your report and close it
 *                 after all your printing is complete. If you take this
 *                 plan of action, comment out the file calls used in
 *                 this function.
 *
 *
 */

super_off()
{ FILE *prn_file, *fopen();
  prn_file = fopen("prn","w");
  fprintf(prn_file,"%cT",'\033');
  fprintf(prn_file,"\nsuperscript off\n");
  fclose(prn_file);
}


/* ******************************************************************
 * Function name  : super_on (  superscript mode on )
 *
 * Description    : This function instructs the printer to turn 
 *                  superscript mode on.  This function, works in
 *                  conjunction with "super_off()". "super_on" causes
 *                  all characters to be printed in superscript 
 *                  format. This type font prints letters in a 
 *                  footnote type fassion, being higher and shorter
 *                  than the regular printed characters.
 *
 * Variables      : string    - The string ESCAPE ( octal 33 ) followed
 *                              by the ASCII characters "S0" is sent 
 *                              to the screen to instruct the printer
 *                              to exit superscript print format.
 *
 *                  *prn_file - This variable is the file pointer used
 *                              used to open, define and close the file.
 *
 * Example       : super_on();
 *
 * Rules         : The function is not passed any variables and is 
 *                 called to enter superscript mode.
 *
 * Special notes : This function is designed to work only on EPSON FX
 *                 printers. If you are using a printer made by another
 *                 manufacturer, you may have to send a different
 *                 control string to the printer. If this is the case,
 *                 you only must change the fprintf() file print line
 *                 function.
 *
 *                 If you will be using many of these special calls 
 *                 in conjunction with report printing, it may 
 *                 be to your advantage to open the printer as a file
 *                 once at the beginning of your report and close it
 *                 after all your printing is complete. If you take this
 *                 plan of action, comment out the file calls used in
 *                 this function.
 *
 *
 */

super_on()
{ FILE *prn_file, *fopen();
  prn_file = fopen("prn","w");
  fprintf(prn_file,"%cS0",'\033');
  fprintf(prn_file,"\nsuperscript on\n");
  fclose(prn_file);
}


/* ******************************************************************
 * Function name  : under_off (  underline mode off )
 *
 * Description    : This function instructs the printer to turn 
 *                  underline mode off.  This function, works in
 *                  conjunction with "under_on()". "under_on" causes
 *                  all characters to be printed with an underline. 
 *
 * Variables      : string    - The string ESCAPE ( octal 33 ) followed
 *                              by the ASCII characters "-0" is sent 
 *                              to the screen to instruct the printer
 *                              to exit underline print format.
 *
 *                  *prn_file - This variable is the file pointer used
 *                              used to open, define and close the file.
 *
 * Example       : under_off();
 *
 * Rules         : The function is not passed any variables and is 
 *                 called to negate the "under_on()" function.
 *
 * Special notes : This function is designed to work only on EPSON FX
 *                 printers. If you are using a printer made by another
 *                 manufacturer, you may have to send a different
 *                 control string to the printer. If this is the case,
 *                 you only must change the fprintf() file print line
 *                 function.
 *
 *                 If you will be using many of these special calls 
 *                 in conjunction with report printing, it may 
 *                 be to your advantage to open the printer as a file
 *                 once at the beginning of your report and close it
 *                 after all your printing is complete. If you take this
 *                 plan of action, comment out the file calls used in
 *                 this function.
 *
 *
 */

under_off()
{ FILE *prn_file, *fopen();
  prn_file = fopen("prn","w");
  fprintf(prn_file,"%c-0",'\033');
  fprintf(prn_file,"\nunderline off\n");
  fclose(prn_file);
}


/* ******************************************************************
 * Function name  : under_on (  underline mode on )
 *
 * Description    : This function instructs the printer to turn 
 *                  underline mode off.  This function, works in
 *                  conjunction with "under_off()". "under_on" causes
 *                  all characters to be printed with an underline. 
 *
 * Variables      : string    - The string ESCAPE ( octal 33 ) followed
 *                              by the ASCII characters "-1" is sent 
 *                              to the screen to instruct the printer
 *                              to exit underline print format.
 *
 *                  *prn_file - This variable is the file pointer used
 *                              used to open, define and close the file.
 *
 * Example       : under_on();
 *
 * Rules         : The function is not passed any variables and is 
 *                 called to turn on underline mode.
 *
 * Special notes : This function is designed to work only on EPSON FX
 *                 printers. If you are using a printer made by another
 *                 manufacturer, you may have to send a different
 *                 control string to the printer. If this is the case,
 *                 you only must change the fprintf() file print line
 *                 function.
 *
 *                 If you will be using many of these special calls 
 *                 in conjunction with report printing, it may 
 *                 be to your advantage to open the printer as a file
 *                 once at the beginning of your report and close it
 *                 after all your printing is complete. If you take this
 *                 plan of action, comment out the file calls used in
 *                 this function.
 *
 *
 */

under_on()
{ FILE *prn_file, *fopen();
  prn_file = fopen("prn","w");
  fprintf(prn_file,"%c-1",'\033');
  fprintf(prn_file,"\nunderline on\n");
  fclose(prn_file);
}
