/**/
/********************************************************/
/*                                                      */
/*                     FRED-UF.C                        */
/*   Copyright 1990 J. W. Hance - All rights reserved   */
/*  Unbatch a decompressed but still batched news file  */
/*         and build a FidoNet Mail Packet              */
/*                                                      */
/********************************************************/
/**/

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <alloc.h>
#include <string.h>
#include <io.h>
#include <fcntl.h>
#include <time.h>
#include <dos.h>
#include <mem.h>

#include "version.h"
#include "fred.h"

int Debug, quiet;

/* Make the stack larger than the default 4K */
extern unsigned _stklen = 8192;

/**/
/******************** MAIN **********************/
/**/

void main(int argc, char *argv[])
{
 char *cpt, c, *cpt2, *cpt3;
 unsigned long i, j, k, lcount;
 int flag, bigflag;
 int offset;
 int ct, tcount;
 unsigned int count;
 int total_processed = 0;
 int total_copied = 0;
 int total_dumped = 0;
 int total_split = 0;
 int batched = 0;
 char *from_node, *to_node, *uucpname, *curgroup, *logfile;
 char *origin_node, *origin;
 char *domain;
 char *filename, *tmp, *buffer, *who;
 char *config_file, *too_bigs;
 int config_file_flag, single_news;
 int save_bigs, split_count, of_split_count;
 unsigned int headers, too_bigs_size;
 char *f0, *f1, *f2, *f3, *f4;

/* This may not be portable to compilers other than TC++ */
#ifdef __COMPACT__ || __LARGE__
 struct tofidos far *tofidosptr, far *firstfidosptr, far *lastfidosptr;
#endif

#ifdef __SMALL__ || __MEDIUM__
 struct tofidos *tofidosptr, *firstfidosptr, *lastfidosptr;
#endif

#ifdef __TINY__ || __HUGE__
#error Cannot COMPILE for TINY or HUGE
#endif

 struct PktHeader *PktHeader;
 struct MsgHeader *MsgHeader;
 FILE *cffp, *tmpfp, *logfp, *ToFidofp;
 time_t t, start_time;
 char ascii_time[32], tmp_path[128];
 char *hold_date;
 char *start_of_message, *end_of_message;
 struct date d;
 struct time tm;
 struct nodeaddr fnode, tnode;
 unsigned long total_length = 0;
 int first_tofido = 0;
 int reading_tofidofile = 0;
 int how_many_tofidos = 0;
 unsigned long startmem, endmem;

 /**/
 /* Set debug mode */
 /**/

 Debug = 0; /* Off */

 /* Record Starting time */
 start_time = time(NULL);

 /**/
 /* Print startup message */
 /**/

 FredUfStartup();

 /**/
 /* Verify correct command line usage */
 /**/

 if(argc < 3){
 Usage:
  FredUfUsage();
 EndIf

 else{

 /**/
 /* Sift out command line options */
 /**/
  config_file_flag = 0;
  quiet = 0;

  for(ct = 1; ct < argc; ct++){
   if(strcmpi(argv[ct], "-c") == 0){
    if(config_file_flag){
     fprintf(stderr, "ERROR: Redundant '-c' switches\n");
     exit(2);
    EndIf

    else{
     config_file = AllocateString(strlen(argv[++ct]));
     strcpy(config_file, argv[ct]);
     config_file_flag++;
    EndElse
   EndIf

   else if(strcmpi(argv[ct], "-quiet") == 0){
    quiet = 1;
   EndIf

   else{
    goto Usage;
   EndElse
  EndFor
 EndElse

 /* Done processing command line options */

 /**/
 /* Allocate a BUFFER for use in reading the batch file */
 /**/

 if((cpt = malloc(ALLOC_SIZE)) == NULL){
  fprintf(stderr,"ERROR: Memory allocation failed.\n");
  fprintf(stderr,"Probably not enough memory to run.\n");
  exit(3);
 EndIf

 /**/
 /* Now process the configuration file */
 /**/

 /* Try to open config file */
 cffp = OpenFile(config_file, "r");

 buffer = AllocateString(255);
 tofidosptr = NULL;
 flag = 0;

 f0 = AllocateString(255);
 f1 = AllocateString(255);
 f2 = AllocateString(255);
 f3 = AllocateString(255);
 f4 = AllocateString(255);

 for(;;){

  if(reading_tofidofile){
   if(fgets(buffer, 256, ToFidofp) == NULL){
    reading_tofidofile = 0;
    fclose(ToFidofp);
    if(fgets(buffer, 256, cffp) == NULL)
     break;
   EndIf
  EndIf

  /* Not reading ToFidoFile */
  else{
   if(fgets(buffer, 256, cffp) == NULL)
    break;
  EndElse

  /* See if it is a comment */
  if(*buffer == '#') continue;
  else if(*buffer == ';'){
   fprintf(stderr, "!! ERROR - Configuration file has changed - Please read the documentation\n");
   exit(-1);
  EndIf

  /* Otherwise look for interesting config statements */

  sscanf(buffer, "%s %s %s %s", f1, f2, f3, f4);

  if(strcmpi(f1, "mapfile") == 0){
   ToFidofp = fopen(f2, "r");
   if(ToFidofp == NULL)
    fprintf(stderr, "ERROR - Can't open MapFile %s\n", f2);
   else{
    reading_tofidofile = 1;
    fprintf(stderr, "-- Reading Map file %s\n", f2);
   EndElse
  EndIf

  if(strcmpi(f1, "tofido") == 0){
   fprintf(stderr, "## You didn't update your config file, did you? ##\n");
   exit(2);
  }

  if(strcmpi(f1, "from_node") == 0){
   from_node = AllocateString(strlen(f2));
   strcpy(from_node, f2);
   continue;
  EndIf

  if(strcmpi(f1, "logfile") == 0){
   logfile = AllocateString(strlen(f2));
   strcpy(logfile, f2);

   /* Try to open the logfile */
   logfp = OpenFile(logfile, "a");

   t = time(NULL);
   fprintf(logfp, "\n#FRED-UF START @TM=%s+<CF=%s>\n", ctime(&t), config_file);
   continue;
  EndIf

  if(strcmpi(f1, "to_node") == 0){
   to_node = AllocateString(strlen(f2));
   strcpy(to_node, f2);
   continue;
  EndIf

  if(strcmpi(f1, "too_bigs") == 0){
   too_bigs = AllocateString(strlen(f2));
   strcpy(too_bigs, f2);
   continue;
  EndIf

  /* Get which headers to include in FTSC message */
  if(strcmpi(f1, "headers") == 0){
   if(strcmpi(f2, "yes") == 0){
    if(strcmpi(f3, "all") == 0){
     headers = 0x00ff;
    EndIf
    else{
     headers = WhichHeaders(f3);
    EndElse
   EndIf
   else{
    headers = 0;
   EndElse
  EndIf

  if(strcmpi(f1, "origin_node") == 0){
   origin_node = AllocateString(strlen(f2));
   strcpy(origin_node, f2);
   continue;
  EndIf

  if(strcmpi(f1, "domain") == 0){
   domain = AllocateString(strlen(f2));
   strcpy(domain, f2);
   continue;
  EndIf

  if(strcmpi(f1, "origin") == 0){
   tmp = memchr(buffer+7, '\n', 80);
   origin = AllocateString(50);
   strncpy(origin, buffer + 7,
	  (tmp - (buffer + 7) > 50) ? 50 : (tmp - (buffer + 7)));
   *(origin + ((tmp - (buffer + 7) > 50) ? 50 : (tmp - (buffer + 7))) ) = NULL;
  EndIf

  if(strcmpi(f1, "bigs") == 0){
   if(strcmpi(f2, "save") == 0){
    save_bigs = 1;
    too_bigs_size = (unsigned)atoi(f3);
    if(too_bigs_size > ALLOC_SIZE){
     ERROR_TOO_BIG:
     fprintf(stderr, "ERROR: BIGS size of %d is > %u\n", too_bigs_size, ALLOC_SIZE);
     exit(2);
    EndIf
    else if(too_bigs_size < 4096){
     ERROR_TOO_SMALL:
     fprintf(stderr, "ERROR: BIGS size of %d is < 4096\n", too_bigs_size);
     exit(2);
    EndIf
   EndIf
   else if(strcmpi(f2, "dump") == 0){
    save_bigs = 0;
    too_bigs_size = (unsigned)atoi(f3);
    if(too_bigs_size > ALLOC_SIZE){
     goto ERROR_TOO_BIG;
    EndIf
    else if(too_bigs_size < 4096){
     goto ERROR_TOO_SMALL;
    EndIf
   EndIf
   else if(strcmpi(f2, "split") == 0){
    save_bigs = 2;
    too_bigs_size = (unsigned)atoi(f3);
    if(too_bigs_size > ALLOC_SIZE){
     goto ERROR_TOO_BIG;
    EndIf
    else if(too_bigs_size < 4096){
     goto ERROR_TOO_SMALL;
    EndIf
   EndIf
   else if(strcmpi(f2, "copy") == 0){
    save_bigs = 3;
    too_bigs_size = ALLOC_SIZE;
   EndIf
   else{
    fprintf(stderr, "ERROR: Illegal option for BIGS in config file: %s.\n", f2);
    exit(5);
   EndElse
  EndIf

  if(strcmpi(f1, "uucpname") == 0){
   uucpname = AllocateString(strlen(f2));
   strcpy(uucpname, f2);
   continue;
  EndIf

  if(strcmpi(f1, "who") == 0){
   who = AllocateString(strlen(f2));
   strcpy(who, f2);
   continue;
  EndIf

  if((strcmpi(f1, "map") == 0) && ((*f2 == '+') || (*f2 == '>'))){
   lastfidosptr = tofidosptr;
   how_many_tofidos++;

#ifdef __COMPACT__ || __LARGE__
   tofidosptr = farmalloc(sizeof(struct tofidos));
   if(first_tofido == 0){
    startmem = farcoreleft();
    first_tofido++;
    fprintf(stderr, "-- You have %lu percent of your memory left (%luK of 640K) before Maps.\n",
                    ((startmem*100L)/655360L), startmem/1024);
   }
#endif
#ifdef __SMALL__ || MEDIUM__
   tofidosptr = malloc(sizeof(struct tofidos));
   if(first_tofido == 0){
    startmem = (unsigned long)coreleft();
    first_tofido++;
    fprintf(stderr, "-- You have %lu percent of your memory left (%luK of 64K) before Maps.\n",
                    ((startmem*100L)/65536L), startmem/1024);
   }
#endif



   if(tofidosptr == NULL){
    fprintf(stderr, "ERROR: Can't allocate memory for Maps.\n");
    exit(6);

   EndIf

   if(flag == 0){
    flag++;
    firstfidosptr = tofidosptr;
   EndIf

   lastfidosptr->next = tofidosptr;
   tofidosptr->next = NULL;

#ifdef __COMPACT__ || __LARGE__
   tofidosptr->uucpname = FarAllocateString(strlen(f3));
   strcpy(tofidosptr->uucpname, f3);
   tofidosptr->fidoname = FarAllocateString(strlen(f4));
   strcpy(tofidosptr->fidoname, f4);
#endif
#ifdef __SMALL__ || MEDIUM__
   tofidosptr->uucpname = AllocateString(strlen(f3));
   strcpy(tofidosptr->uucpname, f3);
   tofidosptr->fidoname = AllocateString(strlen(f4));
   strcpy(tofidosptr->fidoname, f4);
#endif


   tofidosptr->count = 0;
   continue;
  EndIf

 EndWhile

 fclose(cffp);

 /* Done Reading Configuration file */

#ifdef __SMALL__ || __MEDIUM__
 endmem = (unsigned long)coreleft();
 fprintf(stderr, "-- You have %lu percent of your memory left (%luK of 64K) after Maps.\n",
                    ((endmem*100L)/65536L), endmem/1024);
#endif
#ifdef __COMPACT__ || __LARGE__
 endmem = farcoreleft();
 fprintf(stderr, "-- You have %lu percent of your memory left (%luK of 640K) after Maps.\n",
                    ((endmem*100L)/655360L), endmem/1024);
#endif

 fprintf(stderr, "-- Found %d Map Statements which used %luK RAM\n",
                 how_many_tofidos, (startmem-endmem)/1024);

 /**/
 /* Need to check here that all config items are set */
 /**/

 /* Output fidos if in debug mode */
 if(Debug)
  DumpFidos(firstfidosptr);

 /* Find out what time it is */
 t = time(NULL);
 strcpy(ascii_time, ctime(&t));
 sscanf(ascii_time, "%s %s %s %s %s", f0, f1, f2, f3, f4);
 strcpy(ascii_time, f2);
 strcat(ascii_time, " ");
 strcat(ascii_time, f1);
 strcat(ascii_time, " ");
 strcat(ascii_time, &f4[2]);
 strcat(ascii_time, " ");
 strcat(ascii_time, f3);
 /* ascii_time should now be of form dd Mmm yy hh:mm:ss */

 /* Also get the numerical time from DOS */
 getdate(&d);
 gettime(&tm);

 /* Just so I can sleep nights ! */
 if(d.da_year < 2000)
  d.da_year -=1900;
 else
  d.da_year -=2000;

 /* Ok, now we have all the time we need */

 /* Initialize and write the Packet Header to the packet_file */

 /* Make space for the Headers */
 PktHeader = (struct PktHeader *)calloc(sizeof(struct PktHeader), 1);
 if(PktHeader == NULL){
  fprintf(stderr, "ERROR: calloc() failed for PktHeader\n");
  fprintf(logfp, "!calloc()/PktHeader\n");
  exit(7);
 }

 MsgHeader = (struct MsgHeader *)calloc(sizeof(struct MsgHeader), 1);
 if(MsgHeader == NULL){
  fprintf(stderr, "ERROR: calloc() failed for MsgHeader\n");
  fprintf(logfp, "!calloc()/MsgHeader\n");
  exit(8);
 }

 /* Init various fields in PktHeader amd MsgHeader */
 PktHeader->Year = d.da_year;
 PktHeader->Month = --d.da_mon;
 PktHeader->Day = d.da_day;
 PktHeader->Hour = tm.ti_hour;
 PktHeader->Minute = tm.ti_min;
 PktHeader->Second = tm.ti_sec;
 PktHeader->Baud = 0;
 PktHeader->Version = 2;
 PktHeader->ProdSer = 0x0000;
 for(i=0; i< 8; i++) PktHeader->Password[i] = 0x00;
 for(i=0; i<20; i++) PktHeader->Fill[i] = 0x00;

 MsgHeader->Attribute = 0x0000;
 MsgHeader->Cost = 0;
 MsgHeader->Version = 2;

 /* Now get the node numbers into the Packet and message headers */
  if(VerifyNode(from_node, &fnode) == -1){
   fprintf(stderr, "ERROR: Node address %s for FROM_NODE is invalid\n", from_node);
   fprintf(logfp, "!FROM_NODE/%s\n", from_node);
   exit(9);
  }

  PktHeader->OriginNode = fnode.node;
  PktHeader->OriginNet  = fnode.net;
  PktHeader->OriginZone = fnode.zone;
  MsgHeader->OriginNode = fnode.node;
  MsgHeader->OriginNet  = fnode.net;

  if(VerifyNode(to_node, &tnode) == -1){
   fprintf(stderr, "ERROR: Node address %s for TO_NODE is invalid\n", to_node);
   fprintf(logfp, "!TO_NODE/%s\n", to_node);
   exit(10);
  }

  PktHeader->DestNode = tnode.node;
  PktHeader->DestNet  = tnode.net;
  PktHeader->DestZone = tnode.zone;
  MsgHeader->DestNode = tnode.node;
  MsgHeader->DestNet  = tnode.net;

 /* Put stdout in BINARY mode */
 setmode(fileno(stdout), O_BINARY);

 /* Put stdin in TEXT mode */
 setmode(fileno(stdin), O_BINARY);

 /* Make sure the stdin and stdout are at beginning of file */
 fseek(stdin, 0L, SEEK_SET);
 fseek(stdout, 0L, SEEK_SET);

 /* Write out the Packet Header */
 fwrite(PktHeader, sizeof(struct PktHeader), 1, stdout);
 fflush(stdout);

 free(PktHeader);


 /* Header is written - now go on to messages */

 /**/
 /**********************************************/
 /*      Now go through all the messages       */
 /**********************************************/
 /**/

 /* No apologies for the use of gotos in this section */
 /* sometimes it's the best and most convenient method */

 while(1){

   /* Fill the buffer from the batch file */
   Search:
    if(fgets(buffer, 255, stdin) == NULL){
     fprintf(stderr, "\nFred-Uf done processing batch file.\n");
     break;
    EndIf

   /* Look for the '#! rnews <count>' line */
   if(strncmpi(buffer, "#! rnews", 8) == 0){
    batched = 1;
    sscanf(buffer, "%s %s %s", f1, f2, f3);
    if(strcmp(f1, "#!") == 0 && strcmp(f2, "rnews") == 0){
     i = (unsigned long)atol(f3);
     total_length += i;
     total_processed++;
    EndIf
    else{
     goto Search;
    EndElse
   EndIf

   else{

    /* Found no #! rnews line */
    /* Assume that it is a single news message without the #! rnews line */
    /* unless batched is set */
    if(batched) goto Search;

    /* Rewind it */
    fseek(stdin, 0L, SEEK_SET);

    /* Figure out how long it is */
    i = filelength(fileno(stdin));

   EndElse

   /* Now read and process the buffer */
   if(i > too_bigs_size){

    /* To big for buffer - do what save_bigs says */


    /* Put in a file if save_bigs = 1 */
    if(save_bigs == 1){
     strcpy(tmp_path, too_bigs);
     strcat(tmp_path, UniqueName("BIG"));
     if((tmpfp = OpenFile(tmp_path, "w")) != NULL){
      lcount = i;
      while(lcount > ALLOC_SIZE){
       fread(cpt, sizeof(char), ALLOC_SIZE, stdin);
       fwrite(cpt, sizeof(char), ALLOC_SIZE, tmpfp);
       lcount -= ALLOC_SIZE;
      EndWhile
      if(lcount > 0){
       fread(cpt, sizeof(char), (int)lcount, stdin);
       fwrite(cpt, sizeof(char), (int)lcount, tmpfp);
      EndIf
      fclose(tmpfp);
     EndIf
     else{
      fprintf(stderr, "ERROR: Can't open file %s for TOOBIGS.\n", tmp_path);
      fprintf(logfp, "!fopen()/%s\n", tmp_path);
      exit(13);
     EndElse
    EndIf

    /* Split the messages if save_bigs = 2 */
    else if(save_bigs == 2){

     total_split++;
     of_split_count = i / too_bigs_size;
     if(i % too_bigs_size) of_split_count++;

     for(split_count = 1; split_count <= of_split_count; split_count++){
      if(split_count != of_split_count)
       count = fread(cpt, sizeof(char), too_bigs_size, stdin);
      else
       count = fread(cpt, sizeof(char), i % too_bigs_size, stdin);


      /* Make sure we read as many bytes as we asked for */
      if(count != (split_count != of_split_count ? too_bigs_size :
					      i % too_bigs_size)){

       fprintf(stderr, "ERROR: reading stdin - message CORRUPT?\n");
       fprintf(logfp, "!CORRUPT?\n");

       /* Try and recover */
       goto Search;
      EndIf

      if(DoMessageSplit(cpt, count, logfp, headers, &total_processed,
	       &total_copied, &total_dumped, ascii_time,
	       origin, origin_node, domain, &fnode, &tnode,
	       firstfidosptr, MsgHeader, split_count, of_split_count)
	       == -1) goto Search;


     EndFor

    EndIf

    /* Copy the messages whole if save_bigs = 3 */
    else if(save_bigs == 3){

    count = fread(cpt, sizeof(char), ALLOC_SIZE, stdin);

    /* Make sure we read as many bytes as we asked for */
    if(count != ALLOC_SIZE){
     fprintf(stderr, "ERROR: reading stdin - message CORRUPT?\n");

     fprintf(logfp, "!CORRUPT?\n");
     goto Search;
    EndIf


    if(DoMessage(cpt, count, logfp, headers, &total_processed,
	     &total_copied, &total_dumped, ascii_time,
	     origin, origin_node, domain, &fnode, &tnode,
	     firstfidosptr, MsgHeader, i) == -1) goto Search;

   EndIf

   else{
    /* If no save, then seek to next record */
    fseek(stdin, i, SEEK_CUR);
   EndElse

   goto Search;

  EndIf

  else{
   /* Will fit in buffer - do it all at once */
   count = fread(cpt, sizeof(char), (unsigned int)i, stdin);

   /* Make sure we read as many bytes as we asked for */
   if(count != i){
    fprintf(stderr, "ERROR: reading stdin - message CORRUPT?\n");
    fprintf(logfp, "!CORRUPT?\n");
    goto Search;
   EndIf

   Process_Message:

   if(DoMessage(cpt, count, logfp, headers, &total_processed,
	      &total_copied, &total_dumped, ascii_time,
	      origin, origin_node, domain, &fnode, &tnode,
	      firstfidosptr, MsgHeader, 0L) == -1) goto Search;

  EndElse
 EndWhile

  /* Output final 2 bytes of NULL for end of packet */
  fputc(0, stdout);
  fputc(0, stdout);

  /* Close 'em out */
  fclose(stdout);
  fclose(stdin);

  fprintf(stderr, "\n**** Message Processing Summary ****\n");
  fprintf(stderr, "Total Messages Processed: %5d  ", total_processed);
  fprintf(stderr, "Total Messages Split    : %5d\n", total_split);
  fprintf(stderr, "Total Messages Copied   : %5d  ", total_copied);
  fprintf(stderr, "Total Messages Dumped   : %5d\n", total_dumped);
  fprintf(stderr, "Average Message Length  : %5lu\n",
	   (total_processed != 0) ? (total_length/total_processed) : 0);
  fprintf(stderr, "Time to process bundle was %d seconds. ",
		  time(NULL)-start_time);
  if(time(NULL)-start_time > 0){
   fprintf(stderr, "Speed was %d messages/minute.\n",
	   (total_processed/(1 + time(NULL)-start_time))*60);
  EndIf

  fprintf(logfp, "+TP<%5d> ", total_processed);
  fprintf(logfp, "TS<%5d> ", total_split);
  fprintf(logfp, "TC<%5d> ", total_copied);
  fprintf(logfp, "TD<%5d> ", total_dumped);
  fprintf(logfp, "AL<%5lu>\n",
	  (total_processed  != 0) ? (total_length/total_processed) : 0);
  fprintf(logfp, "+TM<%5d> ",
		  time(NULL)-start_time);

  if(time(NULL)-start_time > 0){
   fprintf(logfp, "SP<%5d>\n",
	   (total_processed/(1 + time(NULL)-start_time))*60);
  EndIf

  if(total_copied == 0){

   fprintf(stderr, "NOTE: Empty Packet generated.\n\n");
   fprintf(logfp, "+EMPTY\n");
  EndIf
  else
   fprintf(stderr, "\n");

  if(total_copied != 0)
   DumpFidosToLog(firstfidosptr, logfp);

  t = time(NULL);
  fprintf(logfp, "$FRED-UF STOP @TM=%s", ctime(&t));
  fclose(logfp);

  if(total_copied == 0)
   exit(0);
  else
   exit(1);

EndMain

/*****************************************************/

