
#define ALLOC_SIZE 32768
#define FUALLOC_SIZE 20000

#define SPACE  0x20
#define COMMA  0x2c
#define PERIOD 0x2e
#define LPAREN 0x28
#define RPAREN 0x29
#define COLON  0x3a
#define SLASH  0x2f
#define CR     0x0d
#define LF     0x0a
#define SOFTCR 0x8d
#define QUOTE  0x22

#define EndWhile }
#define EndIf }
#define EndFor }
#define EndElse }
#define EndMain }

/* Prototypes */
struct nodeaddr { int zone, net, node, point; };
int VerifyNode(char *address, struct nodeaddr *node);
char *AllocateString(size_t);
char far *FarAllocateString(size_t);
void DeallocateString(char *);

struct tofidos {
#ifdef __SMALL__ || __MEDIUM__
		 char *uucpname;
		 char *fidoname;
         unsigned int count;
         int dist;
		 struct tofidos *next;
#endif
#ifdef __COMPACT__ || __LARGE__
		 char far *uucpname;
		 char far *fidoname;
		 unsigned int count;
         int dist;
		 struct tofidos far *next;
#endif
		};

struct mail {
		 char *uucpname;
		 char *fidoname;
		 char *system;
		 struct mail *next;
		};

struct tousers {
		 char *uucpname;
		 char *fidoname;
		 char *system;
		 unsigned int count;
		 struct tousers *next;
		};

struct mgroups {
        char firstgroup[128];
        int howmany;
        char g[10][128];
       };

struct Sites {
        int howmany;
        char Site[50][32];
       };

struct Dist {
        int howmany;
        char Dist[10][32];
       };

#ifdef __SMALL__ || __MEDIUM__
void DumpFidos(struct tofidos *);
void DumpFidosToLog(struct tofidos *, FILE *);
#endif
#ifdef __COMPACT__ || __LARGE__
void DumpFidos(struct tofidos far *);
void DumpFidosToLog(struct tofidos far *, FILE *);
#endif

int GetAddressFromOrigin(char *, int, int *, int *, int *, int *);

void FilterSoftCR(char *, unsigned int);
void FilterCTRLA(char *, unsigned int);
unsigned int WhichHeaders(char *);
char *UniqueName(char *);
unsigned int GetSequenceNumber(void);
void WriteCmdXQTFiles(char *, char *, char *, char *, char *);

FILE * WriteUucpMessageHeader(char *, char *, char *, char *, char *,
			      char *, char *, char *, char *,
			      char *, int,
			      int, int, int, char *, char *,
			      int, struct nodeaddr*);

struct tousers *FindUucpUserName(struct tousers *, char *);

char * pascal FindMessageStart(char *, int);
char * pascal FindHeader(char *, int, char *);
int ParsePath(char *, char *, char *);

struct Str { char *cpt; int cnt; };

struct mgroups * pascal FindNewsGroup(char *, int, struct Str *);
char * pascal FindSubject(char *, int, struct Str *);
char * pascal FindOrganization(char *, int, struct Str *);
char * pascal FindReplyTo(char *, int, struct Str *);
char * pascal FindWholeFrom(char *, int, struct Str *);
char * pascal FindFrom(char *, int, struct Str *);
char * pascal FindWholeDate(char *, int, struct Str *);
char * pascal FindDate(char *, int, struct Str *);
char * pascal FindMessageID(char *, int, struct Str *);
char * pascal FindPath(char *, int, struct Str *);
char * pascal FindArea(char *, int, struct Str *);
char * pascal FindToUucp(char *, int, struct Str *);
char * pascal FindToUser(char *, int, struct Str *);

void pascal WriteUseNetHeaders(unsigned int, char *, int, struct Str *, int,
                               struct mgroups *);
void pascal WriteUseNetHeadersMail(unsigned int, char *, int, struct Str *);

int resolve(char *, struct nodeaddr *, char *, char *);
void FredFuStartup(void);
void FredFuUsage(void);
void FredUfStartup(void);
void FredUfUsage(void);
void DoLineBreak(int, char *, int);
FILE *OpenFile(char *, char *);

/* Structure to hold an FTSC mail packet header */
/*   This is probably non-portable in some way */
struct PktHeader {
		    int OriginNode;   /* Node # of origin */
		    int DestNode;     /* Node # of destination */
		    int Year;         /* Year */
		    int Month;        /* Month */
		    int Day;          /* Day */
		    int Hour;         /* Hour */
		    int Minute;       /* Minute */
		    int Second;       /* Second */
		    int Baud;         /* Not used - set to 0x0000*/
		    int Version;      /* Always 0x0002 */
		    int OriginNet;    /* Net # of origin */
		    int DestNet;      /* Net # of destination */
		    int ProdSer;      /* Not used - set to 0x0000 */
		    char Password[8]; /* Password - set to 0s */
		    int  OriginZone;  /* Zone of origin */
		    int  DestZone;    /* Zone of destination */
		    char Fill[20];    /* Zero Fill */
		    };

/* Structure to hold the beginning of a mail message within a packet */
struct MsgHeader {
		    int  Version;
		    int  OriginNode;
		    int  DestNode;
		    int  OriginNet;
		    int  DestNet;
		    int  Attribute;
		    int  Cost;
		  };


int pascal DoMessage(char *,
          unsigned int,
	      FILE *,
	      unsigned int,
	      int *, int *, int *,
	      char *,
	      char *,
	      char *,
	      char *,
	      struct nodeaddr *,
	      struct nodeaddr *,
#ifdef __SMALL__ || __MEDIUM__
          struct tofidos *,
#endif
#ifdef __COMPACT__ || __LARGE__
          struct tofidos far *,
#endif
	      struct MsgHeader *,
	      unsigned long);

int pascal DoMessageSplit(char *,
          unsigned int,
	      FILE *,
	      unsigned int,
	      int *, int *, int *,
	      char *,
	      char *,
	      char *,
	      char *,
	      struct nodeaddr *,
	      struct nodeaddr *,
#ifdef __SMALL__ || __MEDIUM__
          struct tofidos *,
#endif
#ifdef __COMPACT__ || __LARGE__
          struct tofidos far *,
#endif
	      struct MsgHeader *,
	      unsigned int split_count,
	      unsigned int of_split_count);

int DoMailMessage(char *, int, FILE *,
	     char *,
	     char *,
	     struct nodeaddr *,
	     struct nodeaddr *,
         struct nodeaddr *,
	     struct mail *,
	     struct MsgHeader *,
	     unsigned long, char *, char *);

/* Defines for Attribute Word */
#define MsgPrivate  0x0001
#define MsgCrash    0x0002
#define MsgRecd     0x0004
#define MsgSent     0x0008
#define MsgFa       0x0010
#define MsgInTrans  0x0020
#define MsgOrphan   0x0040
#define MsgKill     0x0080
#define MsgLocal    0x0100
#define MsgHold     0x0200
#define MsgNA       0x0400
#define MsgFreq     0x0800
#define MsgRRQ      0x1000
#define MsgIRR      0x2000
#define MsgAudit    0x4000
#define MsgFUreq    0x8000
