/**/
/***************************************************/
/*                                                 */
/*                     FREDTOSS.C                  */
/*                                                 */
/***************************************************/
/**/

#include "version.h"

#include <stdio.h>
#include <dir.h>
#include <dos.h>
#include <process.h>
#include <time.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>

char *munge163(char *, char *, int);
void rename_spools(char *, char *, FILE *, int);

void main(int argc, char *argv[])
{
 struct ffblk ffblk;
 int done, rnews_rmail, result;
 FILE *in, *cffp, *log, *check;
 unsigned char *inbound_path, *spool_path, *temp_path, *buffer;
 char *logfile;
 char pathname[256], sub_path[256], temp_pathname[256];
 char inbname[256], munged_name[32];
 char tmpname[256], filename[256], dfile[256];
 char temp_pathname_z[256];
 char maildest[512];
 int test_flag, looking;
 int quiet = 0;
 int config_file_flag, host_name_flag, delete_flag, ct;
 int delete_files;
 char *config_file, *host_name, *delete, u_host_name[64];
 char *uucpname;
 char *f1, *f2, *f3;
 char FredCmd[32];
 int nbits;
 int newsonly, mailonly, NoSaveTemp = 1, UseLarge = 0;
 time_t t;

 fprintf(stderr,"* FredToss Version %s ", FTVERSION);
 fprintf(stderr,"Built %s at time %s *\n", __DATE__, __TIME__);
 fprintf(stderr,"* Copyright 1990, 1991 J. W. Hance - ");
 fprintf(stderr,"All Rights Reserved *\n");

 if(argc < 7){
  fprintf(stderr, "\nUsage:   FREDTOSS -c <config> -h <host> -d <DELETE|NODELETE> [-<code>]\n");
  fprintf(stderr, "Example: FREDTOSS -c config.fil -h mgcind -d DELETE\n");
  fprintf(stderr, "where:    config.fil is the configuration file\n");
  fprintf(stderr, "          mgcind is the uucp host to process\n");
  fprintf(stderr, "          DELETE will delete processed messages\n");
  fprintf(stderr, "          NODELETE will save processed messages\n");
  fprintf(stderr, "          [-<code>] is optional command to select processing\n");
  fprintf(stderr, "          of only news or mail.\n");
  fprintf(stderr, "          -NewsOnly or -MailOnly are acceptable values.\n");
  exit(1);
 }

 /**/
 /* Sift out command line options */
 /**/
  delete_flag = config_file_flag = host_name_flag = 0;
  test_flag = newsonly = mailonly = 0;

  /* Look for the config file name */
  for(ct = 1; ct < argc; ct++){
   if(strcmpi(argv[ct], "-c") == 0){
    if(config_file_flag){
     fprintf(stderr, "ERROR: Redundant '-c' switches\n");
     exit(1);
    }

    else{
     config_file = malloc(strlen(argv[++ct])+1);
     strcpy(config_file, argv[ct]);
     config_file_flag++;
    }
   }

   /* Look for -SaveTemp */
   else if(strcmp(argv[ct], "-SaveTemp") == 0){
    NoSaveTemp = 0;
    fprintf(stderr, "-SaveTemp specified\n");
   }

   /* Look for the host name */
   else if(strcmpi(argv[ct], "-h") == 0){
    if(host_name_flag){
     fprintf(stderr, "ERROR: Redundant '-h' switches\n");
     exit(1);
    }

    else{
     host_name = malloc(strlen(argv[++ct])+1);
     strcpy(host_name, argv[ct]);
     host_name_flag++;
    }
   }

   /* Look for the -d flag */
   else if(strcmpi(argv[ct], "-d") == 0){
    if(delete_flag){
     fprintf(stderr, "ERROR: Redundant '-d' switches\n");
     exit(1);
    }
    else{
     delete = malloc(strlen(argv[++ct])+1);
     strcpy(delete, argv[ct]);
     delete_flag++;
     if(strcmpi(delete, "DELETE") == 0)
      delete_files = 1;
     else
      delete_files = 0;
    }
   }

   /* Look for the -test flag */
   else if(strcmpi(argv[ct], "-test") == 0){
    test_flag = 1;
    fprintf(stderr, "Running in TEST mode - spawn() will NOT execute\n");
   }

   /* Look for -newsonly or -mailonly flag */
   else if(strcmpi(argv[ct], "-newsonly") == 0){
    newsonly = 1;
    fprintf(stderr, "Note: NewsOnly Mode specified.\n");
   }

   else if(strcmpi(argv[ct], "-mailonly") == 0){
    mailonly = 1;
    fprintf(stderr, "Note: MailOnly Mode specified.\n");
   }

  } /* End For */


 if(!delete_flag){
  fprintf(stderr, "ERROR: '-d' flag not specified.\n");
  exit(1);
 }
 if(!config_file_flag){
  fprintf(stderr, "ERROR: '-c' flag not specified.\n");
  exit(1);
 }
 if(!host_name_flag){
  fprintf(stderr, "ERROR: '-h' flag not specified.\n");
  exit(1);
 }

 /* Done processing command line options */

 /**/
 /* Now process the configuration file */
 /**/

 /* Try to open config file */
 cffp = fopen(config_file, "r");
 if(cffp == NULL){
  fprintf(stderr, "ERROR: Can't open config file: %s\n", config_file);
  exit(1);
 }

 buffer = malloc(256);
 f1 = malloc(128);
 f2 = malloc(128);
 f3 = malloc(512);

 if((buffer == NULL) || (f1 == NULL) || (f2 == NULL)){
  fprintf(stderr, "ERROR: call to malloc() failed.\n");
  exit(1);
 }



 while(fgets(buffer, 256, cffp) != NULL){

  /* See if it is a comment */
  if(*buffer == '#') continue;

  /* Otherwise look for interesting config statements */

  sscanf(buffer, "%s %s", f1, f2);

  /* Find where the temp files go */
  if(strcmpi(f1, "tempdir") == 0){
   temp_path = malloc(strlen(f2)+1);
   strcpy(temp_path, f2);
   continue;
  }

  /* See if UseLarge is specified */
  if(strcmpi(f1, "uselarge") == 0){
   UseLarge = 1;
   continue;
  }

  /* Where to build the packet */
  if(strcmpi(f1, "inbound") == 0){
   inbound_path = malloc(strlen(f2)+1);
   strcpy(inbound_path, f2);
   continue;
  }

  /* What is my Uucpname */
  if(strcmpi(f1, "uucpname") == 0){
   uucpname = malloc(strlen(f2)+1);
   strcpy(uucpname, f2);
   continue;
  }


  /* See if Fred-Uf should be QUIET */
  if(strcmpi(f1, "quiet") == 0){
   if(strcmpi(f2, "yes") == 0)
    quiet = 1;
   else
    quiet = 0;
   continue;
  }
  /* Where the spool files are */
  if(strcmpi(f1, "spool") == 0){
   spool_path = malloc(strlen(f2)+1);
   strcpy(spool_path, f2);
   continue;
  }

  /* Get the logfile */
  if(strcmpi(f1, "logfile") == 0){
   logfile = malloc(strlen(f2)+1);
   strcpy(logfile, f2);

   /* Try to open the logfile */
   log = fopen(logfile, "a");
   if(log == NULL){
    fprintf(stderr, "ERROR: Can't open LOGFILE %s.\n", logfile);
    exit(4);
   }
   else{
    t = time(NULL);
    fprintf(log, "\n#FREDTOSS START @TM=%s+<HS=%s><CF=%s><MO=%d><NO=%d><DF=%d>\n",
							       ctime(&t),
							       host_name,
                                                               config_file,
							       mailonly,
							       newsonly,
							       delete_files);
   }

   continue;
  }
 }

 fclose(cffp);

 /***********************************/
 /* Done Reading Configuration file */
 /***********************************/

 fprintf(log, "+USELARGE %d\n", UseLarge);

 strcpy(pathname, spool_path);
 strcat(pathname, host_name);
 strcat(pathname, "\\");

 strcpy(sub_path, pathname);

 strcat(pathname, "*.x");

 /* pathname has a filename like d:\dir\dir1\host\*.x */
 done = findfirst(pathname, &ffblk, 0);

 while(done == 0){
  rnews_rmail = 0;

  /* Look at the X file that was found */
  /* We need the values from the 'U', 'F', and 'C' lines */
  strcpy(filename, sub_path);
  strcat(filename, ffblk.ff_name);
  in = fopen(filename, "r");

  Look_for_U:
  if(fgets(buffer, 256, in) == NULL){
   fprintf(stderr, "Error reading file %s.\n", filename);
   fprintf(log, "!FREDTOSS/fgets()/%s\n", filename);
   rename_spools(filename, NULL, log, 1);
   done = findnext(&ffblk);
   continue;
  }
  else{
   if(*buffer != 'U') goto Look_for_U;
   sscanf(buffer, "%s %s %s", f1, f2, f3);
   strcpy(u_host_name, f3);
  }

  Look_for_F:
  if(fgets(buffer, 256, in) == NULL){
   fprintf(stderr, "Error reading file %s.\n", filename);
   fprintf(log, "!FREDTOSS/fgets()/%s\n", filename);
   rename_spools(filename, NULL, log, 1);
   done = findnext(&ffblk);
   continue;
  }
  else{
   if(*buffer != 'F') goto Look_for_F;
   sscanf(buffer, "%s %s", f1, f2);
   strcpy(dfile, f2);

   /* We have the D. filename, now determine the munged name we want */

   /* Try and open the munged filename */
   looking = 0;
   while(looking <= (strlen(dfile) - 2)){
    /* See if it matches hosts Uucpname */
    strcpy(munged_name, munge163(host_name, dfile, looking));
    strcpy(temp_pathname, sub_path);
    strcat(temp_pathname, munged_name);

    if((check = fopen(temp_pathname, "r")) == NULL){
     /* Check if it matches our Uucpname */
     if(looking == 0){
      strcpy(munged_name, munge163(uucpname, dfile, looking));
      strcpy(temp_pathname, sub_path);
      strcat(temp_pathname, munged_name);
      if((check = fopen(temp_pathname, "r")) != NULL){
       fclose(check);
       break;
      }
     }
     /* This should never happen but we'll try anyway */
     ++looking;
     fprintf(log, "!EXTRA_MUNGE %d\n", looking);
    }
    else{
     if(looking != 0) looking = -1;
     fclose(check);
     break;
    }
   } /* End While */
   if(looking > 0){
    fprintf(stderr, "Didn't find a matching D file for %s\n", dfile);
    fprintf(log, "!NOMUNGE %s in %s\n", dfile, filename);
    rename_spools(filename, NULL, log, 1);
    done = findnext(&ffblk);
    continue;
   }
   else{
    fprintf(stderr, "Found D file: %s\n", munged_name);
    if(looking == -1){
     fprintf(log, "+MUNGED XOK %s\n", munged_name);
    }
    else{
     fprintf(log, "+MUNGED  OK %s\n", munged_name);
    }
   }
  }

  Look_for_C:
  if(fgets(buffer, 256, in) == NULL){
   fprintf(stderr, "Error - didn't file 'C' line in %s.\n", filename);
   fprintf(log, "!FREDTOSS/No_C_IN/%s\n", filename);
   rename_spools(filename, NULL, log, 1);
   done = findnext(&ffblk);
   continue;
  }

  if(*buffer != 'C') goto Look_for_C;

  sscanf(buffer, "%s %s %s", f1, f2, f3);

  if(strcmp(f2, "rnews") == 0){
   rnews_rmail = 1; /* It is news */
  }
  else if(strcmp(f2, "rmail") == 0){
   rnews_rmail = 2; /* It is mail */
  }
  else
   rnews_rmail = 0; /* Who knows what it is */

  if((rnews_rmail == 2) && !newsonly){
   fprintf(stderr, "X file %s ", filename);
   fprintf(stderr, "-> mail file %s.\n", munged_name);
   fprintf(log, "+MAIL %s %s\n", filename, munged_name);

   if(strcmp(f2, "rmail") == 0){
    strcpy(maildest, f3);
    fprintf(stderr, "Found mail for : %s\n", maildest);
   }
  }
  else if((rnews_rmail == 1) && !mailonly){
   fprintf(stderr, "X file %s ", filename);
   fprintf(stderr, "-> news file %s.\n", munged_name);
   fprintf(log, "+NEWS %s %s\n", filename, munged_name);
  }
  else{
   fprintf(stderr, "X file %s ", filename);
   fprintf(stderr, "-> UNKNOWN file %s.\n", munged_name);
   fprintf(log, "+???? %s %s\n", filename, munged_name);
   rename_spools(filename, NULL, log, 1);
   done = findnext(&ffblk);
   continue;
  }

  fclose(in);

 /**/
 /* DECOMPRESS to a temp file, and run Fred-Uf or Mail-Uf */
 /**/

  if((rnews_rmail == 2) && !newsonly){ /* Must be rmail */

   strcpy(pathname, sub_path);
   strcat(pathname, munged_name);

   sprintf(tmpname, "%ld", time(NULL));

   strcpy(inbname, inbound_path);
   strcat(inbname, tmpname+1);
   strcat(inbname, ".PKT");

   fprintf(stderr,"Tossing to %s...\n", inbname);

   if(freopen(pathname, "r", stdin) == NULL){
    fprintf(stderr, "ERROR: Can't open %s.\n", pathname);
    fprintf(log, "!FREDTOSS/freopen()/%s\n", pathname);
    rename_spools(filename, NULL, log, 1);
    done = findnext(&ffblk);
    continue;
   }

   if(freopen(inbname, "w", stdout) == NULL){
    fprintf(stderr, "ERROR: Can't open %s.\n", inbname);
    fprintf(log, "!FREDTOSS/freopen()/%s\n", inbname);
    exit(1);
   }

   fclose(log);

   if(!test_flag){
    result = spawnlp(P_WAIT, "mail-uf.exe", "mail-uf.exe", "-c", config_file,
		     "-rmail", maildest, NULL);
   }
   else{
    fprintf(stderr, "COMMAND: MAIL-UF -c %s -rmail %s\n", config_file,
		     maildest);
    result = 0;
   }

   if(result == -1){
    if(errno == ENOENT){
     fprintf(stderr, "ERROR: MAIL-UF.EXE not found.\n");
     fprintf(stderr, "It should be located on your PATH!\n");
     exit(2);
    }
   }

   freopen("CON", "r", stdin);
   freopen("CON", "w", stdout);
   log = fopen(logfile, "a");

   switch(result){

    case 0:
     fprintf(stderr,"Deleting %s.\n\n", inbname);
     fprintf(log, "+DELETE_EMPTY/%s\n", inbname);
     remove(inbname);

     /* Delete spool files if delete is specified */
     if(delete_files){
      remove(filename);
      remove(pathname);
     }
     break;

    case 1:
     fprintf(stderr, "Mail-Uf Reports normal exit.\n");

     /* Delete spool files if delete is specified */
     if(delete_files){
      remove(filename);
      remove(pathname);
     }
     break;

    default:
     fprintf(stderr,"Mail-Uf Reports error level of %d at exit.\n", result);
     fprintf(log, "!RVCD ERR %d\n", result);
     rename_spools(filename, pathname, log, 0);
     break;
   }


  }

  else if((rnews_rmail == 1) && !mailonly){ /* Must be rnews */
   strcpy(pathname, sub_path);
   strcat(pathname, munged_name);

   fprintf(stderr, "Decompressing...\n");
   strcpy(temp_pathname, temp_path);
   strcat(temp_pathname, "tempnews");

   if(freopen(pathname, "r", stdin) == NULL){
    fprintf(stderr, "ERROR: Can't open %s.\n", pathname);
    fprintf(log, "!FREDTOSS/freopen()/%s\n", pathname);
    exit(1);
   }

   if(freopen(temp_pathname, "w", stdout) == NULL){
    fprintf(stderr, "ERROR: Can't open %s.\n", temp_pathname);
    fprintf(log, "!FREDTOSS/freopen()/%s\n", temp_pathname);
    exit(1);
   }

   /* Check to see if this is already compressed or if it  */
   /* has #! cunbatch at the start */

   if(fread(buffer, sizeof(char), 18, stdin) == 0){
    fprintf(stderr, "ERROR: can't read news file.\n");
    exit(1);
   }

   if(strncmpi(buffer, "#! cunbatch", 11) == 0){
    nbits = *(buffer + 14) - 0x80;
    fprintf(stderr, "Found CUNBATCH news file compressed with %d bits.\n", nbits);
    fseek(stdin, 12L, SEEK_SET);
   }
   else if((*buffer == 0x1f) && (*(buffer + 1) == 0x9d)){
    nbits = *(buffer + 2) - 0x80;
    fprintf(stderr, "Found non-CUNBATCH news file compressed with %d bits.\n", nbits);
    fseek(stdin, 0L, SEEK_SET);
   }
   else{
    fprintf(stderr, "Found unCOMPRESSED news file.\n");
    fseek(stdin, 0L, SEEK_SET);
    nbits = 0;
   }

   if(nbits != 0){
    fprintf(log, "+DECOMP %s\n", pathname);

    if(!test_flag){
     result = spawnlp(P_WAIT, "compress.exe",
		 "compress.exe",
			     "-dv", NULL);
    }
    else{
     fprintf(stderr, "COMMAND: COMPRESS.EXE -dv <%s >%s\n",
	      pathname, temp_pathname);
     result = 0;
    }

    if(result == -1){
     if(errno == ENOENT){
      fprintf(stderr, "ERROR: COMPRESS.EXE not found.\n");
      fprintf(stderr, "It should be located on your PATH!\n");
      fprintf(log, "!COMPRESS.EXE NOT FOUND\n");
      exit(2);
     }
    }
   }


   if(freopen("CON", "r", stdin) == NULL){
    fprintf(stderr, "ERROR: Can't freopen() stdin.\n");
    exit(1);
   }

   if(freopen("CON", "w", stdout) == NULL){
    fprintf(stderr, "ERROR: Can't freopen() stdout.\n");
    exit(1);
   }

   sprintf(tmpname, "%ld", time(NULL));

   strcpy(inbname, inbound_path);
   strcat(inbname, tmpname+1);
   strcat(inbname, ".PKT");

   fprintf(stderr,"Tossing to %s...\n", inbname);
   fprintf(log, "+TOSS %s\n", inbname);

   if(nbits != 0)
    freopen(temp_pathname, "r", stdin);
   else
    freopen(pathname, "r", stdin);

   freopen(inbname, "w", stdout);

   fclose(log);

   if(!test_flag){

    /* Run the LARGE version if specified */
    if(UseLarge)
     strcpy(FredCmd, "Fred-Ufl");
    else
     strcpy(FredCmd, "Fred-Uf");

    if(quiet)
     result = spawnlp(P_WAIT, FredCmd, FredCmd, "-c", config_file,
		     "-quiet", NULL);
    else
     result = spawnlp(P_WAIT, FredCmd, FredCmd, "-c", config_file,
		     NULL);
   }
   else{
    fprintf(stderr, "COMMAND: %s -c %s\n", FredCmd, config_file);
    result = 0;
   }

   if(result == -1){
    if(errno == ENOENT){
     fprintf(stderr, "ERROR: %s.Exe not found.\n", FredCmd);
     fprintf(stderr, "It should be located on your PATH!\n");
     fprintf(log, "!%s.Exe NOT FOUND\n", FredCmd);
     exit(2);
    }
   }

   freopen("CON", "r", stdin);
   freopen("CON", "w", stdout);

   if((log = fopen(logfile, "a")) == NULL){
    fprintf(stderr, "ERROR: FREDTOSS Can't reopen logfile %s.\n", logfile);
    exit(1);
   }

   switch(result){

    case 0:
     fprintf(stderr,"Deleting %s.\n\n", inbname);
     fprintf(log, "!DLT_EMPTY/%s\n", inbname);
     remove(inbname);
     /* Delete spool files if delete is specified */
     if(delete_files){
      remove(filename);
      remove(pathname);
     }
     break;

    case 1:
     fprintf(stderr, "Fred-Uf Reports normal exit.\n");
     /* Delete spool files if delete is specified */
     if(delete_files){
      remove(filename);
      remove(pathname);
     }
     break;

    default:
     fprintf(stderr,"Fred-Uf Reports error level of %d at exit.\n", result);
     fprintf(log, "!RVCD ERR %d\n", result);
     rename_spools(filename, pathname, log, 0);
     break;
   }

   /* Get rid of the temp files */
   if((nbits != 0) && NoSaveTemp)
    remove(temp_pathname);
  }

  done = findnext(&ffblk);

 } /* End WHILE */

 fprintf(stderr,"FREDTOSS done\n");
 t = time(NULL);
 fprintf(log, "$FREDTOSS STOP @TM=%s\n", ctime(&t));
 fclose(log);

}


/* Rename BAD spool files so they don't get run again */
void rename_spools(char *filename, char *pathname, FILE *log, int code)
{
   char *dup_str;

   /* They were *.X and *.D, rename to *.X!! and *.D!! */

   if((code == 0) || (code == 1)){
    fprintf(stderr,"RENAMING spool file %s\n", filename);
    dup_str = strdup(filename);
    strcat(filename, "!!");
    fprintf(log, "!RENAME %s\n", filename);
    rename(dup_str, filename);
    free(dup_str);
   }

   if((code == 0) || (code == 2)){
    fprintf(stderr, "RENAMING spool file %s\n", pathname);
    dup_str = strdup(pathname);
    strcat(pathname, "!!");
    fprintf(log, "!RENAME %s\n", pathname);
    rename(dup_str, pathname);
    free(dup_str);
   }
}
