/*
File          : getfnl.c            (Get File Name List)
Author        : Thomas P. Devitt
                Pseudonym Software - Data: 818-796-4197
Updated on    : May 23, 1987
Include files : errno.h        (for global var errno and DOS error definitions)
Portibility   : PC/MS-DOS only
Defined in    : Lattice C v 3.xx (Class: Lattice)
Synopsis      : n = getfnl(fnp, fna, fnasize, attr);

                int n;                  number of matching file names
                char *fnp;              file name pattern
                char *fna;              file name array string
                unsigned fnasize;       size of filename array string
                int attr;               file attribute

Description   : gets all file names that match the fnp pattern ans places them
                into the file name array string fna.  Each name is stored as
                a '\0' terminated string and the entire array string is
                terminated by a null string (ie. another '\0' - note that this
                is the same way that DOS stores the environment strings).  If
                fnp contains a drive and/or a path prefix, that prefix is
                included in the array with each file name.

                attr is a set of flag bits as follows:

                    Bit 0 =>   (value   1)  Reserved, must be 0
                    Bit 1 =>   (value   2)  Hidden file flag
                    Bit 2 =>   (value   4)  System file flag
                    Bit 3 =>   (value   8)  Volume label flag
                    Bit 4 =>   (value  16)  Subdirectory flag
                    Bit 5 =>   (value  32)  Reserved, must be 0
                    Bit 6 =>   (value  64)  Reserved, must be 0
                    Bit 7 =>   (value 128)  Reserved, must be 0

                If all bits are 0, getfnl will find only normal and read-only
                files.  To find additional types of files, set the appropriate
                bit in attr to 1.  For example, attr = 6 will find all file
                types.

Returns       : If no error has occurred, the count of file names stored in
                the fna array string is returned.  Otherwise, -1 is returned
                and the global variable errno (defined in errno.h) is set to
                one of the following:

                    ENOENT    Path or file name not found
                    E2BIG     Not enough room in the file name array

*****************************************************************************/

#include <stdio.h>
#include <dos.h>
#include <dir.h>
#include <string.h>
#include <errno.h>

int getfnl(char *fnp, char *fna, unsigned fnasize, int attr)
{
   char far *dta;
   struct   ffblk  newdta;
   char     path[64], temp[81];
   char     *fnaptr;

   int      count = 0;
   int      len   = 0;

   dta = getdta();

   strcpy(path,fnp);
   fnaptr = strrchr(path,'\\');
   if (fnaptr == NULL)
      fnaptr = strrchr(path,':');

   if (fnaptr == NULL)
      *path = '\0';
   else
   {
      *++fnaptr = '\0';
      strupr(path);
   }
   fnaptr = fna;

   if (findfirst(fnp, &newdta, attr | FA_ARCH ) == 0)
   {
       strcpy(temp,path);
       strcat(temp,newdta.ff_name);
       len += strlen(temp);
       if (len > fnasize)
       {
          *fnaptr = '\0';
          errno = E2BIG;
          setdta(dta);
          return -1;
       }
       strcpy(fna,temp);
       fna = strchr(fna,'\0');
       *++fna = '\0';
       count++;
       len++;
       while ( findnext( &newdta ) == 0 )
       {
          strcpy(temp,path);
          strcat(temp,newdta.ff_name);
          len += strlen(temp);
          if (len > fnasize)
          {
             *fnaptr = '\0';
             errno = E2BIG;
             setdta(dta);
             return -1;
          }
          strcpy(fna,temp);
          fna = strchr(fna,0);
          *++fna = '\0';
          count++;
          len++;
       }
   }
   else
   {
      errno = ENOENT;
      setdta(dta);
      return -1;
   }
   setdta(dta);
   return count;
}
