/* This block of code demonstrates quite clearly two problems in the current
   C5 graphics library. This is partially documented in the OUTTEX.ARC file
   in dl1, but the following shows it much more clearly.

   Put it into QC, run it (needs the Program list set because of the graphics)
   and then uncomment the various blocks to see that happens to the text. You
   are prompted for the width of a block - enter a number between 1 and 299.


   The text output illustrates two things .. first is the bug that may be
   at the root of the previously discussed _outtext() + _get/putimage() problems.
   That is that the _imagesize() function does not operate as documented in
   the Run Time Reference manual. The function actually returns a value quite
   different than that documented, except when the image is a certain pixel
   width.

   By fiddling around, I found a formula that duplicates what the function
   returns (see the code below) - and I show on the screen both what the
   _imagesize() function returns, what the formula in the manual says it
   is supposed to return, and the value from my new formula - which duplicates
   exactly what _imagesize returns.

   The second function of the text is to show what happens with it following
   _getimage() and _putimage(). At certain pixel widths, which vary depending
   on whether you are _putting with GPSET or PGRESET or combinations of those,
   you get text that is difficult to read because of verticle white or black
   bars. If you enter numbers that are either

       multiples of 8, or (n x 8) - 1

  OR

       (n x 8) + 4, or (n x 8) + 3

  depending on whether it is GPSET  or GPRESET, you will no longer have
  the vertical bars.

   Tim Frost (Roundhill) helped here, but it is not as straightforward
   as I hoped. I have worked through several combinations
   of get and putimage, and the results can be seen be working through the
   examples below. Different combinations of get/put require different
   pixel widths to prevent the problem!

   I wonder if the get and putimage calls are using the _imagesize(),
   but getting a number that is not what they expect; or, perhaps they are
   using the documented method of calculating it, but this doesn't jibe
   with other functions... A A A R R R G G G H H H !!!!

   I hope this helps some other people; hopefully all this will be corrected
   in C5.1

   Note I've set the videomode to VGA; set to EGA if you have that; the
   same problem occurs.

 */

#include <stdlib.h>
#include <graph.h>
#include <stdio.h>
#include <math.h>

struct videoconfig vcf;

main()
{
	char buf[10];
	char line[80];
	int width;
	char * image;

	long myimagesize;
	struct rccoord textpos;

     _setvideomode(_VRES16COLOR);     /* change to _ERESCOLOR for EGA */
     _getvideoconfig( &vcf );
     _settextwindow(5,5,24,78);
     _settextposition(1,1);
     image = (char *)malloc(_imagesize(1,1,300,300));

     do {
	sprintf(line,"Enter image width: ");
	_outtext(line);
	textpos = _gettextposition();
	gets(buf);
	if( buf[0] ) {
	   width = atoi(buf);
	   _settextposition(textpos.row+1, 1);

	   sprintf(line, "Width: %i  Bits per pixel %i\nImagesize would be: %ld\n",
		   width, vcf.bitsperpixel,(long)_imagesize(1,1,width,width) );
	   _outtext(line);
	   myimagesize = 4 + ((long)((width*vcf.bitsperpixel+7)/8)*(long)width );
	   sprintf(line, "Image size (from Run Time Ref): %ld\n\n", myimagesize);
	   _outtext(line);

	   myimagesize = 4 + ((long)( ( ( (width*vcf.bitsperpixel - 1)/8 )/4 + 1) * 4 ) * (long)width);

	   sprintf(line, "Image size from new formula: %ld\n\n", myimagesize);
	   _outtext(line);

	}
/*********************************************************************/
/*                                                                   */
/*  The following blocks demonstrate quite clearly the effect of the */
/*  bug in the image transfer routines in C5. The text following the */
/*  bitblock transfer is garbled at certain pixel widths in the      */
/*  example; the following blocks show the fixes for different       */
/*  combinations.                                                    */
/*  Note that the fix is not straightforward - it depends on the     */
/*  COMBINATION of bitmap transfers you do, and I certainly haven't  */
/*  explored all of them!                                            */
/*                                                                   */
/*********************************************************************/


/** this block does its damage unless pixel width (8xn)+(3 or 4)  */

	_getimage(1,1,width, width, image);
	_putimage(300,200,image,_GPSET);



/** this block does its damage unless (8xn)+(3 or 4)  */
/*        _getimage(1,1,width, width, image);
	_putimage(300,200,image,_GPRESET);  */



/** this block trashes the copied text at certain widths; if the block
    width is an even multiple of 8 or (multiple of 8) - 1, then works ok **/

/*        _getimage(1,1,width, width, image);
	_putimage(300,200,image,_GPRESET);
	_putimage(320,220,image,_GPSET);  */

/** if you switch the order of the _GPRESET and _GPSET, then the text
    screens are only OK at (( multiples of 8 ) + 4) or (( multiples of 8 ) + 3) */

/*        _getimage(1,1,width, width, image);
	_putimage(320,220,image,_GPSET);
	_putimage(300,200,image,_GPRESET);  */


/** following blocks work. Uncomment the one you want to try **/
/**  note the different combinations of _GPRESET and _GPSET **/

/*  PRESET FOLLOWED BY PSET needs multiple of 8 pixels */

/*        _getimage(1,1,(width/8)*8 ,(width/8)*8,image);
	_putimage(300,200,image,_GPRESET);
	_putimage(320,220,image,_GPSET); */

/*  PSET FOLLOWED BY PRESET needs multiple of 8 pixels + 4 */

/*        _getimage(1,1,(width/8)*8+4 ,(width/8)*8+4,image);
	_putimage(320,220,image,_GPSET);
	_putimage(300,200,image,_GPRESET);  */


/*  PRESET alone needs multiple of 8 pixels + 4 */

/*        _getimage(1,1,(width/8)*8+4 ,(width/8)*8+4,image);
	_putimage(300,200,image,_GPRESET);  */

/* PSET alone needs multiple of 8 pixels + 4 */

/*        _getimage(1,1,(width/8)*8+4 ,(width/8)*8+4,image);
	_putimage(300,200,image,_GPSET);  */


	while( !kbhit() );
	getch();
	_clearscreen(_GWINDOW);
	_settextposition(1,1);

     } while ( buf[0] );
     _setvideomode(_DEFAULTMODE);
}

