/*  INITDEMO.C
 *  demonstrates mem_init() and array1_alloc() functions
 *  last mod.: 17-JAN-91
 */

/*  with Turbo C compile with -a option  */

#include <STDIO.H>
#include <STDLIB.H>

#include <L_MEM.H>

#define M  6
#define N ( 2+(65536/(10+M)) )

struct date_t
    {
    int day;
    int month;
    long year;
    char foo[M];
    char calendar;
    } date;

char msg[60];

void main(void)
{
Far_str_ptr ptr1;
struct date_t far *dptr;
struct date_t huge *ptr2;
int i, err_flag;

date.day = 21;
date.month = 12;
date.year = 2012;
date.calendar = 'G';

printf("\nThis program allocates space for %d date structures.",N);
printf("\nSize of date structure = %d bytes (M = %d).",sizeof(date),M);
printf("\n\nFirst we use the mem_alloc() and mem_init() functions.");

/*  allocate space for date structures  */
ptr1 = mem_alloc(N*sizeof(struct date_t),&err_flag);
if ( err_flag )
    {
    printf("\nFirst allocation failed: %s",mem_err_msg(err_flag,msg));
    exit(1);
    }

mem_init(ptr1,sizeof(struct date_t),(char *)&date,&err_flag);
if ( err_flag )
    {
    printf("\nInitialization failed: %s",mem_err_msg(err_flag,msg));
    exit(2);
    }

/*  display last three dates in allocated space
 *  plus what is beyond last date
 */
for ( i=N-3; i<=N; i++ )
    {
    dptr = (struct date_t far *)
        ( (Huge_str_ptr)ptr1 + (Ulong)i*sizeof(struct date_t) );
    printf("\nDate #%d = %d/%d/%ld (%c)   (dptr = %Fp)",
        i,dptr->month,dptr->day,dptr->year,dptr->calendar,dptr);
    }
printf("\nFinal \"date\" is beyond allocated space.");

/*  now do the same thing using the array1_alloc() function  */
printf("\n\nNow the same thing using array1_alloc()");

ptr2 = (struct date_t huge *)array1_alloc(N,sizeof(struct date_t),
    (char *)&date, &err_flag);
if ( err_flag )
    {
    printf("\nSecond allocation failed: %s",mem_err_msg(err_flag,msg));
    exit(3);
    }

for ( i=N-3; i<=N; i++ )
    printf("\nDate #%d = %d/%d/%ld (%c)",
        i,ptr2[i].month,ptr2[i].day,ptr2[i].year,ptr2[i].calendar);

printf("\nFinal \"date\" is beyond allocated space.\n");
}
