/*
 ****************************************************************
 *                     File ISAMSTR.H                           *
 *      isam structure declarations for allocated memory        *
 ***************************************************************/

struct  m_elem  { /* this structure defines a main element entry */
        long            offset;         /* record position of "center" l2 block */
        unsigned        group;          /* # of l2 block in this main group */
        char            key[1];         /* 1st character of largest key in group */
};

struct  l2_link { /* this defines an l2 block header */
        long            _lowlink;       /* rec position of next lower l2 block */
        long            _highlink;      /* rec position of next higher l2 block */
        unsigned        _l2_ent;        /* # of currently active l2 elements in this block */
        char            _lowkey[1];     /* 1st character of largest key in lowlink block */
};                                      /* (only valid if lowlink is not zero) */

struct  l2_elem { /* this defines an l2 block element */
        long            offset;         /* rec position of l3 block */
        unsigned        group;          /* # of l3 elements in referenced l3 block */
        char            key[1];         /* 1st character of largest key in l3 block */
};

struct  l3_elem { /* this defines an l3 block element */
        long            offset;         /* rec position in data file of record with this key */
        char            l3key[1];       /* 1st character of key */
};

struct  recov   { /* this defines a recovery block */
        long            link;           /* rec position of next recovery block */
        unsigned        rb_ent;         /* # of active free records in this block */
        long            roffset[1];     /* 1st offset of free record in this block */
};                                      /* (array length is actually r_max) */

struct  global  {
        struct m_elem   *_bm_ptr;       /* ptr to 1st element in main in memory */
        struct l2_elem  *_bl2_ptr;      /* ptr to 1st elem in l2 in memory */
        struct l3_elem  *_bl3_ptr;      /* ptr to 1st elem in l3 in memory */
        struct recov    *_br_ptr;       /* ptr to recovery block in memory */
        struct l2_elem  *_wbl2_ptr;     /* ptr to 1st elem in working l2 in memory */
        struct l3_elem  *_wbl3_ptr;     /* ptr to 1st elem in working l3 in memory */
        struct l2_link  *_l2s_ptr;      /* ptr to l2 link header in memory */
        struct l2_elem  *_cal2;         /* ptr to active l2 elem in memory */
        struct m_elem   *_cama;         /* ptr to active main element in memory */
        struct l3_elem  *_cal3;         /* ptr to active l3 elem in memory */
        char            _m_drty;        /* main dirty flag TRUE = dirty */
        char            _l2_drty;       /* l2 dirty flag TRUE = dirty */
        char            _l3_drty;       /* l3 dirty flag TRUE = dirty */
        char            _r_drty;        /* recovery block dirty flag TRUE = dirty */
        long            _l2_group;      /* current active l2 group record position */
        long            _l2_cur;        /* current l2 record in memory */
        long            _l3_cur;        /* current l3 record in memory */
        long            _r_cur;         /* current recovery record in memory */
        unsigned        _l2elem;        /* sizeof an l2 element */
        unsigned        _l3elem;        /* sizeof an l3 element */
        unsigned        _l2bytes;       /* length of an l2 block */
        unsigned        _l3bytes;       /* length of an l3 block */
        unsigned        _rbytes;        /* length of a recovery block */
        int             _ifd;           /* index file descriptor */
/*
 ****************************************************************
 *         main header data from disk starts here               *
 ***************************************************************/

        unsigned        _keylen;        /* length of key */
        unsigned        _reclen;        /* length of record */
        char            _flags;         /* administrative flags */
        long            _ieof;          /* index end of file position */
        long            _deof;          /* data end of file position */
        long            _rbp;           /* first recovery block position */
        long            _l2bp;          /* first freed l2 block position */
        long            _l3bp;          /* first freed l3 block position */
        long            _frbp;          /* first freed recovery block position */
        unsigned        _m_max;         /* maximum number of main elements */
        unsigned        _l2_max;        /* maximum number of l2 elements */
        unsigned        _l3_max;        /* maximum number of l3 elements */
        unsigned        _r_max;         /* maximum number of recovery elements */
        unsigned        _cm_ent;        /* number of currently active main elements */
        char            _lp_key[1];     /* last processed key */
};
/* 
 ****************************************************************
 *      these macros are used to simplify the inline code. they *
 *      prevent having to type constructs like:                 *
 *              index->_l2s_ptr->_highlink                      *
 *      this would be entered as:                               *
 *              highlink                                        *
 ***************************************************************/

#define bm_ptr          (index->_bm_ptr)
#define bl2_ptr         (index->_bl2_ptr)
#define bl3_ptr         (index->_bl3_ptr)
#define br_ptr          (index->_br_ptr)
#define wbl2_ptr        (index->_wbl2_ptr)
#define wbl3_ptr        (index->_wbl3_ptr)
#define ws_ptr          ((struct l2_link *) wbl3_ptr)
#define l2s_ptr         (index->_l2s_ptr)
#define cal2            (index->_cal2)
#define cama            (index->_cama)
#define cal3            (index->_cal3)
#define m_drty          (index->_m_drty)
#define l2_drty         (index->_l2_drty)
#define l3_drty         (index->_l3_drty)
#define r_drty          (index->_r_drty)
#define l2_group        (index->_l2_group)
#define l2_cur          (index->_l2_cur)
#define l3_cur          (index->_l3_cur)
#define r_cur           (index->_r_cur)
#define keylen          (index->_keylen)
#define reclen          (index->_reclen)
#define flags           (index->_flags)
#define ieof            (index->_ieof)
#define deof            (index->_deof)
#define rbp             (index->_rbp)
#define l2bp            (index->_l2bp)
#define l3bp            (index->_l3bp)
#define frbp            (index->_frbp)
#define m_max           (index->_m_max)
#define l2_max          (index->_l2_max)
#define l3_max          (index->_l3_max)
#define r_max           (index->_r_max)
#define cm_ent          (index->_cm_ent)
#define lp_key          (index->_lp_key)
#define highlink        (l2s_ptr->_highlink)
#define lowlink         (l2s_ptr->_lowlink)
#define l2_ent          (l2s_ptr->_l2_ent)
#define lowkey          (l2s_ptr->_lowkey)
#define wl2_highlink    (ws_ptr->_highlink)
#define wl2_lowlink     (ws_ptr->_lowlink)
#define wl2_ent         (ws_ptr->_l2_ent)
#define wl2_lowkey      (ws_ptr->_lowkey)
#define melem           (index->_l2elem)
#define l2elem          (index->_l2elem)
#define l3elem          (index->_l3elem)
#define l2bytes         (index->_l2bytes)
#define l3bytes         (index->_l3bytes)
#define rbytes          (index->_rbytes)
#define ifd             (index->_ifd)
