
/****************************************************************
************\
 *  ltncy.c -- iRMX II interrupt latency benchmarking program.
 *  Method:  This program first sets up an interrupt handler for
an unused
 *    interrupt level. It then reads the count in the system timer
(timer 
 *    0 on the 8254 chip) and simulates an external interrupt to
the CPU by 
 *    a cause$interrupt instruction. The interrupt handler latches
timer 0, 
 *    so this program can read it again after the handler returns
control. 
 *    The difference in the two timer-count values is the interrupt
latency.
 *                    Oct 1989, by R. P. Kar
\****************************************************************
************/

#include <stdio.h>
#include <rmxc.h>

/* Define base address of 8254 (Programmable Interval Timer) chip
*/
#define        PIT_ADDR   0x40

unsigned       status, ticks, timer_cnt1, timer_cnt2;
unsigned       dummy_w;
unsigned char  pri, lo_cnt1, hi_cnt1, lo_cnt2;

extern void int_hndlr();

/***************************   MAIN PROGRAM  
**************************/
main ()

{
printf("                   ***  WARNING ***\n\n");
printf("  This program assumes that timer and interrupt
controller\n");
printf("  hardware is fully compatible with the IBM PC/AT\n\n");

/* Set up local handler for IRQ3 on master 8259 */
rqsetinterrupt( 0x38, 0, int_hndlr, (selector)0, &status );
disable();                                         /* Disable
interrupts */

/* Latch and read timer 0 value. Interrupt handler will latch it
again */
outbyte( PIT_ADDR + 3, 0 );

/* The following two instructions read the value latched in counter
0. They
    are unavoidable measurement overhead and inflate the interrupt
latency
    by a few clock cycles.
 */
lo_cnt1 = inbyte( PIT_ADDR );
hi_cnt1 = inbyte( PIT_ADDR );

/* Activate the interrupt handler. It will latch timer 0 and
return. */
causeinterrupt(59);

/* The interrupt handler has latched the timer 0 count. Now read
it. */
lo_cnt2 = inbyte( PIT_ADDR );
dummy_w = (unsigned int)inbyte( PIT_ADDR );
timer_cnt2 = (unsigned int)lo_cnt2 + (dummy_w << 8);

enable();                                        /* Re-enable
interrupts */

dummy_w = (unsigned int)hi_cnt1;
timer_cnt1 = (unsigned int)lo_cnt1 + (dummy_w << 8);

/* Calculate difference in timer counts (timer counts DOWN to 0)
*/
if (timer_cnt1 > timer_cnt2)
    ticks = timer_cnt1 - timer_cnt2;
else                           /* Rare case when timer has wrapped
around */
     ticks = timer_cnt1 + (0xffff - timer_cnt2 + 1);

/* Display results */
printf("   Interrupt latency = %u timer ticks\n", ticks);

/* Note that timer is pulsed by 1.19 MHz crystal */
printf("                     = %4.1f microseconds\n\n",
((float)ticks)/1.19 );

rqresetinterrupt( 0x38, &status );
dqexit(0);
}

