/**/
/********************************************************/
/*                                                      */
/*                     MAIL-UF.C                        */
/*   Copyright 1991 J. W. Hance - All rights reserved   */
/*         Read a Uucp format mail message              */
/*         and build a FidoNet Mail Packet              */
/*                                                      */
/********************************************************/
/**/

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <alloc.h>
#include <string.h>
#include <io.h>
#include <fcntl.h>
#include <time.h>
#include <dos.h>
#include <mem.h>

#include "version.h"
#include "fred.h"

int Debug;
unsigned int headers;

/**/
/******************** MAIN **********************/
/**/

void main(int argc, char *argv[])
{
 char *cpt, c, *cpt2, *cpt3;
 unsigned long i, j, k, lcount;
 int flag, bigflag;
 int offset, result;
 int ct, tcount;
 unsigned int count;
 char *from_node, *to_node, *uucpname, *curgroup, *logfile;
 char *origin_node, *origin;
 char *domain;
 char *to_uucp_address;
 char *filename, *tmp, *buffer, *who;
 char *config_file, *too_bigs;
 int config_file_flag, single_news;
 char *f0, *f1, *f2, *f3, *f4;
 struct mail *mailptr, *firstmailptr, *lastmailptr;
 struct PktHeader *PktHeader;
 struct MsgHeader *MsgHeader;
 FILE *cffp, *tmpfp, *logfp;
 char ascii_time[32], tmp_path[128];
 char *hold_date;
 time_t t, start_time;
 char *start_of_message, *end_of_message;
 struct date d;
 struct time tm;
 struct nodeaddr fnode, tnode, onode;
 int attribute = 0;

 /**/
 /* Set debug mode */
 /**/

 Debug = 0; /* Off */

 /**/
 /* Print startup message */
 /**/

 fprintf(stderr,"\n* Mail-Uf Version %s ", MUVERSION);
 fprintf(stderr,"Built %s at time %s *\n", __DATE__, __TIME__);
 fprintf(stderr,"* Copyright 1990, 1991 J. W. Hance - ");
 fprintf(stderr,"All Rights Reserved *\n");

 /**/
 /* Verify correct command line usage */
 /**/

 if(argc != 5){
 Usage:
  fprintf(stderr, "\nERROR: Improper commandline usage.\n");
  fprintf(stderr, "usage: mail-uf -c config -rmail address <batch >packet\n");
  fprintf(stderr, "where: config is the configuration file.\n");
  fprintf(stderr, "       address is the address to send mail to.\n");
  fprintf(stderr, "       batch is the mail file.\n");
  fprintf(stderr, "       packet is the FidoNet mail packet to build.\n");
  exit(2);
 EndIf

 else{

 /**/
 /* Sift out command line options */
 /**/
  config_file_flag = 0;

  for(ct = 1; ct < argc; ct++){
   if(strcmpi(argv[ct], "-c") == 0){
    if(config_file_flag){
     fprintf(stderr, "ERROR: Redundant '-c' switches\n");
     exit(2);
    EndIf
    else{
     config_file = AllocateString(strlen(argv[++ct]));
     strcpy(config_file, argv[ct]);
     config_file_flag++;
    EndElse
   EndIf

   else if(strcmpi(argv[ct], "-rmail") == 0){
    to_uucp_address = AllocateString(strlen(argv[++ct]));
    strcpy(to_uucp_address, argv[ct]);
   EndIf

   else{
    goto Usage;
   EndElse
  EndFor
 EndElse

 /* Done processing command line options */

 /**/
 /* Allocate a BUFFER for use in reading the batch file */
 /**/

 if((cpt = malloc(ALLOC_SIZE)) == NULL){
  fprintf(stderr,"ERROR: Memory allocation failed.\n");
  fprintf(stderr,"Probably not enough memory to run.\n");
  exit(3);
 EndIf

 /**/
 /* Now process the configuration file */
 /**/

 /* Try to open config file */
 cffp = OpenFile(config_file, "r");

 buffer = AllocateString(255);
 mailptr = NULL;
 flag = 0;

 f0 = AllocateString(127);
 f1 = AllocateString(127);
 f2 = AllocateString(127);
 f3 = AllocateString(127);
 f4 = AllocateString(127);

 while(fgets(buffer, 256, cffp) != NULL){

  /* See if it is a comment */
  if(*buffer == '#') continue;
  else if(*buffer == ';'){
   fprintf(stderr, "!! ERROR - Config file format has changed, read docs\n");
   exit(5);
  EndIf

  /* Otherwise look for interesting config statements */

  sscanf(buffer, "%s %s %s %s", f1, f2, f3, f4);

  if(strcmpi(f1, "from_node") == 0){
   from_node = AllocateString(strlen(f2));
   strcpy(from_node, f2);
   continue;
  EndIf

  if(strcmpi(f1, "logfile") == 0){
   logfile = AllocateString(strlen(f2));
   strcpy(logfile, f2);

   /* Try to open the logfile */
   logfp = OpenFile(logfile, "a");

   t = time(NULL);
   fprintf(logfp, "\n#MAIL-UF START @TM=%s+<CF=%s><RM=%s>\n",
						     ctime(&t),
						     config_file,
						     to_uucp_address);
   continue;
  EndIf

  /* Get which headers to include in FTSC message */
  if(strcmpi(f1, "headers") == 0){
   if(strcmpi(f2, "yes") == 0){
    if(strcmpi(f3, "all") == 0){
     headers = 0x00ff;
    EndIf
    else{
     headers = WhichHeaders(f3);
    EndElse
   EndIf
   else{
    headers = 0;
   EndElse
   /* Mask out the NewsGroups bit */
   headers &= 0xfff7;
   /* And always include From: and Path: */
   headers &= 0xffcf;
   headers |= 0x0030;
  EndIf

  if(strcmpi(f1, "to_node") == 0){
   to_node = AllocateString(strlen(f2));
   strcpy(to_node, f2);
   continue;
  EndIf

  if(strcmpi(f1, "who") == 0){
   who = AllocateString(strlen(f2));
   strcpy(who, f2);
   continue;
  EndIf

  if(strcmpi(f1, "origin_node") == 0){
   origin_node = AllocateString(strlen(f2));
   strcpy(origin_node, f2);
   continue;
  EndIf

    /* Take care of the SetBits options */
    if(strcmpi(f1, "setbits") == 0){

        if(strcmpi(f2, "crash") == 0)
            attribute |= MsgCrash; /* Set the CRASH bit */
        else if(strcmpi(f2, "private") == 0)
            attribute |= MsgPrivate; /* Set the PRIVATE bit */
        else if(strcmpi(f2, "local") == 0)
            attribute |= MsgLocal; /* Set the LOCAL bit */
        else if(strcmpi(f2, "killsent") == 0)
            attribute |= MsgKill; /* Set the KillSent Bit */
        else if(strcmpi(f2, "hold") == 0)
            attribute |= MsgHold; /* Set the HOLD Bit */
        else if(strcmpi(f2, "intransit") == 0)
            attribute |= MsgInTrans; /* Set the Intransit bit */
        else
            fprintf(stderr, "ERROR - '%s' is an unknown SETBITS option\n", f2);

        continue;
    EndIf

  if(strcmpi(f1, "mailuf") == 0){
   lastmailptr = mailptr;
   mailptr = malloc(sizeof(struct mail));
   if(mailptr == NULL){
    fprintf(stderr, "ERROR: Can't allocate memory for mail struct.\n");
    exit(6);
   EndIf

   if(flag == 0){
    flag++;
    firstmailptr = mailptr;
   EndIf


   lastmailptr->next = mailptr;
   mailptr->next = NULL;
   mailptr->uucpname = AllocateString(strlen(f2));
   strcpy(mailptr->uucpname, f2);

   mailptr->system = AllocateString(strlen(f3));
   strcpy(mailptr->system, f3);

   /* If it is a GateMail type MailUf, set the pointer to NULL */
   if(strcmpi(f4, "gatemail") == 0){
    mailptr->fidoname = NULL;
   EndIf
   else{
    tmp = memchr(buffer, QUOTE, 250);
    tmp = memccpy(f1, ++tmp, QUOTE, 255);
    *(tmp-1) = NULL;

    sscanf(f1, "%s %s", f3, f2);
    mailptr->fidoname = AllocateString(strlen(f3)+strlen(f2)+1);
    strcpy(mailptr->fidoname, f3);
    strcat(mailptr->fidoname, " ");
    strcat(mailptr->fidoname, f2);
   EndElse

   continue;
  EndIf

 EndWhile

 fclose(cffp);

 /* Done Reading Configuration file */

 /**/
 /* Need to check here that all config items are set */
 /**/


 /* Find out what time it is */
 t = time(NULL);
 strcpy(ascii_time, ctime(&t));
 sscanf(ascii_time, "%s %s %s %s %s", f0, f1, f2, f3, f4);
 strcpy(ascii_time, f2);
 strcat(ascii_time, " ");
 strcat(ascii_time, f1);
 strcat(ascii_time, " ");
 strcat(ascii_time, &f4[2]);
 strcat(ascii_time, " ");
 strcat(ascii_time, f3);
 /* ascii_time should now be of form dd Mmm yy hh:mm:ss */

 /* Also get the numerical time from DOS */
 getdate(&d);
 gettime(&tm);

 /* Just so I can sleep nights ! */
 if(d.da_year < 2000)
  d.da_year -=1900;
 else
  d.da_year -=2000;

 /* Ok, now we have all the time we need */

 /* Initialize and write the Packet Header to the packet_file */

 /* Make space for the Headers */
 PktHeader = (struct PktHeader *)calloc(sizeof(struct PktHeader), 1);
 if(PktHeader == NULL){
  fprintf(stderr, "ERROR: calloc() failed for PktHeader\n");
  fprintf(logfp, "!calloc()/PktHeader\n");
  exit(7);
 }

 MsgHeader = (struct MsgHeader *)calloc(sizeof(struct MsgHeader), 1);
 if(MsgHeader == NULL){
  fprintf(stderr, "ERROR: calloc() failed for MsgHeader\n");
  fprintf(logfp, "!calloc()/MsgHeader\n");
  exit(8);
 }

 /* Init various fields in PktHeader amd MsgHeader */
 PktHeader->Year = d.da_year;
 PktHeader->Month = --d.da_mon;
 PktHeader->Day = d.da_day;
 PktHeader->Hour = tm.ti_hour;
 PktHeader->Minute = tm.ti_min;
 PktHeader->Second = tm.ti_sec;
 PktHeader->Baud = 0;
 PktHeader->Version = 2;
 PktHeader->ProdSer = 0x0000;
 for(i=0; i< 8; i++) PktHeader->Password[i] = 0x00;
 for(i=0; i<20; i++) PktHeader->Fill[i] = 0x00;

 MsgHeader->Attribute = attribute;
 MsgHeader->Cost = 0;
 MsgHeader->Version = 2;

 /* Now get the node numbers into the Packet and message headers */
  if(VerifyNode(from_node, &fnode) == -1){
   fprintf(stderr, "ERROR: Node address %s for FROM_NODE is invalid\n", from_node);
   fprintf(logfp, "!FROM_NODE/%s\n", from_node);
   exit(9);
  }

  PktHeader->OriginNode = fnode.node;
  PktHeader->OriginNet  = fnode.net;
  PktHeader->OriginZone = fnode.zone;

  if(VerifyNode(origin_node, &onode) == -1){
   fprintf(stderr, "ERROR: Node address %s for ORIGIN_NODE is invalid\n", origin_node);
   fprintf(logfp, "!ORIGIN_NODE/%s\n", origin_node);
   exit(10);
  }

  MsgHeader->OriginNode = onode.node;
  MsgHeader->OriginNet  = onode.net;

  if(VerifyNode(to_node, &tnode) == -1){
   fprintf(stderr, "ERROR: Node address %s for TO_NODE is invalid\n", to_node);
   fprintf(logfp, "!TO_NODE/%s\n", to_node);
   exit(10);
  }

  PktHeader->DestNode = tnode.node;
  PktHeader->DestNet  = tnode.net;
  PktHeader->DestZone = tnode.zone;
  MsgHeader->DestNode = tnode.node;
  MsgHeader->DestNet  = tnode.net;

 /* Put stdout in BINARY mode */
 setmode(fileno(stdout), O_BINARY);

 /* Put stdin in TEXT mode */
 setmode(fileno(stdin), O_BINARY);

 /* Make sure the stdin and stdout are at beginning of file */
 fseek(stdin, 0L, SEEK_SET);
 fseek(stdout, 0L, SEEK_SET);

 /* Write out the Packet Header */
 fwrite(PktHeader, sizeof(struct PktHeader), 1, stdout);
 fflush(stdout);

 free(PktHeader);


 /* Header is written - now go on to messages */

 /**/
 /**********************************************/
 /*      Now go through all the messages       */
 /**********************************************/
 /**/

 /* No apologies for the use of gotos in this section */
 /* sometimes it's the best and most convenient method */

 while(1){

  /* Fill the buffer from the batch file */
  Search:
   if(fgets(buffer, 255, stdin) == NULL){
    fprintf(stderr, "\nMail-Uf done processing mail file.\n");
    break;
   EndIf

   else{

    /* Rewind it */
    fseek(stdin, 0L, SEEK_SET);

    /* Figure out how long it is */
    i = filelength(fileno(stdin));

   EndElse

   /* Now read and process the buffer */
   if(i > ALLOC_SIZE){

    count = fread(cpt, sizeof(char), ALLOC_SIZE, stdin);

    /* Make sure we read as many bytes as we asked for */
    if(count != ALLOC_SIZE){
     fprintf(stderr, "ERROR: reading stdin - message CORRUPT?\n");

     fprintf(logfp, "!CORRUPT??\n");
     exit(2);
    EndIf


    result = DoMailMessage(cpt, count, logfp,
	     ascii_time,
         origin_node, &fnode, &tnode, &onode,
	     firstmailptr, MsgHeader, i, to_uucp_address, who);


   EndIf

   else{
    /* Will fit in buffer - do it all at once */
    count = fread(cpt, sizeof(char), (unsigned int)i, stdin);

    /* Make sure we read as many bytes as we asked for */
    if(count != i){
     fprintf(stderr, "ERROR: reading stdin - message CORRUPT?\n");
     fprintf(logfp, "!CORRUPT??\n");
     exit(2);
    EndIf

    Process_Message:
    result = DoMailMessage(cpt, count, logfp,
	     ascii_time,
         origin_node, &fnode, &tnode, &onode,
	     firstmailptr, MsgHeader, i, to_uucp_address, who);

   EndElse
 EndWhile

  /* Output final 3 bytes of NULL for end of packet */
  fputc(0, stdout);
  fputc(0, stdout);
  fputc(0, stdout);

  /* Close 'em out */
  fclose(stdout);
  fclose(stdin);

  t = time(NULL);
  fprintf(logfp, "$MAIL-UF STOP @TM=%s", ctime(&t));
  fclose(logfp);

  if(result == 0)
   exit(1);
  else
   exit(0);

EndMain

/*****************************************************/

/**/
/* Output the message */
/**/
int DoMailMessage(char *cpt, int count, FILE *logfp,
	     char *ascii_time,
	     char *origin_node,
	     struct nodeaddr *fnode,
	     struct nodeaddr *tnode,
         struct nodeaddr *onode,
         struct mail *firstmailptr,
	     struct MsgHeader *MsgHeader,
	     unsigned long length,
	     char *to_uucp_address,
	     char *who)

{
 char *tmp, *from;
 char *start_of_message, *end_of_message;
 int flag, header_length;
 static char dumped_group[128];
 char whoto[32];
 char domainto[64];
 struct mail *mailptr;
 struct Str Str;
 struct nodeaddr znode, naddr;
 char *TmpStr, *TmpTok;

   start_of_message = FindMessageStart(cpt, count);

   if(start_of_message == NULL){
    fprintf(stderr, "ERROR: Message body seems to be corrupt\n");
    fprintf(logfp, "!FindMessageStart()\n");
    exit(15);
   EndIf

   header_length = start_of_message - cpt;
   end_of_message = cpt + count;

   /* Now find the proper address to send it to */
   mailptr = firstmailptr;
   flag = 0;

   do{

    /* Search the MAILUF lines for a match */
    if(flag == 0){
     if(strcmpi(to_uucp_address, mailptr->uucpname) == 0){
      fprintf(stderr, "Found map for %s in MailUf.\n", to_uucp_address);
      fprintf(logfp, "+MAILMAP %s %s\n", mailptr->uucpname, mailptr->fidoname);
      strcpy(whoto, mailptr->fidoname);
      flag++;
     }
     else{
      TmpStr = strdup(to_uucp_address);
      TmpTok = strtok(TmpStr, "@");

      if(strchr(to_uucp_address, '@') != NULL && TmpTok != NULL){
       /* This is of form name@site so TmpTok points to name */
       TmpTok = strtok(NULL,"@");
       if(TmpTok != NULL){
        if((strcmpi(TmpTok, mailptr->uucpname) == 0) &&
           (mailptr->fidoname == NULL)){

     /* !!! */
	 strcpy(whoto, TmpStr);
	 if((TmpTok = strchr(whoto, '.')) != NULL)
	  *TmpTok = ' ';
	 if((TmpTok = strchr(whoto, '!')) != NULL)
	  *TmpTok = ' ';
     if((TmpTok = strchr(whoto, '_')) != NULL)
      *TmpTok = ' ';

     fprintf(stderr, "Found map for %s in MailUf.\n", to_uucp_address);
     fprintf(logfp, "+GATEMAIL %s : '%s'\n", mailptr->uucpname, whoto);
	 flag++;
	}
       }
      }
      else{  /* It's not name@site, maybe it's site!name */
       strcpy(TmpStr, to_uucp_address);
       TmpTok = strtok(TmpStr, "!");

       if(strchr(to_uucp_address, '!') != NULL && TmpTok != NULL){
	if((strcmpi(TmpTok, mailptr->uucpname) == 0) &&
	   (mailptr->fidoname == NULL)){

     /* !!! */
	 strcpy(whoto, strtok(NULL, "!"));
	 if((TmpTok = strchr(whoto, '.')) != NULL)
	  *TmpTok = ' ';
	 if((TmpTok = strchr(whoto, '!')) != NULL)
	  *TmpTok = ' ';
     if((TmpTok = strchr(whoto, '_')) != NULL)
      *TmpTok = ' ';

	 fprintf(stderr, "Found map for %s in MailUf.\n", to_uucp_address);
     fprintf(logfp, "+GATEMAIL %s : '%s'\n", mailptr->uucpname, whoto);
	 flag++;
	}
       }
      }
      free(TmpStr);
     }
    }

    if(flag == 1){
     /* Send to the right node address */
     if(VerifyNode(mailptr->system, &znode) == -1){
      fprintf(stderr, "ERROR: Node address %s for TO_NODE is invalid\n", mailptr->system);
      fprintf(logfp, "!TO_NODE/%s\n", mailptr->system);
      exit(10);
     }
    }

     if(flag > 0){
      MsgHeader->DestNode = znode.node;
      MsgHeader->DestNet  = znode.net;

      /* Write the message header */
      fwrite(MsgHeader, sizeof(struct MsgHeader), 1, stdout);

      /* Switch the file to text mode */
      fflush(stdout); /* Flush it out first */
      setmode(fileno(stdout), O_TEXT); /* Set it to Text mode */

      /* Write the DATE: field */
      /* Use the current message date if it can be found */
      /* Otherwise, use the current system date */
      if(FindDate(cpt, header_length, &Str) == NULL)
       fprintf(stdout, "%s%c", ascii_time, 0);
      else{
       fwrite(Str.cpt, sizeof(char), Str.cnt, stdout);
       fputc(0, stdout);
      EndElse

      /* Write the TO: field */
      strlwr(whoto);
      TmpStr = strdup(whoto);
      *whoto = toupper(*whoto);
      TmpTok = strtok(TmpStr, " ");
      if((TmpTok = strtok(NULL, " ")) != NULL)
        *(whoto + (TmpTok-TmpStr)) = toupper(*(whoto + (TmpTok-TmpStr)));
      free(TmpStr);
      fputs(whoto, stdout);
      fputc(0, stdout);

      /* Write the FROM: field */
      fputs(who, stdout);
      fputc(0, stdout);

      /* Write the SUBJECT: Field */
      if(FindSubject(cpt, header_length, &Str) != NULL){
       fwrite(Str.cpt, sizeof(char), Str.cnt, stdout);
       fputc(0, stdout);
      }
      else{
       fprintf(stdout, " ");
       fputc(0, stdout);
      }

      /* Write a ^AINTL line if required */
      if(znode.zone != onode->zone)
       fprintf(stdout, "%cINTL %d:%d/%d %d:%d/%d\n", 0x01,
                                                     znode.zone,
                                                     znode.net,
                                                     znode.node,
                                                     onode->zone,
                                                     onode->net,
                                                     onode->node);

      /* Write a ^ATOPT line if required */
      if(znode.point != 0)
       fprintf(stdout,"%cTOPT %d\n", 0x01, znode.point);

      /* Write a ^AFMPT line if required */
      if(onode->point != 0)
       fprintf(stdout,"%cFMPT %d\n", 0x01, onode->point);

      /* Write out the UseNet headers */
      WriteUseNetHeadersMail(headers, cpt, header_length, &Str);

      /* Write out the message body */
      fwrite(start_of_message, sizeof(char),
	     end_of_message - start_of_message, stdout);

      /* If it is a long message, then copy out the rest of the text */
      if(length){

       /* Print a warning just in case */
	fprintf(stderr, "\r** NOTE - (%8ld byte) Mail Message was output in packet **\n",
			 length);
	fprintf(logfp, "+MAIL <%8ld>\n", length);

       length -= count;
       while(length){
	count = fread(cpt, sizeof(char),
		     (length > ALLOC_SIZE) ? ALLOC_SIZE : length, stdin);

	/* Make sure we read as many bytes as we asked for */
	if(count != ((length > ALLOC_SIZE) ? ALLOC_SIZE : length)){
	 fprintf(stderr, "ERROR: reading stdin - message CORRUPT?\n");
	EndIf

	fwrite(cpt, sizeof(char), count, stdout);
	length -= count;
       EndWhile
      EndIf

      fprintf(stdout, "\n--- Mapped by %s (Mail-Uf %s) @ %s\n",
		      who, MUVERSION, origin_node);
      fprintf(stdout, "%cPID: %s\n", 0x01, PIDVERSION);

      /* Switch back to binary mode on file */
      fflush(stdout);
      setmode(fileno(stdout), O_BINARY);

      break;
     EndIf

     if(flag == 0){
      /* Point to the next record */
      mailptr = mailptr->next;
      if(mailptr == NULL){
       flag = resolve(to_uucp_address, &znode, whoto, domainto);
       if(flag == 2){
        fprintf(stderr, "Generated address for: %s.\n", to_uucp_address);
        fprintf(logfp, "+FIDO %s %s\n", to_uucp_address, whoto);
       }
       else if(flag == 3){
        fprintf(logfp, "+LOCAL %s %s@", to_uucp_address, whoto);
        fprintf(stderr, "Generated LOCAL address for: %s at ", whoto);
        znode.zone  = tnode->zone;
        znode.net   = tnode->net;
        znode.node  = tnode->node;
        znode.point = tnode->point;
        fprintf(stderr, "%d:%d/%d.%d\n", znode.zone, znode.net,
					 znode.node, znode.point);
        fprintf(logfp, "%d:%d/%d.%d\n", znode.zone, znode.net,
				      znode.node, znode.point);
       }
      }
    }

   } while(flag >= 0);

   if(flag == -1){
    fprintf(stderr, "Couldn't map mail message for address: %s.\n",
		      to_uucp_address);
    fprintf(logfp, "+NOMAP %s\n", to_uucp_address);
    return(-1);
   EndIf

   return(0);
}

/**/
/* Find the start of message body by looking for first LFLF */
/**/

char * pascal FindMessageStart(char *cptr, int count)
{
 static char *pt;

 while((pt = (char *)memchr(cptr, LF, count)) != NULL){

  if(*(pt + 1) == LF){
   pt += 2;
   return pt;
  EndIf
  else{
   count -= (pt - cptr + 1);
   cptr = ++pt;
  EndElse

 EndWhile

 return NULL;
}

/**/
/* Determine who message in the current buffer is to */
/**/

char * pascal FindToUser(char *cptr, int count, struct Str *Str)
{
 int flag1;
 static char *pt;
 char *ept, *tpt;

 pt = FindHeader(cptr, count, "To:");
 if(pt != NULL){

  /* Now get the end of the line */
  ept = memchr(pt, '\n', count);
  if(ept == NULL){
   fprintf(stderr, "ERROR: Can't find EndOfLine in FindToUser()\n");
   exit(2);
  EndIf

  /* Look for a '(' */
  flag1 = 0;
  tpt = memchr(pt, LPAREN, ept - pt);
  if(tpt != NULL){
   pt = ++tpt;
   /* And find the ')' */
   tpt = memchr(pt, RPAREN, ept - pt);
   if(tpt != NULL){
    flag1++;
    ept = tpt;
   EndIf
  EndIf


  /* Not found - return NULL */
  if(flag1 == 0)
   pt = NULL;

 EndIf
 else{
  fprintf(stderr, "WARNING: 'From:' not found in this message.\n");
  pt = NULL;
 EndElse

 /* Truncate if too long */
 if((ept - pt) >35)
  ept = pt + 35;

 /*
 if(Debug)
  if(pt != NULL){
   fprintf(stderr, "\nFrom is: ");
   fwrite(pt, sizeof(char), ept - pt, stderr);
   fprintf(stderr, "\n");
  EndIf
  else{
   fprintf(stderr, "From is: NULL\n");
  EndElse
 */

 Str->cpt = pt;
 Str->cnt = ept - pt;

 return pt;

}


/**/
/* Determine what is the SUBJECT in the current buffer */
/**/

char * pascal FindSubject(char *cptr, int count, struct Str *Str)
{
 static char *pt;
 char *ept, *tpt;

 pt = FindHeader(cptr, count, "Subject:");

 if(pt != NULL){
  /* Now get the end of the field */
  ept = memchr(pt, '\n', count);
  if(ept == NULL){
   fprintf(stderr, "WARNING: Can't find EndOfLine in FindSubject()\n");
   return(NULL);
  EndIf

 EndIf
 else{
  fprintf(stderr, "WARNING: 'Subject:' not found in this message.\n");
  return(NULL);
 EndElse

 /* Truncate if Subject is too long */
 if((ept - pt) > 71)
  ept = pt + 71;

 /*
 if(Debug)
  fprintf(stderr,"\nSubject is: ");
  fwrite(pt, sizeof(char), ept - pt, stderr);
  fprintf(stderr, "\n");
 */

 Str->cpt = pt;
 Str->cnt = ept - pt;

 return pt;

}

/**/
/* Find the start of the field after the requested header */
/**/

char * pascal FindHeader(char *cptr, int count, char *findme)
{
 int flag;
 static char *pt;

 flag = 0;
 while((pt = (char *)memchr(cptr, *findme, count)) != NULL){
  if(memicmp(pt, findme, strlen(findme)) == 0){
   /* Found it */
   flag++;
   break;
  EndIf
  else{
   count -= (pt - cptr + 1);
   cptr = ++pt;
  EndElse
 EndWhile

 if(flag){
  /* Now get to the character after the ':' */
  /* pt points to first char of string now */
  pt++ ;
  while(!isspace(*pt)){
   pt++;
  EndWhile
  pt++;
 EndIf
 else
  pt = NULL;

 return pt;

}

/**/
/* Write out the UseNet Headers to output */
/**/

void pascal WriteUseNetHeadersMail(unsigned int headers, char *cpt,
			       int header_length, struct Str *Str)
{
			 
      if(headers & 0x0020){
       if(FindWholeFrom(cpt, header_length, Str) != NULL){
        if(headers & 0x2000) fputc(0x01, stdout);
        fputs("From: ", stdout);
        fwrite(Str->cpt, sizeof(char), Str->cnt, stdout);
        fputc('\r', stdout);
       EndIf
      EndIf

      if(headers & 0x0010){
       if(FindPath(cpt, header_length, Str) != NULL){
        if(headers & 0x1000) fputc(0x01, stdout);
        fputs("Path: ", stdout);
        fwrite(Str->cpt, sizeof(char), Str->cnt, stdout);
        fputc('\r', stdout);
       EndIf
      EndIf

      if(headers & 0x0004){
       if(FindSubject(cpt, header_length, Str) != NULL){
        if(headers & 0x0400) fputc(0x01, stdout);
        fputs("Subject: ", stdout);
        fwrite(Str->cpt, sizeof(char), Str->cnt, stdout);
        fputc('\r', stdout);
       EndIf
      EndIf

      if(headers & 0x0002){
       if(FindMessageID(cpt, header_length, Str) != NULL){
        if(headers & 0x0200) fputc(0x01, stdout);
        fputs("Message-ID: ", stdout);
        fwrite(Str->cpt, sizeof(char), Str->cnt, stdout);
        fputc('\r', stdout);
       EndIf
      EndIf

      if(headers & 0x0001){
       if(FindWholeDate(cpt, header_length, Str) != NULL){
        if(headers & 0x0100) fputc(0x01, stdout);
        fputs("Date: ", stdout);
        fwrite(Str->cpt, sizeof(char), Str->cnt, stdout);
        fputc('\r', stdout);
       EndIf
      EndIf

      /* Put out an extra CR to separate headers from message body */
      if(headers & 0xff00)
       fputc('\r', stdout);
}

/**/
/* Determine who message in the current buffer is from */
/**/

char * pascal FindFrom(char *cptr, int count, struct Str *Str)
{
 int flag1;
 static char *pt;
 char *ept, *tpt;

 pt = FindHeader(cptr, count, "From:");
 if(pt != NULL){

  /* Now get the end of the line */
  ept = memchr(pt, '\n', count);
  if(ept == NULL){
   fprintf(stderr, "ERROR: Can't find EndOfLine in FindFrom()\n");
   exit(2);
  EndIf

  /* Look for a '(' */
  flag1 = 0;
  tpt = memchr(pt, LPAREN, ept - pt);
  if(tpt != NULL){
   pt = ++tpt;
   /* And find the ')' */
   tpt = memchr(pt, RPAREN, ept - pt);
   if(tpt != NULL){
    flag1++;
    ept = tpt;
   EndIf
  EndIf


  /* Not found - return NULL */
  if(flag1 == 0)
   pt = NULL;

 EndIf
 else{
  fprintf(stderr, "WARNING: 'From:' not found in this message.\n");
  pt = NULL;
 EndElse

 /* Truncate if too long */
 if((ept - pt) >35)
  ept = pt + 35;

 /*
 if(Debug)
  if(pt != NULL){
   fprintf(stderr, "\nFrom is: ");
   fwrite(pt, sizeof(char), ept - pt, stderr);
   fprintf(stderr, "\n");
  EndIf
  else{
   fprintf(stderr, "From is: NULL\n");
  EndElse
 */

 Str->cpt = pt;
 Str->cnt = ept - pt;

 return pt;

}

/**/
/* Determine the date of the current message */
/**/

char * pascal FindDate(char *cptr, int count, struct Str *Str)
{
 int i, j, k, flag, flag1;
 static char *pt, *dt;
 static char GoodDate[32], HoldDate[32];
 char *ept, *tpt;

 pt = FindHeader(cptr, count, "Date:");

 if(pt != NULL){

  /* Now get the end of the field */
  ept = memchr(pt, '\n', count);
  if(ept == NULL){
   fprintf(stderr, "ERROR: Can't find EndOfLine in FindDate()\n");
   return NULL;
  EndIf

  /* Look for the first Digit */
  for(i=0; i<32; i++){
   if(isdigit(*(pt+i))){
    dt = pt + i;
    break;
   EndIf
  EndFor

  tpt = memchr(dt, COLON, ept - dt);
  if(tpt != NULL){
   ept = tpt + 6;
  EndIf

 EndIf
 else{
  fprintf(stderr, "WARNING: 'Date:' not found in this message.\n");
  return NULL;
 EndElse

 if(( ept - dt) > 22)
  return NULL;

 strncpy(HoldDate, dt, ept-dt);
 HoldDate[ept-dt] = '\0';

 /* Get the day into GoodDate */
 tpt = strtok(HoldDate, " ");
 if(atoi(tpt) == 0 || atoi(tpt) > 31) return NULL;
 strcpy(GoodDate, tpt);
 strcat(GoodDate, " ");

 /* Get the Month */
 tpt = strtok(NULL, " ");
 if(strlen(tpt) != 3) return NULL;
 strcat(GoodDate, tpt);
 strcat(GoodDate, " ");

 /* Get the year */
 tpt = strtok(NULL, " ");
 if(atoi(tpt) > 1900)
  tpt += 2;
 if(strlen(tpt) != 2) return NULL;
 strcat(GoodDate, tpt);
 strcat(GoodDate, " ");

 /* Get the time */
 tpt = strtok(NULL, " ");
 if(strlen(tpt) != 8) return NULL;
 strcat(GoodDate, tpt);

 Str->cpt = GoodDate;
 Str->cnt = strlen(GoodDate);

 return GoodDate;

}


/**/
/* Get the whole date string for current message */
/**/

char * pascal FindWholeDate(char *cptr, int count, struct Str *Str)
{
 int i, j, k, flag, flag1;
 static char *pt;
 char *ept, *dt, *tpt;

 pt = FindHeader(cptr, count, "Date:");

 if(pt != NULL){

  /* Now get the end of the field */
  ept = memchr(pt, '\n', count);
  if(ept == NULL){
   fprintf(stderr, "ERROR: Can't find EndOfLine in FindWholeDate()\n");
   exit(2);
  EndIf
 EndIf
 else{
  fprintf(stderr, "WARNING: Date: not found in this message.\n");
  return NULL;
 EndElse

  Str->cpt = pt;
  Str->cnt = ept - pt;

  return pt;
}

/**/
/* Get the whole Message-ID: string */
/**/

char * pascal FindMessageID(char *cptr, int count, struct Str *Str)
{
 static char *pt;
 char *ept, *tpt;

 pt = FindHeader(cptr, count, "Message-ID:");
 if(pt != NULL){

  /* Now get the end of the line */
  ept = memchr(pt, '\n', count);
  if(ept == NULL){
   fprintf(stderr, "ERROR: Can't find EndOfLine in FindMessageID()\n");
   exit(2);
  EndIf
 EndIf
 else{
  fprintf(stderr, "WARNING: Message-ID: not found in this message.\n");
  return NULL;
 EndElse

  Str->cpt = pt;
  Str->cnt = ept - pt;

  return pt;
}

/**/
/* Get the whole From: string */
/**/

char * pascal FindWholeFrom(char *cptr, int count, struct Str *Str)
{
 static char *pt;
 char *ept, *tpt;

 pt = FindHeader(cptr, count, "From:");
 if(pt != NULL){

  /* Now get the end of the line */
  ept = memchr(pt, '\n', count);
  if(ept == NULL){
   fprintf(stderr, "ERROR: Can't find EndOfLine in FindWholeFrom()\n");
   exit(2);
  EndIf
 EndIf
 else{
  fprintf(stderr, "WARNING: From: not found in this message.\n");
  return NULL;
 EndElse

  Str->cpt = pt;
  Str->cnt = ept - pt;

  return pt;
}

/**/
/* Get the whole Path: string */
/**/

char * pascal FindPath(char *cptr, int count, struct Str *Str)
{
 static char *pt;
 char *ept, *tpt;

 pt = FindHeader(cptr, count, "Path:");
 if(pt != NULL){

  /* Now get the end of the line */
  ept = memchr(pt, '\n', count);
  if(ept == NULL){
   fprintf(stderr, "ERROR: Can't find EndOfLine in FindPath()\n");
   exit(2);
  EndIf
 EndIf
 else{
  fprintf(stderr, "WARNING: Path: not found in this message\n");
  return NULL;
 EndElse

  Str->cpt = pt;
  Str->cnt = ept - pt;

  return pt;

}

/**/
/* Figure out which headers to output and which to hide */
/**/

unsigned int WhichHeaders(char *s)
{
 static unsigned int tmp, factor;
 int i;

 if(strlen(s) != 16){
  fprintf(stderr, "ERROR: In 'HEADERS YES <code>' -  <code> must specify exactly 16 fields.\n");
  exit(2);
 EndIf

 tmp = 0;
 factor = 1;

 for(i=15; i>=0; i--){
  if(*(s+i) == '1'){
   tmp += factor;
  EndIf
  factor *= 2;
 EndFor

 return tmp;
}
