// MECHILD.CXX - Multi-Edit MDI Child Window class
//               (C) Sturmer Hauss Corp 1991

#define INCL_TIER_TYPES                    // data types like LONG, HWND, HANDLE
#define INCL_TIER_T1OBJECT                 // base class of the TIER1 Library and Window and Overlapped classes 
#define INCL_TIER_WINDOW                   // TIER Window class
   #define INCL_TIER_WINDOWMSGS            // MS-Window compatible message #defines
   #define INCL_TIER_WINDOWSTYLES          // MS-Window compatible window style #defines
   #define INCL_TIER_MDIFRAME              // MDI Frame window class  
   #define INCL_TIER_MDICLIENT             // MDI Client window class
   #define INCL_TIER_MDICHILD              // MDI Child window class
      #define INCL_TIER_DIALOG             // Dialog class
         #define INCL_TIER_CONTROLWINDOW   // ControlWindow class
         #define INCL_TIER_EDIT            // EditWindow class
         #define INCL_TIER_EDITMESSAGES    // EditWindow window messages
         #define INCL_TIER_MODALDLG        // ModalDlg class
#define INCL_TIER_MENU                     // Menu class
#define INCL_TIER_ICON                     // Icon class
#define INCL_TIER_MEMORY                   // LocalMemory class
#define INCL_TIER_MSGBOX                   // MsgBox class
#define INCL_TIER_CURSOR                   // Cursor class
#define INCL_TIER_DC                       // DC classes  
#define INCL_TIER_GDIFONT                  // GdiFont class  
   #define INCL_TIER_GDISTYLEDFONT         // GdiStyledFont class  
#define INCL_TIER_PRINTER                  // Printer class
#define INCL_TIER_RESOURCE                 // Resource class



// Module includes .......................................................
#ifdef _TURBOL
extern "C"        {
#define __MSC
#endif // _TURBOL

#ifdef _GLOCK20L
extern "C"        {
#endif // _GLOCK20L

#ifdef _ZTC201L
extern "Pascal"   {
#endif // _ZTC201L

#include <stdio.h>       // sprintf(), fprintf()

#ifdef _GLOCK20L
                  }
#endif // _GLOCK20L

#ifdef _ZTC201L
                  }
#endif // _ZTC201L

#ifdef _TURBOL
                  }
#endif // _TURBOL


#include <tier1.hxx>    // TIER Library .hxx's
#include "defines.h"    // Our modules common #defines for the .RC file and here.
#include "savasdlg.hxx" // SaveAsDlg class
#include "edparser.hxx" // EditParser class
#include "srchdlg.hxx"  // SearchDlg class
#include "mechild.hxx"  // MEChildWindow class




//////////////////////////////////////////////////////////////////////////////
// MEChildWindow class
//////////////////////////////////////////////////////////////////////////////

MEChildWindow :: FAR MEChildWindow( MDIFrameWindow FAR *lpFrameWindow,
                                    MDIClientWindow FAR *lpClientWindow,
                                    HANDLE hPrevInstance, LPSTR lpszClassName,
                                    BOOL bUseFixedFont, BOOL bStartIconic, 
                                    HANDLE hPrinterSetupData ) : 
                                    ( lpClientWindow, hPrevInstance, lpszClassName )
   {
      int    x, y, nWidth, nHeight;

   // Init IVAR's
   lpFrame           = lpFrameWindow;
   lpClient          = lpClientWindow;
   lpEdit            = (EditWindow FAR *)0L;
   bCreated          = FALSE;
   bEdited           = FALSE;
   bAbort            = FALSE;
   szFileName[0]     = '\0';
   szSearch[0]       = '\0';
   szDevice[0]       = '\0';
   bCaseSensitive    = FALSE;
   nDirection        = DIR_FORWARD;
   lpszDriver        = (LPSTR)0L;
   lpszPort          = (LPSTR)0L;
   lpszTitle         = (LPSTR)0L;
   iPrinter          = 0;
   hSetupData        = 0;
   hEditData         = 0;
   bWrap             = FALSE;
   wMaxSize          = 0xFFFF;   // 64K
   bFixedFont        = bUseFixedFont;
   hFont             = 0;
   bLocalEdit        = FALSE;
   lstrcpy( szExtDevMode, "EXTDEVICEMODE" );
   GetResString( IDS_APPNAME, szAppName, DEF_APPNAMELEN );


   GetResString( IDS_UNTITLED, szMsgText, DEF_APPNAMELEN );
   WinSetWindowName( szMsgText );

   // If the parent is not iconic, set our new position
   // else set some ramdom initial size.
   if( !bStartIconic )
      {
      lpFrame->GetClient( x, y, nWidth, nHeight );
      WinSetWindowPosition( x+3, y+3, nWidth-6, nHeight-6 );
      }
   else
      {
      WinSetWindowPosition( 3, 3, 400, 350 );
      }


   if( !Create() )
      return;

   bCreated    = TRUE;

   // Initialize the device and port. 
   // if hPrinterSetupData is supplied, just
   // make a copy of it.
   if( CopyPrinterData( hPrinterSetupData ) )
      InitDevPort( FALSE );
   else  InitDevPort( TRUE );
   }







MEChildWindow :: FAR ~MEChildWindow()
   {
   // Get rid of the font we were using
   // for the editor.
   // NOTE: setting the font in this way
   //       will allow it to be deleted
   //       when going out of scope.
   if( hFont )
      {
      GdiStyledFont  font;
      font.SetHandle( hFont );
      }

   if( hSetupData )
      {
      LocalMemory    mem;
      mem.SetHandle( hSetupData );
      mem.Free();
      }
   }






LONG MEChildWindow :: FAR WinMessageHandler( HWND hWnd, unsigned iMessage, WORD wParam, LONG lParam )
   {

      int            x, y, nWidth, nHeight;


   switch( iMessage )
      {
      case  WM_CREATE:
            // This will create the edit control as well
            // as set the wrap mode to off.
            if( !SetWrap( FALSE ) )
               WinDestroy();
      break;

      case  WM_INITMENU:
            InitMenus( wParam );
      break;

      case  WM_WININICHANGE:
      case  WM_DEVMODECHANGE:
            if( !InitDevPort() )
               {
               GetResString( IDS_ERRPRINTER, szMsgText, DEF_MSGLEN );
               MsgBox( this ).Show( szMsgText, szAppName, MB_OK | MB_ICONHAND );
               }
      break;

      case  WM_SETFOCUS:
            if( lpEdit )
               lpEdit->WinSetFocus();
      break;
      
      case  WM_COMMAND:
            switch( wParam )
               {
               case  ID_EDIT:
                     switch( HIWORD( lParam ) )
                        {
                        case  EN_UPDATE:
                              bEdited = TRUE;
                        break;
      
                        case  EN_ERRSPACE:
                              ::MessageBeep(0);
                        break;
                        }
               break;

               case  IDM_FILESAVE:
                     {
                     Cursor  cursor;
                     cursor.Load( CURSOR_WAIT );
                     cursor.Set();
                     cursor.Show();
                     
                     if( szFileName[0] == '\0' )
                        {
                        return( WinSendMessage( this, WM_COMMAND, IDM_FILESAVEAS, 0L ) );
                        }

                     if( !Save() )
                        {
                        GetResString( IDS_CANTWRITE, szMsgTemp, DEF_MSGLEN );
                        wsprintf( szMsgText, szMsgTemp, szFileName );
                        MsgBox( this ).Show( szMsgText, szAppName, MB_OK | MB_ICONEXCLAMATION );
                        return( 0L );
                        }

                     bEdited = FALSE;
                     return( 1L );
                     }
               break;

               case  IDM_FILESAVEAS:
                     {
                     Cursor            cursor;
                     SaveAsDlg FAR    *lpSave;
                     int               nRet;

                     cursor.Load( CURSOR_WAIT );
                     cursor.Set();
                     cursor.Show();

                     lpSave = new SaveAsDlg( this, IDD_SAVEAS, szFileName, (LPSTR)0L );
                     nRet   = lpSave->GetResult();
                     if( nRet )
                        lpSave->GetFileName( szFileName, DEF_FILENAME_LEN );
                     delete  lpSave;

                     if( nRet )
                        {
                        if( !Save() )
                           {
                           GetResString( IDS_CANTWRITE, szMsgTemp, DEF_MSGLEN );
                           wsprintf( szMsgText, szMsgTemp, szFileName );
                           MsgBox( this ).Show( szMsgText, szAppName, MB_OK | MB_ICONEXCLAMATION );
                           return( 0L );
                           }

                        // Tell the parent to push this new file name
                        // on to the file menu for prev edits.
                        WinSendMessage( lpFrame, WM_PUSHFILE, 0, (LONG)(LPSTR)szFileName );

                        bEdited = FALSE;
                        return( 1L );
                        }
                     }
               break;

               case  IDM_FILEPRINT:
                     {
                     Cursor  cursor;
                     cursor.Load( CURSOR_WAIT );
                     cursor.Set();
                     cursor.Show();
                     
                     bAbort = FALSE;
                     Print();
                     }
               break;

               case  IDM_FILESETUP:
                     {
                     Cursor  cursor;
                     cursor.Load( CURSOR_WAIT );
                     cursor.Set();
                     cursor.Show();
                     
                     // Bring up the printer setup dialog box, and allow the
                     // user to change the settings.
                     PrinterSetup();
                     }
               break;
            
               case  IDM_FONTVARIABLE:
                     SetEditFont( FNT_VARIABLE );
                     WinSendMessage( lpFrame, WM_USEFIXEDFONT, FALSE, 0L );
               break;

               case  IDM_FONTFIXED:
                     SetEditFont( FNT_FIXED );
                     WinSendMessage( lpFrame, WM_USEFIXEDFONT, TRUE, 0L );
               break;

               case  IDM_EDITUNDO:
                     lpEdit->Undo();
               break;

               case  IDM_EDITCUT:
                     lpEdit->Cut();
               break;

               case  IDM_EDITCOPY:
                     lpEdit->Copy();
               break;

               case  IDM_EDITPASTE:
                     lpEdit->Paste();
               break;

               case  IDM_EDITCLEAR:
                     lpEdit->ReplaceSelect( "" );
               break;

               case  IDM_EDITSELECT:
                     lpEdit->SetSelect( 0, 0xFFF0 );
               break;

               case  IDM_EDITWRAP:
                     {
                     Cursor   cursor;

                     cursor.Load( CURSOR_WAIT );
                     cursor.Set();
                     cursor.Show();

                     SetWrap( !bWrap );
                     }
               break;

               case  IDM_SEARCHFIND:
                     {
                     SearchDlg  FAR   *lpDlg;
                     EditParser        ep( lpEdit, 0, DATA_GLOBAL );
                     Cursor            cursor;
                     BOOL              bRet;

                     cursor.Load( CURSOR_WAIT );
                     cursor.Set();
                     cursor.Show();
                     
                     lpDlg = new SearchDlg( this, IDD_SEARCH, szSearch,
                                            DEF_SEARCHSTR_LEN, bCaseSensitive,
                                            nDirection+1 );

                     bRet = lpDlg->GetResult();
                     if( bRet )
                        {
                        lpDlg->GetSearchText( szSearch, DEF_SEARCHSTR_LEN );
                        bCaseSensitive = lpDlg->IsCaseSensitive();

                        if( lpDlg->IsForwardSearch() )
                           nDirection = DIR_FORWARD;
                        else nDirection = DIR_BACKWARD;

                        delete lpDlg;
                        }
                     else
                        {
                        delete lpDlg;
                        return( 0L );
                        }

                     cursor.Set();
                     cursor.Show();

                     if( !ep.Find( szSearch, nDirection, bCaseSensitive ) )
                        {
                        GetResString( IDS_CANTFIND, szMsgTemp, DEF_MSGLEN );
                        wsprintf( szMsgText, szMsgTemp, szSearch );
                        MsgBox( this ).Show( szMsgText, szAppName, MB_OK | MB_ICONEXCLAMATION );
                        }
                     }
               break;

               case  IDM_SEARCHNEXT:
                     {
                     EditParser     ep( lpEdit, 0, DATA_GLOBAL );
                     Cursor         cursor;

                     cursor.Load( CURSOR_WAIT );
                     cursor.Set();
                     cursor.Show();

                     if( !ep.Find( szSearch, DIR_FORWARD, bCaseSensitive ) )
                        {
                        GetResString( IDS_CANTFIND, szMsgTemp, DEF_MSGLEN );
                        wsprintf( szMsgText, szMsgTemp, szSearch );
                        MsgBox( this ).Show( szMsgText, szAppName, MB_OK | MB_ICONEXCLAMATION );
                        }
                     }
               break;

               case  IDM_SEARCHPREV:
                     {
                     EditParser     ep( lpEdit, 0, DATA_GLOBAL );
                     Cursor         cursor;

                     cursor.Load( CURSOR_WAIT );
                     cursor.Set();
                     cursor.Show();

                     if( !ep.Find( szSearch, DIR_BACKWARD, bCaseSensitive ) )
                        {
                        GetResString( IDS_CANTFIND, szMsgTemp, DEF_MSGLEN );
                        wsprintf( szMsgText, szMsgTemp, szSearch );
                        MsgBox( this ).Show( szMsgText, szAppName, MB_OK | MB_ICONEXCLAMATION );
                        }
                     }
               break;
               }
      break;

      case  WM_SIZE:
            GetClient( x, y, nWidth, nHeight );
            if( lpEdit && !bWrap )
               {  
               lpEdit->Move( x, y, nWidth, nHeight );
               break;
               }

            // If bWrap mode on, put up a wait cursor.
            // Large files take a while for the edit to
            // reformat the text.
            if( lpEdit && bWrap && !WinIsIconic() )
               {
               Cursor   cursor;

               cursor.Load( CURSOR_WAIT );
               cursor.Set();
               cursor.Show();

               lpEdit->Move( x, y, nWidth, nHeight );
               }
      break;

      case  WM_CLOSE:
            if( Close() != IDOK )
               return( 0L );
      break;

      case  WM_DESTROY:
            if( lpEdit )
               {
               lpEdit->WinDestroy();
               delete lpEdit;
               lpEdit = (EditWindow FAR *)0L;
               }

            if( !bLocalEdit && hEditData && hEditData != GethInstance() )
               {
               GlobalMemory    mem;
               mem.SetHandle( hEditData );
               mem.Free();
               hEditData = 0;
               }

            // Tell our frame that we are dying so it can delete our
            // object.
            WinPostMessage( lpFrame, WM_CHILDKILLED, MECHILDWIN, (LONG)(LPSTR)this );

            return( 0L );
      break;
      }

   return( WinDefMDIChildMsgHandler( hWnd, iMessage, wParam, lParam ) );
   }





BOOL MEChildWindow :: FAR CopyPrinterData( HANDLE hData )
   {
      LocalMemory    mem1;
      LocalMemory    mem2;
      LPSTR          lpszTemp1;
      LPSTR          lpszTemp2;
      int            nSize;
      int            i;

   if( !hData )
      return( FALSE );

   mem1.SetHandle( hData );
   nSize = mem1.Size();

   mem2.Allocate( nSize );
   lpszTemp1 = (LPSTR)mem1.Lock();
   lpszTemp2 = (LPSTR)mem2.Lock();

   if( !lpszTemp1 || !lpszTemp2 )
      {
      mem1.Unlock();
      mem2.Unlock();
      mem2.Free();
      GetResString( IDS_ERRPRNALLOC, szMsgText, DEF_MSGLEN );
      MsgBox(this).Show( szMsgText, szAppName, MB_OK | MB_ICONEXCLAMATION );
      return( FALSE );
      }

   for( i = 0; i < nSize; i++ )
      lpszTemp2[i] = lpszTemp1[i];

   mem1.Unlock();
   mem2.Unlock();

   hSetupData = mem2.GetHandle();

   return( TRUE );
   }






HANDLE MEChildWindow :: FAR SetEditFont( int nType )
   {
   GdiStyledFont  font;
   HANDLE         hTemp;

   if( nType == FNT_FIXED )
      {
      font.Create( -1,0,0,0,FONT_FW_THIN,0,0,0,FONT_ANSI_CHARSET,
                   FONT_OUT_DEFAULT_PRECIS, FONT_CLIP_DEFAULT_PRECIS,
                   FONT_DEFAULT_QUALITY,FONT_FIXED_PITCH,
                   "Courier" );
      bFixedFont = TRUE;
      }
   else
      {
      font.Create( -1,0,0,0,FONT_FW_THIN,0,0,0,FONT_ANSI_CHARSET,
                   FONT_OUT_DEFAULT_PRECIS, FONT_CLIP_DEFAULT_PRECIS,
                   FONT_DEFAULT_QUALITY,FONT_VARIABLE_PITCH,
                   "Helv" );
      bFixedFont = FALSE;
      }

   // Since GdiFont deletes it's font
   // when it goes out of scope, use
   // that knowledge to delete the 
   // previous font by using the 
   // SetHandle() member.
   hTemp = font.GetHandle();
   if( hTemp )
      {
      font.SetHandle( hFont );
      hFont = hTemp;
      }

   if( lpEdit && hFont )
      WinSendMessage( lpEdit, WM_SETFONT, hFont, 0L );

   return( hFont );
   }






int MEChildWindow :: FAR GetResString( WORD wResID, LPSTR lpszStringBuff, int nMaxChars )
   {
      Resource    res( this );

   return( res.LoadStr( wResID, lpszStringBuff, nMaxChars ) );
   }

