// MECHILD1.CXX - Multi-Edit MDI Child Window class
//                (C) Sturmer Hauss Corp 1991

#define INCL_TIER_TYPES                    // data types like LONG, HWND, HANDLE
#define INCL_TIER_T1OBJECT                 // base class of the TIER1 Library and Window and Overlapped classes 
#define INCL_TIER_WINDOW                   // TIER Window class
   #define INCL_TIER_WINDOWMSGS            // MS-Window compatible message #defines
   #define INCL_TIER_WINDOWSTYLES          // MS-Window compatible window style #defines
   #define INCL_TIER_MDIFRAME              // MDI Frame window class  
   #define INCL_TIER_MDICLIENT             // MDI Client window class
   #define INCL_TIER_MDICHILD              // MDI Child window class
      #define INCL_TIER_DIALOG             // Dialog class
         #define INCL_TIER_CONTROLWINDOW   // ControlWindow class
         #define INCL_TIER_EDIT            // EditWindow class
            #define INCL_TIER_EDITSTYLES   // ES_ edit styles
         #define INCL_TIER_MODELESSDLG     // ModelessDlg class
#define INCL_TIER_GDITEXT                  // GdiText class 
#define INCL_TIER_DC                       // DC classes  
#define INCL_TIER_FILE                     // File class
#define INCL_TIER_MEMORY                   // LocalMemory class
#define INCL_TIER_MSGBOX                   // MsgBox class
#define INCL_TIER_PROFILE                  // Profile class
#define INCL_TIER_PRINTER                  // Printer class
#define INCL_TIER_DEVCAPS                  // DevCaps class
#define INCL_TIER_CURSOR                   // Cursor class
#define INCL_TIER_MENU                     // Menu class
#define INCL_TIER_EXTDEVMODE               // ExtDevMode class
#define INCL_TIER_CLIPBOARD                // Clipboard class



// Module includes .......................................................
#ifdef _TURBOL
extern "C"        {
#define __MSC
#endif // _TURBOL

#ifdef _GLOCK20L
extern "C"        {
#endif // _GLOCK20L

#ifdef _ZTC201L
extern "Pascal"   {
#endif // _ZTC201L

#include <stdio.h>       // sprintf(), fprintf()

#ifdef _GLOCK20L
                  }
#endif // _GLOCK20L

#ifdef _ZTC201L
                  }
#endif // _ZTC201L

#ifdef _TURBOL
                  }
#endif // _TURBOL


#include <tier1.hxx>    // TIER Library .hxx's
#include "defines.h"    // Our modules common #defines for the .RC file and here.
#include "mechild.hxx"  // MEChildWindow class
#include "meprint.hxx"  // MEPrinter class
#include "printdlg.hxx" // PrintDlg class




// Module defines
// Printer availibility
#define  NO_PRINTER                   0
#define  PRINTER_AVAIL                1
#define  EXTDEVMODE_PRINTER_AVAIL     2





int MEChildWindow :: FAR Open( LPSTR lpszFileName )
   {
      FileIO            file;
      GlobalMemory      gMem( GMEM_MOVEABLE | GMEM_ZEROINIT | GMEM_NOT_BANKED | GMEM_DDESHARE );
      LocalMemory       lMem;
      LONG              lLen, lPos;
      LPSTR             lpszBuffer;
      int               nChunk;
      LONG              lCnt;


   // Update the caption bar
   SetFileName( lpszFileName );
   WinSetWindowName( szFileName );

   if( !lpEdit )
      return( 0 );

   if( !hEditData )
      return( 0 );

   // Open the file. If it does not exist return FALSE
   if( file.Open( lpszFileName, OFILE_READ ) == -1 )
      return( 0 );


   // Check the file length to make sure it is less than
   // 64K.
   lLen = file.Seek( 0, SFILE_END );
   if( lLen >= (LONG)wMaxSize )
      {
      file.Close();
      GetResString( IDS_CANTLOAD, szMsgTemp, DEF_MSGLEN );
      wsprintf( szMsgText, szMsgTemp, lpszFileName );
      MsgBox( this ).Show( szMsgText, szAppName, MB_OK | MB_ICONHAND );
      return( -1 );
      }


   // Allocate a 64K buffer for the file if
   // we are using global memory.
   if( wMaxSize == 0xFFFF )
      {
      gMem.Allocate( 0xFFFF );
      lpszBuffer = (LPSTR)gMem.Lock();
      if( !lpszBuffer )
         {
         file.Close();
         return( 0 );
         }
      }
   else
      {
      hEditData = lpEdit->GetDataHandle();
      lMem.SetHandle( hEditData );
      lMem.ReAlloc( lLen+20L );
      lpszBuffer = (LPSTR)lMem.Lock();
      if( !lpszBuffer )
         {
         file.Close();
         return( 0 );
         }
      }


   // Seek to the beginning of the file and
   // read the data.
   file.Seek( 0, SFILE_SET );

   lCnt     = 0L;
   lPos     = 0L;
   nChunk   = 32766;

   if( lLen < nChunk )
      nChunk = (int)lLen;

   while( lPos < lLen )
      {
      lCnt += file.Read( &lpszBuffer[ lPos ], nChunk );
      lPos += nChunk;
      if( (lPos + (LONG)nChunk) > lLen )
         nChunk = (int)(lLen-lPos);
      if( nChunk <= 0 )
         break;
      }

   file.Close();

   // Null terminate the buffer
   lpszBuffer[lLen] = '\0';

   lpEdit->SetText( lpszBuffer );
   lpEdit->SetTextLimit( (WORD)wMaxSize );


   if( wMaxSize == 0xFFFF )
      {
      gMem.Unlock();
      gMem.Free();
      }
   else
      {
      lMem.Unlock();
      }

   return( 1 );
   }







BOOL MEChildWindow :: FAR Save( void )
   {
      FileIO            fileIO;
      File              file;
      GlobalMemory      mem( GMEM_MOVEABLE | GMEM_ZEROINIT | GMEM_NOT_BANKED | GMEM_DDESHARE );
      LocalMemory       lmem;
      LONG              lLen, lPos;
      LPSTR             lpszBuffer;
      int               nChunk;
      LONG              lCnt;
      OFSTRUCT          of;


   if( !lpEdit )
      return( FALSE );

   // Open the file. If it does not exist return FALSE
   if( fileIO.Open( szFileName, &of, OFILE_WRITE | OFILE_CREATE ) == -1 )
         {
         mem.Unlock();
         return( FALSE );
         }

   // Seek to the beginning of the file and
   // write the data.
   fileIO.Seek( 0, SFILE_SET );
   lLen = (LONG)lpEdit->GetTextLen();

   // Allocate a buffer for the file.
   mem.Allocate( lLen+10 );
   lpszBuffer = (LPSTR)mem.Lock();
   if( !lpszBuffer )
      {
      fileIO.Close();
      return( FALSE );
      }

   lpEdit->GetText( lpszBuffer, (WORD)lLen+5 );

   lCnt     = 0L;
   lPos     = 0L;
   nChunk   = 32766;

   if( lLen < nChunk )
      nChunk = (int)lLen;

   while( lPos < lLen )
      {
      lCnt += fileIO.Write( &lpszBuffer[ lPos ], nChunk );
      lPos += nChunk;
      if( (lPos + (LONG)nChunk) > lLen )
         nChunk = (int)(lLen-lPos);
      if( nChunk <= 0 )
         break;
      } 

   fileIO.Close();

   mem.Unlock();
   mem.Free();

   // Update the caption bar.
   WinSetWindowName( szFileName );

   return( TRUE );
   }






int MEChildWindow :: FAR GetFileName( LPSTR lpszFileName, int nMaxChars )
   {
      int   i;

   for( i = 0; i < nMaxChars; i++ )
      {
      lpszFileName[i] = szFileName[i];
      if( szFileName[i] == '\0' )
         return( i-1 );
      }

   lpszFileName[i+1] = '\0';

   return( i );
   }






void MEChildWindow :: FAR SetFileName( LPSTR lpszFileName )
   {
   lstrcpy( szFileName, lpszFileName );
   }





int MEChildWindow :: FAR GetSearchString( LPSTR lpszSearchStr, int nMaxChars )
   {
      int   i;

   for( i = 0; i < nMaxChars; i++ )
      {
      lpszSearchStr[i] = szSearch[i];
      if( szSearch[i] == '\0' )
         return( i-1 );
      }

   lpszSearchStr[i+1] = '\0';

   return( i );
   }



void MEChildWindow :: FAR SetSearchString( LPSTR lpszSearchStr )
   {
   lstrcpy( szSearch, lpszSearchStr );
   }





int MEChildWindow :: FAR Close( void )
   {
      static char szTemp[280];
      int         nRet;

   // If the user has edited the file, ask if they want to
   // save it.
   if( bEdited )
      {
      if( szFileName[0] )
         {
         GetResString( IDS_CLOSESAVE, szMsgTemp, DEF_MSGLEN );
         wsprintf( szMsgText, szMsgTemp, szFileName );
         nRet = MsgBox( this ).Show( szMsgText, szAppName, MB_YESNOCANCEL | MB_ICONQUESTION );
         }
      else
         {
         GetResString( IDS_CLOSESAVEUNTIT, szMsgText, DEF_MSGLEN );
         nRet = MsgBox( this ).Show( szMsgText , szAppName, MB_YESNOCANCEL | MB_ICONQUESTION );
         }
   
      if( nRet == IDYES )
         {
         if( !WinSendMessage( this, WM_COMMAND, IDM_FILESAVE, 0L ) )
            return( 0 );
         }

      if( nRet == IDCANCEL )
         return( nRet );
   
      bEdited = FALSE;
      }               


   // Destroy the edit.
   if( lpEdit )
      {
      lpEdit->WinDestroy();
      delete lpEdit;
      lpEdit = (EditWindow FAR *)0L;
      }

   if( !bLocalEdit && hEditData && hEditData != GethInstance() )
      {
      GlobalMemory    mem;
      mem.SetHandle( hEditData );
      mem.Free();
      hEditData = 0;
      }

   if( bEdited == FALSE )
      return( IDOK );

   
   return( FALSE );
   }





BOOL MEChildWindow :: FAR PrinterSetup( BOOL bLoadDialog )
   {
      LocalMemory Mem;  // old data
      LocalMemory Mem1; // new data
      LPSTR       lpszOldInitData = (LPSTR)0L;
      LPSTR       lpszNewInitData = (LPSTR)0L;
      int         nNumBytes;
      int         nFlag;


   nFlag = DM_COPY;


   if( bLoadDialog )
      nFlag |= DM_PROMPT;


   // Check to see if the user has changed the printer setup.
   // If so then we need to clear the hSetupData.
   if( hSetupData )
      {
      Mem.SetHandle( hSetupData );
      lpszOldInitData = (LPSTR)Mem.Lock();

      // If the user has changed the device clear everything.
      if( lstrcmp( szDevice, lpszOldInitData ) )
         {
         Mem.Unlock();
         Mem.Free();
         hSetupData = 0;
         lpszOldInitData = (LPSTR)0L;
         }
      else nFlag |= DM_MODIFY;
      }


   // Query for number of bytes to allocate for the device mode struct.   
   nNumBytes = ExtDevMode().Init( this, lpszDriver, (LPDEVMODE)0L, szDevice, lpszPort,
                                  (LPDEVMODE)0L, (LPSTR)0L, 0 );  

   if( nNumBytes < 0 )
      {
      if( hSetupData )
         {
         Mem.Unlock();
         Mem.Free();
         hSetupData = 0;
         }
      return( FALSE );
      }

   iPrinter = EXTDEVMODE_PRINTER_AVAIL;

   Mem1.Allocate( nNumBytes );
   lpszNewInitData = (LPSTR)Mem1.Lock();
   if( !lpszNewInitData )
      {
      Mem1.Free();
      return( FALSE );
      }


   // Get a copy of the latest data.
   if( ExtDevMode().Init( this, lpszDriver, (LPDEVMODE)lpszNewInitData, szDevice, lpszPort,
                          (LPDEVMODE)lpszOldInitData, (LPSTR)0L, nFlag ) == IDOK )
      {
      nFlag = 0; 
      }


   // Get rid of the old data.
   Mem1.Unlock();
   if( hSetupData )
      {
      Mem.Unlock();
      Mem.Free();
      }
   hSetupData = Mem1.GetHandle();

   return( TRUE );   
   }





BOOL MEChildWindow :: FAR InitDevPort( BOOL bPrnSetup )
   {
      Profile     prof;


   // Initialize the iPrinter IVAR to no printer.
   iPrinter = NO_PRINTER;

   // Get printer info from WIN.INI
   prof.GetString( "windows", "device", "", szDevice, sizeof( szDevice ) );
   for( lpszDriver = szDevice; *lpszDriver && (*lpszDriver != ','); lpszDriver++ )
      ;

   if( *lpszDriver )
      *lpszDriver++ = 0;


   // From the current position in the buffer, null terminate the list
   // of ports.
   for( lpszPort = lpszDriver; *lpszPort && (*lpszPort != ','); lpszPort++ )
      ;

   if( *lpszPort )
      *lpszPort++ = 0;


   // If the device, driver and port buffers all contain meaningfull data
   // proceed, if not return false.
   if( !*szDevice || !*lpszPort || !*lpszDriver )
      {
      *szDevice = 0;
      return( FALSE );
      }

   // Get printer initialization
   // information but don't allow it
   // to bring up the printer dialog box.
   // If bPrnSetup is true then we do not
   // need to check, because the intial
   // data was passed in to us and hSetupData
   // is set.
   
   if( bPrnSetup )
      PrinterSetup( FALSE );
   else iPrinter = EXTDEVMODE_PRINTER_AVAIL;


   if( iPrinter != EXTDEVMODE_PRINTER_AVAIL )
      iPrinter = PRINTER_AVAIL;

   return( TRUE );   
   }





int MEChildWindow :: FAR Print( void )
   {
      LocalMemory    Mem;
      LPSTR          lpszDevMode = (LPSTR)0L;
      int            nRet = 0;

   if( hSetupData )
      {
      Mem.SetHandle( hSetupData );
      lpszDevMode = (LPSTR)Mem.Lock();
      }

   nRet = PrintEditText( lpszDevMode );

   if( nRet == FALSE )
      {
      GetResString( IDS_PRINTERROR, szMsgTemp, DEF_MSGLEN );
      wsprintf( szMsgText, szMsgTemp, szFileName );
      MsgBox( this ).Show( szMsgText, szAppName, MB_OK | MB_ICONEXCLAMATION );
      }

   if( hSetupData )
      Mem.Unlock();

   return( nRet );
   }






BOOL MEChildWindow :: FAR PrintEditText( LPSTR lpszDevMode )
   {
      GlobalMemory      Mem1( GMEM_MOVEABLE | GMEM_ZEROINIT | GMEM_NOT_BANKED | GMEM_DDESHARE );
      DeviceDC          dc( lpszDriver, szDevice, lpszPort, lpszDevMode );
      MEPrinter         prnt( this, dc );
      GdiText           text( dc );
      DevCaps           dcaps( dc );
      PrintDlg FAR     *lpPrnDlg;
      int               yExtPage;
      WORD              cch;
      WORD              ich;
      LPSTR             pch;
      WORD              iLine;
      WORD              nLinesEc;
      WORD              dy;
      int               yExtSoFar;
      WORD              fError = TRUE;      
      LPSTR             lpszBuffer;
      LONG              lLen;


   // If no printer is avail. return FALSE.
   if( !dc.GetHandle() )
      {
      iPrinter = NO_PRINTER;
      return( iPrinter );
      }

   // Create the title.
   lpszTitle = szFileName;

   // Disable the frame.   
   lpFrame->Disable();

   
   // Create the printer dialog.
   lpPrnDlg = new PrintDlg( this, szDevice, lpszPort, lpszTitle );
   
   if( prnt.SetAbortProc( GethInstance() ) < 0 )
      return( FALSE );

   // Initialize the document.
   if( prnt.StartDoc( lstrlen( lpszTitle ), lpszTitle ) < 0 )
      return( FALSE );
      
   // Get the height of one line.
   dy       = LOWORD( text.GetExtent( "CC", 2 ) );
   yExtPage = dcaps.GetValue( DCAPS_VERTRES );
   

   // Get the number of lines in the document.
   iLine       = 0;
   yExtSoFar   = 0;
   nLinesEc    = lpEdit->GetLineCount();

   // Allocate a buffer for the text.
   lLen = (DWORD)lpEdit->GetTextLen();
   Mem1.Allocate( lLen );
   lpszBuffer = (LPSTR)Mem1.Lock();
   if( !lpszBuffer )
      return( FALSE );

   lpEdit->GetText( lpszBuffer, (WORD)lLen );
   Mem1.Unlock();


   // While more lines, print out the text.
   while( iLine < nLinesEc )
      {
      // If we have reached the end of a page.
      if( (yExtSoFar + dy) > yExtPage )
         {
         if( (prnt.NewFrame()) < 0 || bAbort )
            {
            prnt.AbortDoc();
            lpFrame->Enable();
            lpPrnDlg->WinDestroy();

            delete    lpPrnDlg;

            Mem1.Unlock();
            Mem1.Free();
            return( FALSE );
            }
         yExtSoFar = 0;
         }

      ich = lpEdit->GetLineIndex( iLine );
      cch = lpEdit->GetLineLength( ich );
      pch = (LPSTR)(Mem1.Lock()) + ich;
      
      text.Out( 0, yExtSoFar, (LPSTR)pch, cch );
      Mem1.Unlock();

      if( bAbort )
         {
         prnt.AbortDoc();
         lpFrame->Enable();
         lpPrnDlg->WinDestroy();
         delete    lpPrnDlg;
         Mem1.Unlock();
         Mem1.Free();
         return( FALSE );
         }

      yExtSoFar += dy;
      iLine++;
      }


   // Eject the last page
   if( prnt.NewFrame() < 0 )
      {
      prnt.AbortDoc();
      lpFrame->Enable();
      lpPrnDlg->WinDestroy();
      delete    lpPrnDlg;
      Mem1.Unlock();
      Mem1.Free();
      return( FALSE );
      }


   if( prnt.EndDoc() < 0 )
      {
      prnt.AbortDoc();
      }
   else
      {
      fError = FALSE;
      }   
      
   // Enable the Frame window and
   // get rid of the print cancel dialog
   lpFrame->Enable();
   lpPrnDlg->WinDestroy();
   delete lpPrnDlg;
   Mem1.Unlock();
   Mem1.Free();

   if( fError )
      {
      GetResString( IDS_PRINTERROR, szMsgTemp, DEF_MSGLEN );
      wsprintf( szMsgText, szMsgTemp, szFileName );
      MsgBox( this ).Show( szMsgText, szAppName, MB_OK | MB_ICONEXCLAMATION );
      }

   return( TRUE );   
   }







BOOL MEChildWindow :: FAR SetWrap( BOOL bON )
   {

      GlobalMemory      gMem( GMEM_MOVEABLE | GMEM_ZEROINIT | GMEM_NOT_BANKED | GMEM_DDESHARE );
      LONG              lLen;
      LPSTR             lpszBuffer = (LPSTR)0L;
      EditWindow FAR   *lpNewEdit;


   // If hEditData is not valid then this is the
   // first control so get the initial buffer.
   // else we need to retrieve the text from
   // the previous control and set it in the new
   // one.
   if( !hEditData )
      {
      ////////////////////////////////////////////////////
      // TRICK - Making an edit use a global memory buffer
      //         instead of a local.
      ////////////////////////////////////////////////////
      // Allocate approx. 64K of global memory as a default
      // text buffer and give it to the edit.
      ////////////////////////////////////////////////////
      if( !gMem.Allocate( (DWORD)wMaxSize ) )
         {
         GetResString( IDS_ERRLOWMEM, szMsgText, DEF_MSGLEN );
         MsgBox( this ).Show( szMsgText, szAppName, MB_OK | MB_ICONHAND );
         return( FALSE );
         }

      // Set the global handle as the instance handle.
      // This tells the edit to use our handle.
      hEditData = gMem.GetHandle();

      if( hEditData >= 0x2000 )
         {
         gMem.Free();
         hEditData = GethInstance();
         bLocalEdit = TRUE;

         // set max to 32K
         wMaxSize  = 0x8000;
         GetResString( IDS_ERR32KEDIT, szMsgText, DEF_MSGLEN );
         MsgBox( this ).Show( szMsgText, szAppName, MB_OK | MB_ICONEXCLAMATION );
         }
      }
   else
      {
      lLen = (LONG)lpEdit->GetTextLen();
      gMem.Allocate( (DWORD)max( lLen+2, 1L ) );
      lpszBuffer = (LPSTR)gMem.Lock();
      if( !lpszBuffer )
         {
         gMem.Free();
         GetResString( IDS_ERRLOWMEM, szMsgText, DEF_MSGLEN );
         MsgBox( this ).Show( szMsgText, szAppName, MB_OK | MB_ICONASTERISK );
         return( FALSE );
         }
      lpEdit->GetText( lpszBuffer, (WORD)lLen+1 );
      }

   lpNewEdit = new EditWindow( this );
   if( !lpNewEdit )
      return( FALSE );

   if( bON )
      {
      lpNewEdit->WinSetWindowStyle( WS_CHILD  |
                                    WS_VISIBLE | WS_VSCROLL |
                                    ES_AUTOVSCROLL | ES_MULTILINE );
      bWrap = TRUE;
      }
   else
      {
      lpNewEdit->WinSetWindowStyle( WS_CHILD | WS_HSCROLL |
                                    WS_VISIBLE | WS_VSCROLL | ES_AUTOHSCROLL |
                                    ES_AUTOVSCROLL | ES_MULTILINE );
      bWrap = FALSE;
      }


   // Set the ID of the new control and other init vals
   lpNewEdit->WinSetWindowMenuHandle( ID_EDIT );
   lpNewEdit->WinSetWindowInstance( hEditData );
   lpNewEdit->WinSetWindowPosition( 0,0,0,0 );

   if( lpEdit )
      {
      lpEdit->WinDestroy();
      delete lpEdit;
      lpEdit = (EditWindow FAR *)0L;
      }


   // Set the lpEdit IVAR and create the edit.
   lpEdit = lpNewEdit;
   lpEdit->Create();

   // Set the font.
   if( bFixedFont )
      SetEditFont( FNT_FIXED );
   else SetEditFont( FNT_VARIABLE );


   // If the buffer was allocated and filled above
   // then free it.  If the buffer was not filled
   // but just handed off to the edit leave it 
   // alone.
   if( lpszBuffer )
      {
      lpEdit->SetText( lpszBuffer );
      gMem.Unlock();
      gMem.Free();
      }

   lpEdit->SetTextLimit( wMaxSize );

   WinSendMessage( this, WM_SIZE, 0, 0L );
   lpEdit->WinSetFocus();
   
   return( TRUE );
   }








void MEChildWindow :: FAR InitMenus( HANDLE hMenu )
   {
      Clipboard   cb( lpFrame );
      Menu        menu;
      int         nStatus;
      WORD        wStart, wEnd;
      BOOL        bIconic;


   menu.SetHandle( hMenu );
   bIconic = WinIsIconic();

   // If an edit exists then enable appropriate menu items.
   if( lpEdit && !bIconic )
      {
      if( lpEdit->CanUndo() )
         menu.Enable( IDM_EDITUNDO );
      else menu.Gray( IDM_EDITUNDO ); 

      if( iPrinter == EXTDEVMODE_PRINTER_AVAIL )
         menu.Enable( IDM_FILESETUP );
      else menu.Gray( IDM_FILESETUP );
      
      if( lpEdit->GetTextLen() > 0 )
         {
         menu.Enable( IDM_FILESAVE );
         menu.Enable( IDM_FILESAVEAS );
         menu.Enable( IDM_SEARCHFIND );
         menu.Enable( IDM_EDITSELECT );

         if( iPrinter != NO_PRINTER )
            menu.Enable( IDM_FILEPRINT );
         else menu.Gray( IDM_FILEPRINT );

         lpEdit->GetSelect( wStart, wEnd );
         if( wStart != wEnd )
            {
            menu.Enable( IDM_EDITCUT );
            menu.Enable( IDM_EDITCOPY );
            menu.Enable( IDM_EDITCLEAR );
            }
         else
            {
            menu.Gray( IDM_EDITCUT );
            menu.Gray( IDM_EDITCOPY );
            menu.Gray( IDM_EDITCLEAR );
            }

         // If there is a search string set then enable
         // next and prev searches.
         if( szSearch[0] )
            {
            menu.Enable( IDM_SEARCHNEXT );
            menu.Enable( IDM_SEARCHPREV );
            }
         else
            {
            menu.Gray( IDM_SEARCHNEXT );
            menu.Gray( IDM_SEARCHPREV );
            }
         }
      else
         {
         menu.Gray( IDM_FILESAVE );
         menu.Gray( IDM_FILESAVEAS );
         menu.Gray( IDM_FILEPRINT );
         menu.Gray( IDM_EDITSELECT );
         menu.Gray( IDM_EDITCUT );
         menu.Gray( IDM_EDITCOPY );
         menu.Gray( IDM_EDITCLEAR );
         menu.Gray( IDM_EDITWRAP );
         menu.Gray( IDM_FILESETUP );
         menu.Gray( IDM_SEARCHFIND );
         menu.Gray( IDM_FONTFIXED );
         menu.Gray( IDM_FONTVARIABLE );
         }


      // If the clipboard contains some CF_TEXT data, allow
      // paste.
      nStatus = FALSE;
      if( cb.Open() )
         {
         int   wFmt = 0;
         while( (wFmt = cb.EnumFormats( wFmt )) )
            {
            if( wFmt == CBFORMAT_TEXT )
               {
               nStatus = TRUE;
               break;
               }
            }
         cb.Close();
         }
      }


   menu.Enable( IDM_FONTFIXED );
   menu.Enable( IDM_FONTVARIABLE );
   if( bFixedFont )
      {
      menu.Check( IDM_FONTFIXED );
      menu.UnCheck( IDM_FONTVARIABLE );
      }
   else
      {
      menu.Check( IDM_FONTVARIABLE );
      menu.UnCheck( IDM_FONTFIXED );
      }


   if( nStatus && !bIconic )
      menu.Enable( IDM_EDITPASTE );
   else menu.Gray( IDM_EDITPASTE );

   // If the edit control has more than
   // 32K of text in it restrict disable
   // word wrap due to edit bug.
   if( !bIconic && lpEdit && (((WORD)lpEdit->GetTextLen()) < (WORD)0x7FFF) )
      {
      menu.Enable( IDM_EDITWRAP );
      if( bWrap )
         menu.Check( IDM_EDITWRAP );
      else menu.UnCheck( IDM_EDITWRAP );
      }
   else menu.Gray( IDM_EDITWRAP );
   
   menu.Draw();
   }


