// MEFRAME.CXX - Multi-Edit Frame MDI Window class
//               (C) Sturmer Hauss Corp. 1991


// Include only the objects from the TIER1 Library we will be using in this
// module.
#define INCL_TIER_TYPES           // data types like LONG, HWND, HANDLE
#define INCL_TIER_T1OBJECT        // base class of the TIER1 Library and Window and Overlapped classes  
#define INCL_TIER_WINDOW          // TIER Window class
   #define INCL_TIER_WINDOWMSGS   // MS-Window compatible message #defines
   #define INCL_TIER_MDIFRAME     // MDI Frame window class  
   #define INCL_TIER_MDICLIENT    // MDI Client window class
   #define INCL_TIER_MDICHILD     // MDI Child window class
   #define INCL_TIER_DIALOG       // Dialog class
      #define INCL_TIER_MODALDLG  // ModalDlg class
#define INCL_TIER_STARTUP         // TIER StartUp class
#define INCL_TIER_MENU            // Menu class  
#define INCL_TIER_ICON            // Icon class
#define INCL_TIER_SYSMETRICS      // SysMetrics class
#define INCL_TIER_MSGBOX          // MsgBox class  
#define INCL_TIER_USERCONTROL     // UserCtrlReg, UserCtrlWindow classes
#define INCL_TIER_MDICHILDPROC    // MDIChildProc window MsgHandler
#define INCL_TIER_CURSOR          // Cursor class
#define INCL_TIER_MEMORY          // Memory class
#define INCL_TIER_RESOURCE        // Resource class
#define INCL_TIER_PEEKMSG         // PeekMsg class
#define INCL_TIER_PROFILE         // Profile class
#define INCL_TIER_LOADER          // Loader class
#define INCL_TIER_CHARARRAY       // CharArray class
#define INCL_TIER_FILE            // File class

#define INCL_TIER_DEBUG



// Module includes .......................................................
#ifdef _TURBOL
extern "C"        {
#define __MSC
#endif // _TURBOL

#ifdef _GLOCK20L
extern "C"        {
#endif // _GLOCK20L

#ifdef _ZTC201L
extern "Pascal"   {
#endif // _ZTC201L

#include <stdio.h>       // sprintf(), fprintf()

#ifdef _GLOCK20L
                  }
#endif // _GLOCK20L

#ifdef _ZTC201L
                  }
#endif // _ZTC201L

#ifdef _TURBOL
                  }
#endif // _TURBOL



// Module includes
#include <tier1.hxx>    // TIER 1 Library .hxx's
#include <tier2.hxx>    // TIER 2 Library .hxx's
#include "defines.h"    // Our modules common #defines for the .RC file and here.
#include "collect.hxx"  // Collection class
#include "aboutdlg.hxx" // AboutDlg class
#include "paramdlg.hxx" // ParamDlg class
#include "opendlg.hxx"  // OpenDlg class
#include "mechild.hxx"  // MEChildWindow object
#include "meframe.hxx"  // Object definition for MEFrameWindow object.





/////////////////////////////////////////////////////////////////////////////
// MEFrameWindow object Constructor
/////////////////////////////////////////////////////////////////////////////

MEFrameWindow :: FAR MEFrameWindow( HANDLE hInstance, HANDLE hPrevInstance, LPSTR lpszCmdLine, int nCmdShow ) :
                                       (hInstance, hPrevInstance, nCmdShow )
   {
      Menu                 menu;
      MEChildWindow  FAR  *lpTempChild;
      int                  nIconHeight;
      int                  nScreenWidth;
      int                  nScreenHeight;
      int                  i;


   // Initialize IVARS.
   hPrevInst         = hPrevInstance;
   bShortMenu        = TRUE;
   lpMDIClient       = (MDIClientWindow FAR *)0L;
   szFileExts[0]     = '\0';           // This is not just one extension.
   szFileName[0]     = '\0';
   hPrinterSetupData = 0;
   bFixedFont        = TRUE;
   bOpenInitial      = TRUE;
   bSaveParams       = TRUE;
   bSaveScreen       = FALSE;
   bIsIconic         = FALSE;
   lpChildList       = (Collection FAR *)0L;

   // Initialize the file list with a - string
   // this indicates no file for this item.
   for( i = 0; i < MAX_RESTOREFILES; i++ )
      FileList[i] = "-";

   GetResString( IDS_APPNAME, szAppName, DEF_APPNAMELEN );
   GetResString( IDS_ADDEXT, szFileExts, DEF_FILENAME_LEN+1 );

   if( lpszCmdLine[0] )
      bOpenInitial = FALSE;


   // Load up our menus for the Frame&Client
   menu.Load( hInstance, ID_MAINMENU );
   hMenuInit        = menu.GetHandle();

   menu.GetPopUpMenu( MENU_LONG_POS );
   hMenuInitWindow  = menu.GetHandle();


   // If this is the first instance then initialize our window class vars.
   if( !hPrevInstance )
      {
      Icon     MEFrameIcon( this );

      MEFrameIcon.Load( IDI_MEFRAME );

      WinSetClassName( szAppName );
      WinSetClassIcon( MEFrameIcon.GetHandle() );
      }


   // Initialize our window create vars.
   WinSetWindowName( szAppName );
   WinSetWindowClassName( szAppName );
   WinSetWindowMenuHandle( hMenuInit );


   // Get the system sizes we will need to set our initial window position.
   nIconHeight       = SysMetrics().GetValue( SYSMETS_CYICON );
   nScreenWidth      = SysMetrics().GetValue( SYSMETS_CXSCREEN );
   nScreenHeight     = SysMetrics().GetValue( SYSMETS_CYSCREEN );


   // Set our window's new position prior to the create call.
   Move( 3, 3, (nScreenWidth-6), (nScreenHeight-(nIconHeight * 2)) );

   // Load ME.INI if it exists.
   GetResString( IDS_INIFILE, szMsgText, DEF_FILENAME_LEN );
   LoadProfile( szMsgText );

   // If the user last ended their session
   // as an icon move it back to that state 
   // and reset bSaveScreen to TRUE or 1.
   if( bSaveScreen == ICONIC )
      {
      Show( SW_SHOWMINIMIZED );
      bSaveScreen = 1;
      bIsIconic = TRUE;
      }


   // Create our window
   if( !Create() )
      {
      GetResString( IDS_ERRMAINWIN, szMsgText, DEF_MSGLEN );
      MsgBox().Show( szMsgText, szAppName, MB_OK | MB_ICONHAND );
      return;
      }


   // Load the Accelerator table.
   // NOTE: The accelerators should be loaded after the window they are to
   //       be associated with, has been created.
   WinLoadAccelerator( hInstance, "AccelTable" );


   // If there was a file on the command line.
   if( !bOpenInitial )
      {
      int     nIndex;
      int     nRet;

      nIndex = (int)WinSendMessage( this, WM_COMMAND, IDM_FILENEW, 0L );
      if( nIndex != -1 )
         {
         Cursor  cursor;

         cursor.Load( CURSOR_WAIT );
         cursor.Set();
         cursor.Show();

         if( lpTempChild = (MEChildWindow FAR *)lpChildList->Get( (DWORD)nIndex ) )
            {
            nRet = lpTempChild->Open( lpszCmdLine );
            if( nRet == 0 )
               {
               GetResString( IDS_CANTOPEN, szMsgTemp, DEF_MSGLEN );
               wsprintf( szMsgText, szMsgTemp, lpszCmdLine ); 
               MsgBox( this ).Show( szMsgText, szAppName, MB_OK | MB_ICONEXCLAMATION );
               WinPostMessage( lpTempChild, WM_CLOSE, 0, 0L );
               }
            if( nRet == -1 )
               {
               WinPostMessage( lpTempChild, WM_CLOSE, 0, 0L );
               }
            }
         }
      }


   }






/////////////////////////////////////////////////////////////////////////////
// MEFrameWindow object Destructor
/////////////////////////////////////////////////////////////////////////////

MEFrameWindow :: FAR ~MEFrameWindow()
   {
      MEChildWindow  FAR     *lpTempChild;
      int                     i;

   // Perform a final check to make sure all 
   // of the children are out of the collection
   // before ending the app.  This should be
   // empty always.  This is done just as a
   // safety measure.
   lpTempChild = (MEChildWindow FAR *)0L;
   while( (lpTempChild = (MEChildWindow FAR *)lpChildList->Enum( lpTempChild )) )
      {
      lpChildList->Delete( lpTempChild );
      delete lpTempChild;
      }

   delete lpChildList;


   if( hPrinterSetupData )
      {
      LocalMemory    mem;
      mem.SetHandle( hPrinterSetupData );
      mem.Free();
      }

   }




/////////////////////////////////////////////////////////////////////////////
// MEFrameWindow object WinMessageHandler()
/////////////////////////////////////////////////////////////////////////////

LONG MEFrameWindow :: FAR WinMessageHandler( HWND hWnd, unsigned iMessage, WORD wParam, LONG lParam )
   {
      static HWND                    hWndClient  = 0;
      static char                    szTemp[ DEF_FILENAMELEN+1 ];
      MEChildWindow  FAR            *lpTempChild;
      HWND                           hWndChild;
      int                            nRet;
      int                            nIndex;
      BOOL                           bNotAll;
      HANDLE                         hTemp;
      LPSTR                          lpszTemp;
      int                            i;
      int                            nTemp;


   switch( iMessage )
      {
      case  WM_CREATE:
            // Create list we will keep the children in.
            lpChildList      = new Collection();

            lpMDIClient = new MDIClientWindow( this, hMenuInitWindow, IDM_WINDOWCHILD );
            lpMDIClient->Create();
            hWndClient = lpMDIClient->GethWnd();
            if( !hWndClient )
               {
               GetResString( IDS_ERRCLIENTWIN, szMsgText, DEF_MSGLEN );
               MsgBox().Show( szMsgText, szAppName, MB_OK | MB_ICONHAND );
               }

            // Get the menu set up for new children so it will display
            // their names as created.
            bShortMenu = TRUE;
            WinSendMessage( this, WM_COMMAND, IDM_FILEMENU, 0L );

            // Open the initial editor window.
            if( bOpenInitial )
               {
               // If the window is set to open iconic then set the lParam to
               // -1L.
               if( bIsIconic )
                  WinPostMessage( this, WM_COMMAND, IDM_FILENEW, -1L );
               else WinPostMessage( this, WM_COMMAND, IDM_FILENEW, 0L );
               }
      break;

      case  WM_USEFIXEDFONT:
            // Message from child.
            bFixedFont = wParam;
      break;

      case  WM_WININICHANGE:
      case  WM_DEVMODECHANGE:
            // Pass this on to all children
            lpTempChild = (MEChildWindow FAR *)0L;
            while( (lpTempChild = (MEChildWindow FAR *)lpChildList->Enum( lpTempChild )) )
               WinSendMessage( lpTempChild, WM_WININICHANGE, 0, 0L );
            return( 0L );
      break;

      case  WM_PUSHFILE:
            PushFile( (LPSTR)lParam );
      break;

      case  WM_INITMENU:
            {
            Menu     menu;
            menu.SetHandle( wParam );

            // Pass this on to the active child so it's menus can be updated.
            // If there is no child set the menus to defaults
            hWndChild = (HWND)WinSendMessage( lpMDIClient, WM_MDIGETACTIVE, 0, 0L );
            if( hWndChild )
               {
               WinSendMessage( hWndChild, iMessage, wParam, lParam );
               }
            else
               {
               menu.SetHandle( wParam );
               menu.Gray( IDM_FILESAVE );
               menu.Gray( IDM_FILESAVEAS );
               menu.Gray( IDM_FILEPRINT );
               menu.Gray( IDM_FILESETUP );
               menu.Gray( IDM_EDITUNDO );
               menu.Gray( IDM_EDITCOPY );
               menu.Gray( IDM_EDITPASTE );
               menu.Gray( IDM_EDITCLEAR );
               menu.Gray( IDM_EDITWRAP );
               menu.Gray( IDM_EDITSELECT );
               menu.Gray( IDM_SEARCHFIND );
               menu.Gray( IDM_SEARCHNEXT );
               menu.Gray( IDM_SEARCHPREV );
               menu.UnCheck( IDM_FONTVARIABLE );
               menu.UnCheck( IDM_FONTFIXED );
               menu.Gray( IDM_FONTVARIABLE );
               menu.Gray( IDM_FONTFIXED );
               }

            }
      break;

      case  WM_INITMENUPOPUP:
            {
            // Update files listing in File menu.
            Menu     menu;
            menu.SetHandle( wParam );

            // Small hack here.  Make sure this is not a system menu.
            // System menu's start with &Restore.  Our File menu starts
            // off with &New.
            menu.GetItemString( 0, szTemp, DEF_FILENAMELEN, BY_POSITION );
            if( lstrcmp( szTemp, "&New" ) != 0 )
               return( 0L );

            // Delete all of the old files.
            nTemp = menu.GetItemCount();
            for( i = 0; i < nTemp; i++ )
               menu.Delete( MENU_AFTER_ABOUT, BY_POSITION );

            for( i = 0; i < MAX_RESTOREFILES; i++ )
               {
               lpszTemp = FileList[i].GetPtr();
               if( lpszTemp[0] != '-' )
                  {
                  wsprintf( szTemp, "&%d. %s", i+1, lpszTemp );
                  menu.Append( szTemp, IDM_PREVFILES+i, USE_STRING | BY_POSITION );
                  }
               }
            }
      break;


      case  WM_COMMAND:
            switch( wParam )
               {
               case  IDM_FILENEW:
                     {
                     Cursor  cursor;
                     BOOL    bIconic = FALSE;

                     cursor.Load( CURSOR_WAIT );
                     cursor.Set();
                     cursor.Show();
               
                     if( lParam == -1L )
                        bIconic = TRUE;


                     // Setting the hPrevInstance param to 1 tells TIER
                     // the window class is already registered.
                     
                     lpTempChild = new MEChildWindow( this, lpMDIClient,
                                                      1, "MEChild",
                                                      bFixedFont, bIconic,
                                                      hPrinterSetupData );

                     if( !lpTempChild->IsCreated() || !lpChildList->Insert( lpTempChild ) )
                        {
                        delete lpTempChild;
                        return( -1L );
                        }


                     if( !hPrinterSetupData )
                        {
                        hTemp = lpTempChild->GetSetupData();
                        if( hTemp )
                           CopyPrinterData( hTemp );
                        }
   
                     return( lpChildList->Find( lpTempChild ) );
                     }
               break;

               case  IDM_FILEOPEN:
                     {
                     Cursor         cursor;
                     OpenDlg FAR   *lpOpen;
                     int            nRet;

                     cursor.Load( CURSOR_WAIT );
                     cursor.Set();
                     cursor.Show();

                     // lParam will be set to -1L if we are opening a file
                     // from the prev file list.  If so we don't need to
                     // bring up the open dialog.
                     if( lParam != -1L )
                        {
                        lpOpen = new OpenDlg( this, IDD_FILEOPEN, szFileExts );
                        if( (nRet = lpOpen->GetResult()) )
                           lpOpen->GetFileName( szFileName, DEF_FILEANDPATH );

                        delete lpOpen;
                        }

                     if( nRet )
                        {
                        // If the file is already open set the focus to the
                        // open child.
                        lpTempChild = (MEChildWindow FAR *)0L;
                        while( (lpTempChild = (MEChildWindow FAR *)lpChildList->Enum( lpTempChild )) )
                           {           
                           lpTempChild->GetFileName( szTemp );
                           if( lstrcmp( szTemp, szFileName ) == 0 )
                              {
                              if( lpTempChild->WinIsIconic() )
                                 lpTempChild->Show( SW_RESTORE );
                              lpTempChild->WinBringWindowToTop();
                              TopFile( szFileName );
                              return( 0L );
                              }
                           }

                        nIndex = (int)WinSendMessage( this, WM_COMMAND, IDM_FILENEW, 0L );
                        if( nIndex != -1 )
                           {
                           cursor.Load( CURSOR_WAIT );
                           cursor.Set();
                           cursor.Show();

                           lpTempChild = (MEChildWindow FAR *)lpChildList->Get( (DWORD)nIndex );
                           nRet = lpTempChild->Open( szFileName );
                           if( nRet == -1 )
                              {
                              WinPostMessage( lpTempChild, WM_CLOSE, 0, 0L );
                              return( 0L );
                              }

                           if( nRet == 0 )
                              {
                              GetResString( IDS_CANTOPEN, szMsgTemp, DEF_MSGLEN );
                              wsprintf( szMsgText, szMsgTemp, szFileName );
                              MsgBox( this ).Show( szMsgText, szAppName, MB_OK | MB_ICONEXCLAMATION );
                              WinPostMessage( lpTempChild, WM_CLOSE, 0, 0L );
                              return( 0L );
                              }
                     
                           // If lParam == -1L then a selection was
                           // made from the previous files section of
                           // the file menu it is in the list so no
                           // need to add it, but bring it to the top
                           // of the list.
                           if( lParam == -1L )
                              TopFile( szFileName );
                           else PushFile( szFileName );
                           }
                        }
                     }
               break;

               case  IDM_FILEMENU:
                     {
                     Menu     NewMenu;
                     Menu     FrameMenu( this );
                     Menu     TempMenu;
                     HANDLE   hMainMenu;
                     HANDLE   hSubMenu;
                     HANDLE   hTemp;         

                     if( bShortMenu )
                        {
                        NewMenu.Load( GethInstance(), ID_MAINMENU );
                        hMainMenu = NewMenu.GetHandle();

                        NewMenu.GetPopUpMenu( MENU_LONG_POS );
                        hSubMenu = NewMenu.GetHandle();

                        bShortMenu = FALSE;
                        }      
                     else
                        {
                        NewMenu.Load( GethInstance(), ID_MAINMENU2 );
                        hMainMenu = NewMenu.GetHandle();

                        NewMenu.GetPopUpMenu( MENU_SHORT_POS );
                        hSubMenu = NewMenu.GetHandle();

                        bShortMenu = TRUE;
                        }
   
                     hTemp = (HANDLE)WinSendMessage( lpMDIClient, WM_MDISETMENU, 0, MAKELONG( hMainMenu, hSubMenu ) );

                     // Use the TempMenu passively to destroy the old menu.
                     TempMenu.SetHandle( hTemp );
                     TempMenu.Destroy(); 
                     
                     FrameMenu.Draw();
                     }
               break;

               case  IDM_FILEEXIT:
                     WinPostMessage( this, WM_CLOSE, 0, 0L );
               break;

               case  IDM_ABOUT:
                     AboutDlg( this );
               break;

               case  IDM_WINDOWTILE:
                     lpMDIClient->Tile();
               break;

               case  IDM_WINDOWCASCADE:
                     lpMDIClient->Cascade();
               break;

               case  IDM_WINDOWICONS:
                     lpMDIClient->ArrangeIcons();
               break;

               case  IDM_WINDOWCLOSEALL:
                     // Check all the children to see if they are
                     // ready to close down.
                     lpTempChild = (MEChildWindow FAR *)0L;
                     while( (lpTempChild = (MEChildWindow FAR *)lpChildList->Enum( lpTempChild )) )
                        {
                        nRet = lpTempChild->Close();
                        if( nRet == IDOK )
                           {
                           WinSendMessage( lpMDIClient, WM_MDIDESTROY, lpTempChild->GethWnd(), 0L );
                           }
                        if( nRet == IDCANCEL )
                           return( 0L );
                        }
               break;

               case  IDM_PARAMS:
                     Cursor         cursor;
                     ParamDlg FAR   *lpParam;
                     int            nRet;

                     cursor.Load( CURSOR_WAIT );
                     cursor.Set();
                     cursor.Show();

                     lpParam = new ParamDlg( this, szFileExts, bFixedFont, bSaveScreen, bSaveParams  );
                     if( (nRet = lpParam->GetResult()) )
                        {
                        lpParam->GetFileExts( szFileExts, DEF_FILEANDPATH );
                        bFixedFont  = lpParam->GetFixed();
                        bSaveScreen = lpParam->GetSavePos();
                        bSaveParams = lpParam->GetSaveParams();
                        }

                     delete lpParam;
               break;

               default:
                  {
                  // Check to see if the menu selected was one of the 
                  // Previous files.  If so open it.  Open will take care
                  // of seeing if this file is already open.
                  if( wParam >= IDM_PREVFILES && wParam < (IDM_PREVFILES+MAX_RESTOREFILES) )
                     {                       
                     lpszTemp = FileList[ wParam-IDM_PREVFILES ].GetPtr();
                     lstrcpy( szFileName, lpszTemp );
                     WinPostMessage( this, WM_COMMAND, IDM_FILEOPEN, -1L );
                     return( WinDefMDIFrameMsgHandler( hWnd, hWndClient, iMessage, wParam, lParam ) );
                     }

                  //Forward the message on to the active child then default.
                  hWndChild = (HWND)WinSendMessage( lpMDIClient, WM_MDIGETACTIVE, 0, 0L );
                  if( hWndChild )
                     WinSendMessage( hWndChild, iMessage, wParam, lParam );
                  return( WinDefMDIFrameMsgHandler( hWnd, hWndClient, iMessage, wParam, lParam ) );
                  }
               }
      break;

      case  WM_CHILDKILLED:
            // This message is sent to us from one of the child windows
            // letting us know the user closed the window, and now it's 
            // time for us to delete the object.

            lpTempChild = (MEChildWindow FAR *)lParam;
            if( lpChildList->Find( lpTempChild ) != -1L )
               {
               switch( wParam )
                  {
                  case  MECHILDWIN:
                        lpChildList->Delete( lpTempChild );
                        delete lpTempChild;
                        return( 0L );
                  break;
                  }
               }
      break;

      case  WM_QUERYENDSESSION:
      case  WM_CLOSE:
            {
            Cursor  cursor;
            cursor.Load( CURSOR_WAIT );
            cursor.Set();
            cursor.Show();

            // Check all the children to see if they are
            // ready to close down.
            bNotAll = FALSE;
            lpTempChild = (MEChildWindow FAR *)0L;
            while( (lpTempChild = (MEChildWindow FAR *)lpChildList->Enum( lpTempChild )) )
               {
               nRet = lpTempChild->Close();
               if( nRet == IDOK )
                  {
                  WinSendMessage( lpMDIClient, WM_MDIDESTROY, lpTempChild->GethWnd(), 0L );
                  }
               else bNotAll = TRUE;

               if( nRet == IDCANCEL )
                  return( 0L );
               }

            if( bNotAll )
               return( 0L );

            // Hide the frame and all children.
            Hide();

            if( lpMDIClient )
               {
               delete lpMDIClient;
               lpMDIClient = (MDIClientWindow FAR *)0L;
               }

            // We want to handle our own shutdown, instead
            // of passing it off to the default.
            WinSendMessage( this, WM_DESTROY, 0, 0L );

            return( 1L );
            }
      break;

      case  WM_DESTROY:
            // Process all outstanding messages.  Like
            // WM_CHILDKILLED, etc.
            PeekMsg().Begin();

            // Save the profile parameter information.
            GetResString( IDS_INIFILE, szMsgText, DEF_FILENAME_LEN );
            SaveProfile( szMsgText );


            WinPostQuitMessage();
            if( lpMDIClient )
               {
               delete lpMDIClient;
               lpMDIClient = (MDIClientWindow FAR *)0L;
               }
            return( 1L );
      break;

      default:
         return( WinDefMDIFrameMsgHandler( hWnd, hWndClient, iMessage, wParam, lParam ) );
      }
   
   return( 0L );
   }






MEChildWindow FAR *MEFrameWindow :: FAR GetActiveChild( void )
   {
      HWND  hWndChild;

   hWndChild   = (HWND)WinSendMessage( lpMDIClient, WM_MDIGETACTIVE, 0, 0L );

   return( (MEChildWindow FAR *)WinGetThisFromList( hWndChild ) );
   }





BOOL MEFrameWindow :: FAR CopyPrinterData( HANDLE hData )
   {
      LocalMemory    mem1;
      LocalMemory    mem2;
      LPSTR          lpszTemp1;
      LPSTR          lpszTemp2;
      int            nSize;
      int            i;


   if( !hData )
      return( FALSE );

   mem1.SetHandle( hData );
   nSize = mem1.Size();

   mem2.Allocate( nSize );
   lpszTemp1 = (LPSTR)mem1.Lock();
   lpszTemp2 = (LPSTR)mem2.Lock();

   if( !lpszTemp1 || !lpszTemp2 )
      {
      mem1.Unlock();
      mem2.Unlock();
      mem2.Free();
      GetResString( IDS_ERRPRINTER, szMsgText, DEF_MSGLEN );
      MsgBox(this).Show( szMsgText, szAppName, MB_OK | MB_ICONEXCLAMATION );
      return( FALSE );
      }

   for( i = 0; i < nSize; i++ )
      lpszTemp2[i] = lpszTemp1[i];

   mem1.Unlock();
   mem2.Unlock();

   hPrinterSetupData = mem2.GetHandle();

   return( TRUE );
   }





int MEFrameWindow :: FAR GetResString( WORD wResID, LPSTR lpszStringBuff, int nMaxChars )
   {
      Resource    res( this );

   return( res.LoadStr( wResID, lpszStringBuff, nMaxChars ) );
   }





BOOL  MEFrameWindow :: FAR LoadProfile( LPSTR lpszFileName )
   {
   Profile        profile;
   static char    szFile[ DEF_FILENAME_LEN+1 ];
   static char    szTemp[ DEF_FILENAME_LEN+1 ];
   static char    szStr[ 10 ];
   int            x, y, cx, cy;
   int            i, j;
   
   profile.GetWindowsDir( szFile, DEF_FILENAME_LEN );
   lstrcat( szFile, "\\" );
   lstrcat( szFile, lpszFileName );

   bSaveParams  = profile.GetPrivateInt( "MDI-EDIT", "SaveParams", TRUE, szFile );
   bSaveScreen  = profile.GetPrivateInt( "MDI-EDIT", "SaveScreen", TRUE, szFile );

   if( bSaveParams )
      {
      bFixedFont   = profile.GetPrivateInt( "MDI-EDIT", "FixedFont", TRUE, szFile );
      profile.GetPrivateString( "MDI-EDIT", "DefaultFileExt", szFileExts, szFileExts, DEF_FILENAME_LEN+1, szFile );
      }

   if( bSaveScreen && bSaveScreen != ICONIC )
      {
      GetPos( x, y, cx, cy );
      x  = profile.GetPrivateInt( "MDI-EDIT", "ScreenX", x, szFile );
      y  = profile.GetPrivateInt( "MDI-EDIT", "ScreenY", y, szFile );
      cx = profile.GetPrivateInt( "MDI-EDIT", "ScreenCX", cx, szFile );
      cy = profile.GetPrivateInt( "MDI-EDIT", "ScreenCY", cy, szFile );
      SetPos( x, y, cx-x, cy-y );
      }

   // Load previous files for the menu listing.
   j = 0;
   for( i = 0; i < MAX_RESTOREFILES; i++ )
      {
      sprintf( szStr, "File%d", i );
      profile.GetPrivateString( "MDI-EDIT", szStr, "-", szTemp, DEF_FILENAME_LEN+1, szFile );

      // Check to see if the file exists.  If it
      // doesn't then the user has moved or deleted
      // is so don't put it in the list.
      if( File().Exists( szTemp ) )
         FileList[j++] = szTemp;
      }

   return( TRUE );
   }





BOOL  MEFrameWindow :: FAR SaveProfile( LPSTR lpszFileName )
   {
   Profile        profile;
   static char    szFile[ DEF_FILENAME_LEN+1 ];
   static char    szFile2[ DEF_FILENAME_LEN+1 ];
   static char    szTemp[10];
   int            x, y, cx, cy;
   int            i;

   // Put our current path in the .INI
   Loader().GetModuleFileNm( GethInstance(), szFile2, DEF_FILENAME_LEN );
   
   profile.GetWindowsDir( szFile, DEF_FILENAME_LEN );
   lstrcat( szFile, "\\" );
   lstrcat( szFile, lpszFileName );

   // Put our current path and file name in the .INI for
   // others to be able to spawn us.
   profile.WritePrivateString( "MDI-EDIT", "Location", szFile2, szFile );
   

   // No matter what is set as far as saving goes we need to 
   // save bSaveParams and bSaveScreen so we know what to do
   // the next time we load.

   sprintf( szTemp, "%d", bSaveParams );
   profile.WritePrivateString( "MDI-EDIT", "SaveParams", szTemp, szFile );

   sprintf( szTemp, "%d", bSaveScreen );
   profile.WritePrivateString( "MDI-EDIT", "SaveScreen", szTemp, szFile );


   if( bSaveParams )
      {
      sprintf( szTemp, "%d", bFixedFont );
      profile.WritePrivateString( "MDI-EDIT", "FixedFont", szTemp, szFile );
      profile.WritePrivateString( "MDI-EDIT", "DefaultFileExt", szFileExts, szFile );
      }
   
    if( bSaveScreen )
      {
      GetPos( x, y, cx, cy );
      // If is an icon!
      if( WinIsIconic() )
         {
         bSaveScreen = ICONIC;
         sprintf( szTemp, "%d", bSaveScreen );
         profile.WritePrivateString( "MDI-EDIT", "SaveScreen", szTemp, szFile );
         }
      else bSaveScreen = 1;

      sprintf( szTemp, "%d", x );
      profile.WritePrivateString( "MDI-EDIT", "ScreenX", szTemp, szFile );
      sprintf( szTemp, "%d", y );
      profile.WritePrivateString( "MDI-EDIT", "ScreenY", szTemp, szFile );
      sprintf( szTemp, "%d", cx );
      profile.WritePrivateString( "MDI-EDIT", "ScreenCX", szTemp, szFile );
      sprintf( szTemp, "%d", cy );
      profile.WritePrivateString( "MDI-EDIT", "ScreenCY", szTemp, szFile );
      }


   // Save previous files for the menu listing.   
   for( i = 0; i < MAX_RESTOREFILES; i++ )
      {
      sprintf( szTemp, "File%d", i );
      profile.WritePrivateString( "MDI-EDIT", szTemp, FileList[i].GetPtr(), szFile );
      }


   return( TRUE );
   }






void MEFrameWindow :: FAR PushFile( LPSTR lpszFileName )
   {
      int            i;

   // If the file is already in the list move it to top
   // and return.
   for( i = 0; i < MAX_RESTOREFILES; i++ )
      {
      if( lstrcmp( FileList[i].GetPtr(), lpszFileName ) == 0 )
         {
         TopFile( lpszFileName );
         return;
         }
      }


   // Move everyone down and make the 0 index the new
   // string.
   for( i = MAX_RESTOREFILES-1; i > 0; i-- )
      FileList[i] = FileList[i-1];

   FileList[0] = lpszFileName;
   }





void MEFrameWindow :: FAR TopFile( LPSTR lpszFileName )
   {
      int           i;
      CharArray     array;
      int           nIndex = -1;

   for( i = 0; i < MAX_RESTOREFILES; i++ )
      {
      if( lstrcmp( FileList[i].GetPtr(), lpszFileName ) == 0 ) 
         {
         nIndex = i;
         array = FileList[i];
         FileList[i] = "-";
         break;
         }
      }

   if( i >= MAX_RESTOREFILES || nIndex == -1 )
      return;

   // Bubble to top.
   for( i = nIndex; i > 0; i-- )
      FileList[i] = FileList[i-1];

   FileList[0] = array;
   }
