/*
**           --- modem.c ---
**
**   EXAMPLE CODE: Sets/clears DTR and RTS, and
**                 displays DSR,CTS,DCD, and RI
**
**   This example program (not the PCL4C library) is donated to
**   the Public Domain by MarshallSoft Computing, Inc. It is
**   provided as an example of the use of the PCL4C.
**
*/

#include <stdio.h>
#include <dos.h>
#include "pcl4c.h"

char Buffer[8];

main(argc,argv)
int argc;
char *argv[];
{
 int Port;
 int BaudCode = 0;
 int row;
 int col;
 int xrow;
 char *p;
 int rc, i;
 char c;
 if(argc!=2)
       {puts("Usage: 'MODEM x' where x=1,2,3, or 4\n");
        exit(1);
       }
 p = argv[1];
 if((strlen(p)!=1)||(*p<'1')||(*p>'4'))
       {printf("Port selected was %s; expected 1,2,3, or 4\n",argv[1]);
        exit(2);
       }
 Port = *p - '1';
 /* must call SioRxBuf before SioReset */
 SioRxBuf(Port,Buffer,Size8);
 /* reset port */
 rc = SioReset(Port,NORESET);
 if(rc<0)
    {printf("Cannot reset port COM%d\n",1+Port);
     SioError(rc);
     exit(1);
    }
 s_clear();
 xrow = s_getrow();
 display(Port);
 putchar('\n');
 printf("1) Set DTR    3) Clear DTR    5) Redisplay\n");
 printf("2) Set RTS    4) Clear RTS    0) QUIT\n");
 putchar('\n');
 row = s_getrow();
 while(1)
       {/* clear keyboard */
        while(kbhit()) s_getc();
        /* get users' response */
        s_pos(row,0);
        printf("Select 0 to 4:   ");
        s_pos(row,14);
        c = (char) s_getc();
        if((c<'0')||(c>'5'))
              {s_pos(row+1,0);
               printf("Must select 0 to 5. Type ENTER to continue");
               getchar();
               s_pos(row+1,0);
               for(i=0;i<40;i++) putchar(' ');
               continue;
              }
        switch(c)
              {
               case '0':  /* QUIT */
                     SioDone(Port);
                     exit(0);
               case '1':  /* Set DTR */
                     SioDTR(Port,'S');
                     break;
               case '2':  /* Set RTS */
                     SioRTS(Port,'S');
                     break;
               case '3':  /* Clear DTR */
                     SioDTR(Port,'C');
                     break;
               case '4':  /* Clear RTS */
                     SioRTS(Port,'C');
                     break;
               case '5':  /* redisplay */
                     break;
              }
        s_pos(xrow,0);
        display(Port);
       }
}

int display(Port)
int Port;
{int i;
 s_pos(0,0);
 printf("     MODEM CONTROL\n\n");
 printf("Data Terminal Ready (DTR): ");
 if(SioDTR(Port,'R')) puts("ON "); else puts("OFF");
 printf("Request to Send     (RTS): ");
 if(SioRTS(Port,'R')) puts("ON "); else puts("OFF");
 printf("\n     MODEM STATUS\n\n");
 printf("Data Set Ready      (DSR): ");
 if(SioDSR(Port)) puts("ON "); else puts("OFF");
 printf("Clear To Send       (CTS): ");
 if(SioCTS(Port)) puts("ON "); else puts("OFF");
 printf("Data Carrier Detect (DCD): ");
 if(SioDCD(Port)) puts("ON "); else puts("OFF");
 printf("Ring Indicator      (RI):  ");
 if(SioRI(Port)) puts("ON "); else puts("OFF");
}

int s_pos(row, col)       /* sets current screen position */
unsigned row, col;
{union REGS reg;
 reg.h.ah = 2;
 reg.h.bh = 0;
 reg.h.dh = row;
 reg.h.dl = col;
 int86(0x10, &reg, &reg);
}

int s_getrow()       /* gets current screen cursor row position */
{union  REGS  reg;
 reg.h.ah = 3;
 reg.h.bh = 0;
 int86(0x10, &reg, &reg);
 return(reg.h.dh);
}

int s_getcol()       /* gets current screen cursor col position */
{union  REGS  reg;
 reg.h.ah = 3;
 reg.h.bh = 0;
 int86(0x10, &reg, &reg);
 return(reg.h.dl);
}

int s_clear()
{s_scroll(0,0,24,79,0,7);
}

int s_scroll(urow, lcol, lrow, rcol, nrows, attrib)
unsigned  urow, lcol, lrow, rcol;
int nrows, attrib;
{union REGS reg;
 reg.h.ah = 6;
 reg.h.ch = urow;
 reg.h.cl = lcol;
 reg.h.al = nrows;
 reg.h.bh = attrib;
 reg.h.dh = lrow;
 reg.h.dl = rcol;
 int86(0x10, &reg, &reg);
}

int s_getc()       /* read next character from keyboard */
{union REGS reg;
 reg.h.ah = 0;
 int86(0x16, &reg, &reg);
 return(reg.h.al);
}