#include <dos.h>
#include "mou.h"
#pragma check_stack (off)

/*    THIS FUNCTION CHECKS FOR PRESENCE OF MOUSE DRIVER      */



int MouInstalled(void) {
    unsigned char far *FarPtrMouDriver;  /* Points to mouse driver  */

    /* Get contents of mouse interrupt vector (0x33).               */
    FarPtrMouDriver = (unsigned char far *)getvect(0x33);

    /*  If int 33h vector is 0 or points to an IRET instruction (0xcf) */
    /*  then, mouse driver is not installed.                           */
    if (FarPtrMouDriver == 0 || *FarPtrMouDriver == 0xcf)
        return (0);
    else
        return (1);

}
/*-------------------------------------------------------------------*/

/*    THIS FUNCTION RESETS THE MOUSE AND GET THE NUMBER OF BUTTONS   */

int MouReset (int *NumButtons) {
    union REGS Reg;
    if (!MouInstalled ())      /* Call MouInstalled Function */
        return (1);

    Reg.x.ax = 0;
    int86 (0x33, &Reg, &Reg);   /* Mouse Reset and Status function */

    *NumButtons = Reg.x.bx;     /* BX contians teh number of buttons */

    return (Reg.x.ax ? 0 : 1);  /* AX = -1 if mouse properly installed */
                                /* or 0 if mouse NOT installed      */

}
/*-------------------------------------------------------------------*/

/*   THIS FUNCTION INCREMENTS THE INTERNAL POINTER FLAG     */
/*   DISPLAY MOUSE POINTER                                  */

int MouShowPointer (void) {
    union REGS Reg;
    if (!MouInstalled ())
        return (1);

    Reg.x.ax = 1;       /* int 33, function 01h */
    int86 (0x33, &Reg, &Reg);
    return (0);
}
/*---------------------------------------------------------------------*/

/*                  REMOVE MOUSE POINTER                               */

int MouHidePointer (void) {
    union REGS Reg;
    if (!MouInstalled ())
        return (1);

    Reg.x.ax = 2;
    int86(0x33, &Reg, &Reg);    /* int 33, function 02h                */
    return (0);
/* Even though the pointer is not shown the mouse driver still keeps   */
/* track of its location                                               */
}
/*---------------------------------------------------------------------*/

/*   GET POINTER POSITION / BUTTON STATUS                              */

int MouGetButtons( int *ButtonStatus, int *PointerCol, int *PointerRow) {
    union REGS Reg;
    if (!MouInstalled ())
        return (1);

    Reg.x.ax = 3;
    int86( 0x33, &Reg, &Reg);   /*  int 33, function 03h               */
    *ButtonStatus = Reg.x.bx;   /* BX contains buttons status          */
				/* bit 0 = 1 : Left Button Activated   */
				/* bit 1 = 1 : Right Button Activated  */
				/* bit 2 = 1 : Center Button Activated */
    *PointerCol = Reg.x.cx;     /* CX contains pointer column          */
    *PointerRow = Reg.x.dx;     /* DX contains pointer row             */
    return (0);
}
/*---------------------------------------------------------------------*/

/*      MOVE MOUSE POINTER  to a position on the screen                */

int MouSetPointer (int PointerCol, int PointerRow) {
    union REGS Reg;
    if (!MouInstalled ())  return (1);

    Reg.x.ax = 4;
    Reg.x.cx = PointerCol;
    Reg.x.dx = PointerRow;
    int86( 0x33, &Reg, &Reg);
    return (0);
}
/*---------------------------------------------------------------------*/

/*   DETERMINE NUMBER OF TIMES MOUSE BUTTON WAS ACTIVATED              */

int MouGetButtonPress (int Button, int *ButtonStatus, int *NumberPresses,
                       int *PointerCol, int *PointerRow) {

    union REGS Reg;
    if (!MouInstalled ())  return (1);

    Reg.x.ax = 5;      /* Function 05h */
    Reg.x.bx = Button;
    int86( 0x33, &Reg, &Reg);
    *ButtonStatus = Reg.x.ax;   /*  AX contains button status */
                                /*  Bit 0=1 : Left Button Activated */
                                /*  Bit 1=1 : Right Button Activated */
                                /*  Bit 2=1 : Center Button Activated */
    *NumberPresses = Reg.x.bx;  /*  BX contains the number of presses */
    *PointerCol = Reg.x.cx;     /*  CX contains the pointer column */
    *PointerRow = Reg.x.dx;     /*  DX contains the pointer row    */
    return (0);
/*  The coordinates return from CX and DX refer to the pixel postions */
/*  in the virtual mouse display, not the physical display sreen      */

}
/*--------------------------------------------------------------------*/

/*    DETERMINE NUMBER OF TIME MOUSE BUTTON WAS RELEASED           */

int MouGetButtonRelease (int Button, int *ButtonStatus, int *NumberReleases,
                         int *PointerCol, int *PointerRow) {

    union REGS Reg;
    if (!MouInstalled ()) return (1);

    Reg.x.ax = 6;       /* Function 06h */
    Reg.x.bx = Button;
    int86( 0x33, &Reg, &Reg);
    *ButtonStatus = Reg.x.ax;
    *NumberReleases = Reg.x.bx;
    *PointerCol = Reg.x.cx;
    *PointerRow = Reg.x.dx;
    return (0);
}
/*-------------------------------------------------------------------*/

/*        SET HORIZONTAL RANGE OF MOVEMENT                          */

int MouSetPointerHorizArea (int MinCol, int MaxCol) {
    union REGS Reg;
    if (!MouInstalled ()) return (1);

    Reg.x.ax = 7;
    Reg.x.cx = MinCol;
    Reg.x.dx = MaxCol;
    int86( 0x33, &Reg, &Reg);
    return (0);
}
/*----------------------------------------------------------------------*/

/*      SET VERTICAL RANGE OF MOVEMENT                              */

int MouSetPointerVertArea (int MinRow, int MaxRow) {
    union REGS Reg;
    if (!MouInstalled ())   return (1);

    Reg.x.ax = 8;
    Reg.x.cx = MinRow;
    Reg.x.dx = MaxRow;
    int86( 0x33, &Reg, &Reg);
    return (0);
}
/*-------------------------------------------------------------------*/

/*      SET MOUSE POINTER IN GRAPHICS MODE                          */

int MouSetGraphPointer (int HotSpotHoriz, int HotSpotVert,
                        unsigned far *FarPtrMasks) {

    union REGS Reg;
    struct SREGS SReg;
    if (!MouInstalled ())  return (1);

    Reg.x.ax = 9;
    Reg.x.bx = HotSpotHoriz;
    Reg.x.cx = HotSpotVert;
    Reg.x.dx = FP_OFF(FarPtrMasks);
    SReg.es =  FP_SEG(FarPtrMasks);
    int86x( 0x33, &Reg, &Reg, &SReg);
    return (0);
}
/*----------------------------------------------------------------------*/

/*      SET MOUSE POINTER IN TEXT MODE                                  */

int MouSetTextPointer (int PointerType, int SMaskScanStart,
                       int PMaskScanStop) {

    union REGS Reg;
    if (!MouInstalled ())  return (1);

    Reg.x.ax = 10;
    Reg.x.bx = PointerType;
    Reg.x.cx = SMaskScanStart;
    Reg.x.dx = PMaskScanStop;
    int86( 0x33, &Reg, &Reg);
    return (0);
}
/*----------------------------------------------------------------------*/

/*         DETERMINE MOVEMENT VALUES                                */

int MouGetMickeys (int *HorizCount, int *VertCount) {
    union REGS Reg;
    if (!MouInstalled ()) return (1);

    Reg.x.ax = 11;
    int86( 0x33, &Reg, &Reg);
    *HorizCount = Reg.x.cx;
    *VertCount = Reg.x.dx;
    return (0);
}
/*--------------------------------------------------------------------*/

/*        SET EVENT HANDLER                                            */

int MouSetIntHandler (int CallMask, void (far *FarPtrHandler) (void)) {
    union REGS Reg;
    struct SREGS SReg;
    if (!MouInstalled ())  return (1);

    Reg.x.ax = 12;
    Reg.x.cx = CallMask;
    Reg.x.dx = FP_OFF (FarPtrHandler);
    SReg.es =  FP_SEG (FarPtrHandler);
    int86x( 0x33, &Reg, &Reg, &SReg);
    return (0);
}
/*--------------------------------------------------------------------*/

/*       SET MICKEY / PIXEL RATIO                                   */

int  MouSetRatio (int HorizRatio, int VertRatio) {
    union REGS Reg;
    if (!MouInstalled ())  return (1);

    Reg.x.ax = 15;
    Reg.x.cx = HorizRatio;
    Reg.x.dx = VertRatio;
    int86( 0x33, &Reg, &Reg);
    return (0);
}
/*---------------------------------------------------------------------*/

/*          Exclusion Area - The mouse pointer disapears if move        */
/*                           into the exclusion area                    */
int MouExcludeArea (int UpLftX_Coord, int UpLftY_Coord,
                    int LoRghtX_Coord, int LoRghtY_Coord) {
    union REGS Reg;
    if (!MouInstalled ()) return (1);

    Reg.x.cx = UpLftX_Coord;
    Reg.x.dx = UpLftY_Coord;
    Reg.x.si  = LoRghtX_Coord;
    Reg.x.di  = LoRghtY_Coord;
    int86( 0x33, &Reg, &Reg);
    return (0);
/*  A subsequent call to Reset Mouse function 00h or Display Mouse Pointer */
/*  function 01h destroys Exclusion Area!                                  */
}
/*------------------------------------------------------------------------*/

/*          DETERMINE MOOUSE STATUS BUFFER SIZE                         */

int MouGetStorage (int *BufferSize) {
    union REGS Reg;
    if (!MouInstalled ())  return (1);

    Reg.x.ax = 21;
    int86( 0x33, &Reg, &Reg);
    *BufferSize = Reg.x.bx;
    return (0);
}
/*-------------------------------------------------------------------------*/

/*          STORE MOUSE STATUS                                             */

int MouSaveState (unsigned char far *FarPtrBuffer) {
    union REGS Reg;
    struct SREGS SReg;
    if (!MouInstalled ())  return (1);

    Reg.x.ax = 22;
    Reg.x.dx = FP_OFF(FarPtrBuffer);
    SReg.es  = FP_SEG(FarPtrBuffer);
    int86x( 0x33, &Reg, &Reg, &SReg);
    return (0);
}
/*-----------------------------------------------------------------------*/

/*          RESTORE  MOUSE  STATUS                                      */

int MouRestoreState (unsigned char far *FarPtrBuffer) {
    union REGS Reg;
    struct SREGS SReg;
    if (!MouInstalled ())  return (1);

    Reg.x.ax = 23;
    Reg.x.dx = FP_OFF(FarPtrBuffer);
    SReg.es  = FP_SEG(FarPtrBuffer);
    int86x( 0x33, &Reg, &Reg, &SReg);
    return (0);
}
/*------------------------------------------------------------------------*/





