/* the following program (tested under Turbo C 1.5) demonstrates reading
 * of dbase III+ .NDX files.  The format of the .NDX files was determined
 * by empirical testing.  This was tested under the small memory model.
 *
 * The program should run under TC 1.0, but I haven't tested this.
 *
 * The program isn't that practical, but does demonstrate reading of the
 * .NDX files.  To run simply type READNDX1 ndxfile1 [ndxfile2 ...].  The
 * .NDX file names must be fully qualified and must include the .NDX
 * extension.  The program displays the header information from the file
 * followed by the keys in order by the index.  Each key line displays the
 * .DBF file record number and the key value.
 *
 * These routines are placed in the public domain by the author, but I would
 * appreciate hearing about any improvements or refinements that are made.
 * I would especially appreciate hearing about any routines to add/delete
 * entries from the .NDX file.
 *
 * Mark Erbaugh
 * Microcomputer Enhancement
 * CIS 71370,1475
 */

/* program: READNDX1.C
 * author: Mark Erbaugh
 * copyright: (c) 1988 Microcomputer Enhancement
 * date: May 6, 1987
 * last change: 12-27-87 23:33
 * ported to TURBOC & tested under v1.5 (2/20/88)
 *
 * reads the keys in a dBASE III(+) .NDX file in order
 */

/*                    FORMAT OF dBASE III(+) .NDX FILES
 *                        (from empirical testing)
 *
 * The file is composed of 512 byte blocks.  The first block contains
 * header information, the remaining contain data.
 *
 *                              HEADER block
 * 4 bytes - block with root page of b+ tree
 * 4 bytes - first unused block (block numbering starts from 0)
 * 4 bytes - ???? (so far always 0)
 * 2 bytes - keylen: key length
 * 2 bytes - maximum keys per page (block)
 * 2 bytes - 1 if key is numeric, 0 if key is character
 * 2 bytes - entsize: length of individual entry (key plus pointers)
 * x bytes - key formula terminated with '\0'
 * remaining bytes - garbage (unused)
 *
 *                               DATA blocks
 * 4 bytes - numactive: number of active entries in block
 * numactive * entsize bytes - active entries in this block
 * 4 bytes - block number of block after last entry
 * remaining bytes - garbage (unused)
 *
 *                                 ENTRIES
 * 4 bytes - block number containing entries less than or equal to key
 * 4 bytes - record number matching key
 * keylen bytes - computed key (no '\0')
 * x bytes - if keylen is not multiple of 4, garbage pad bytes to fill out
 *
 * Note: if a record number or block number does not apply, it is 0.  All
 * of the indexes examined to date have either a block number or record
 * number specified for each entry, never both.
 *
 * A numeric (includes date format key) is stored as an 8 byte double
 * number.
 */

#include <STDIO.H>

int ixfile, entrysiz, keylen, type, numkeys, level;
void *malloc();
long lseek();
void do_page();

void main(argc,argv)
int argc;
char *argv[];
{
    char out[80];
    int i,flag;

    puts("READNDX1: - read .NDX file");
    puts("copyright (c) 1987 Microcomputer Enhancement\n");
    if (argc < 2) {
        puts("usage: READNDX1 file1 [file2] [file3] ...");
        exit();
    }
    for (i=1;i<argc;i++)
      rdndxo (argv[i]);

    puts("Done.");
}


struct hd1 {
    long root, free, blank;
    int keylen, numkeys, type, entrysiz;
    long blank1;
    char key[488];
};

rdndxo(name)    /* read index in order */
char *name;
{
    struct hd1 *buffer1;
    long root;

    puts(name);
    if ((ixfile = _open(name,0)) == EOF) {
        puts("Error opening file");
        return;
    }
    if ((buffer1 = (struct hd1 *) malloc(512)) == NULL) {
        _close(ixfile);
        puts("Not enough memory");
        return;
    }
    if (_read(ixfile,buffer1,512) != 512) {
        close(ixfile);
        free(buffer1);
        puts("Error reading header");
        return;
    }
    root = buffer1->root;
    keylen = buffer1->keylen;
    numkeys = buffer1->numkeys;
    type = buffer1->type;
    entrysiz = buffer1->entrysiz;
    printf("Key = %s\n",buffer1->key);
    printf("Key size = %d\n",keylen);
    printf("Key type = %s\n",type ? "numeric" : "character");
    free(buffer1);
    level = 0;
    do_page(root);
    close(ixfile);
    return;
}

struct page {
    int numelem,blank;
    char data[508];
};

struct item {
    long bleq, recno;
    union {
        char keyc[8];
        double keyd;
    } key;
};

void do_page(number)
long number;
{
    struct page *buffer2;
    struct item *itmptr;
    long offset;
    int i,j,base;

    if ((buffer2 = (struct page *) malloc(512)) == NULL) {
        puts("Not enough memory");
        return;
    }
    level++;
    offset = number << 9;     /* *512 */
    lseek(ixfile,offset,0);
    if (_read(ixfile,buffer2,512) != 512) {
        puts("Error reading page");
        return;
    }
    for (i=0,base=0; i<buffer2->numelem; i++,base+=entrysiz) {
        itmptr = (struct item *) &(buffer2->data[base]);
        if (itmptr->bleq)
            do_page(itmptr->bleq);
        if (itmptr->recno) {
            printf("%6ld - ",itmptr->recno);
            if (type)
                printf("%f",itmptr->key.keyd);
            else
                for (j=0; j<keylen; putchar(itmptr->key.keyc[j++]))
                    ;
            putchar('\n');
        }
    }
    itmptr = (struct item *) &(buffer2->data[base]);
    if (itmptr->bleq)
        do_page(itmptr->bleq);
    free(buffer2);
    level--;
}
