/* ******************************************************************
 * Function name : bell (sound bell)
 *
 * Description   : This function instructs the PC to sound ( ring )
 *                  the bell
 *
 * Variables      : none - However the string of ASCII 7 ( octal 7 )
 *                         is displayed to the screen, thus sounding 
 *                         the bell
 *
 * Example       : bell();                   produces the computer
 *                                              bell sound
 *
 * Rules         : The function is not passed any variables and is called
 *                 at the time the bell should be sounded.
 *
 * Calling ex.   : bell();
 *
 */

bell()
{ printf("\7");
}

/* ******************************************************************
 * Function name : blink_on ( screen blink on )
 *
 * Description   : This function instructs the PC to blink all 
 *                  characters printed on the screen until the 
 *                  "blink_off" function is called.  Note, that
 *                  even after the "blink_off" function is called,
 *                  the characters printed with blink on will still
 *                  blink.  The only way to stop them from blinking is
 *                  to reprint them with the blink off or remove them
 *                  from the screen by a clear screen or natural screen
 *                  scrolling.
 *
 * Variables      : none - However the string of ESCAPE ( octal 33 )
 *                         followed by "[5m" is sent to the screen  
 *                         instructing the screen to blink.
 *
 * Example       : blink_on();                  instructs the computer
 *                                              to turn blink mode on
 *
 * Rules         : The function is not passed any variables and is called
 *                 prior to printing the characters you wish blink.
 *
 *                 Blink mode will stay on until it is turned off by
 *                 calling the "blink_off()" function.
 *
 * Calling ex.   : blink_on();
 *
 */

blink_on()
{ printf("%c[5m",'\33');
}

/* ******************************************************************
 * Function name : blink_off ( screen blink mode off )
 *
 * Description   : This function instructs the PC to turn blink
 *                  mode off.  This function works in conjunction
 *                  with "blink_on()". "blink_on" causes all characters
 *                  printed to the screen to blink. This function
 *                  turns this blink mode off.  Note, that this
 *                  function will not stop previously printed characters
 *                  from printing, only stop future characters.
 *
 * Variables      : none - However the string of ESCAPE ( octal 33 )
 *                         followed by "[0m" is sent to the screen
 *                         instructing the screen to blink.
 *
 * Example       : blink_off();                 instructs the computer
 *                                              to turn blink mode off
 *
 * Rules         : The function is not passed any variables and is called
 *                 to turn negate the "blink_on" function.
 *
 *                 Blink mode will stay off until it is turned on by
 *                 calling the "blink_on()" function.
 *
 * Caution       : The string used to turn blink mode off ( escape [0m )
 *                 is also used to turn off other functions such as
 *                 "hi_intensity()" and "rv_on()". Therefore, after turning
 *                  the blink off, if applicable, you may have to turn
 *                  high intensity or reverse video back on.
 *
 * Calling ex.   : blink_off();
 *
 */

blink_off()
{ printf("%c[0m",'\33');
}

/* ******************************************************************
 * Function name : box_1 ( print box type 1 on the screen )
 *
 * Description   : This function instructs the PC to print a 
 *                  single line box on the screen. Note that its
 *                  location on the screen and its size are defined
 *                  by input parameters.
 *
 * Variables      : x - This parameter refers to the horizontal row on
 *                      which the top line of the box will be placed.
 *                      Consider this "x" to be the "x" part of an
 *                      "x,y" axis.
 *
 *                  y - This parameter refers to the vertical row on
 *                      which the left line of the box will be placed.
 *                      Consider this "y" to be the "y" part of an
 *                      "x,y" axis.
 *
 *                  w - This parameter refers to the width of the box
 *                      being displayed in the form of horizontal print
 *                      positions.
 *
 *                  h - This parameter refers to the height of the box
 *                      being printed in the form of vertical print lines.
 *
 * Example       : box_1(4,5,6,7);         creates a single line box that
 *                                           in 6 rows wide and 7 lines high
 *                                           with the top left corner being
 *                                           at screen location (4,5)
 *
 * Rules         : The parameters passed must physically allow the box
 *                 being created to fit on the screen. For example, on 
 *                 an 80 character screen, a box starting in column 60 can
 *                 not be 30 characters wide.
 *
 * Calling ex.   : int x_axis, y_axis, wide, high;
 *                 x_axis = 4;
 *                 y_axis = 5;
 *                 wide = 6;
 *                 high = 7;
 *                 box_1(x_axis, y_axis, wide, high);
 *
 *                 box_1(x_axis, y_axis, 5, 10);
 *
 */

box_1(x,y,w,h)
 int x,y,w,h;

{ int w_count, h_count, left, right;

  locate(x,y);
  printf("%c",'\332');
  for ( w_count = 0; w_count <= w; ++w_count )  printf("\304");
  printf("%c",'\277');
  for ( h_count = 0; h_count <= h; ++h_count )
    { locate(x + h_count + 1, y );
      printf("%c",'\263');
      for ( w_count = 0; w_count <= w; ++w_count )  printf(" ");
      printf("%c",'\263');
    }

  locate(x + h + 1, y);
  printf("%c",'\300');
  for ( w_count = 0; w_count <= w; ++w_count )  printf("%c",'\304');
  printf("%c",'\331');

}


/* ******************************************************************
 * Function name : box_2 ( print box type 2 on the screen )
 *
 * Description   : This function instructs the PC to print a 
 *                  double line box on the screen. Note that its
 *                  location on the screen and its size are defined
 *                  by input parameters.
 *
 * Variables      : x - This parameter refers to the horizontal row on
 *                      which the top line of the box will be placed.
 *                      Consider this "x" to be the "x" part of an
 *                      "x,y" axis.
 *
 *                  y - This parameter refers to the vertical row on
 *                      which the left line of the box will be placed.
 *                      Consider this "y" to be the "y" part of an
 *                      "x,y" axis.
 *
 *                  w - This parameter refers to the width of the box
 *                      being displayed in the form of horizontal print
 *                      positions.
 *
 *                  h - This parameter refers to the height of the box
 *                      being printed in the form of vertical print lines.
 *
 * Example       : box_2(4,5,6,7);         creates a double line box that
 *                                           in 6 rows wide and 7 lines high
 *                                           with the top left corner being
 *                                           at screen location (4,5)
 *
 * Rules         : The parameters passed must physically allow the box
 *                 being created to fit on the screen. For example, on 
 *                 an 80 character screen, a box starting in column 60 can
 *                 not be 30 characters wide.
 *
 * Calling ex.   : int x_axis, y_axis, wide, high;
 *                 x_axis = 4;
 *                 y_axis = 5;
 *                 wide = 6;
 *                 high = 7;
 *                 box_2(x_axis, y_axis, wide, high);
 *
 *                 box_2(x_axis, y_axis, 5, 10);
 *
 */

box_2(x,y,w,h)
 int x,y,w,h;

{ int w_count, h_count, left, right;

  locate(x,y);
  printf("%c",'\311');
  for ( w_count = 0; w_count <= w; ++w_count )  printf("\315");
  printf("%c",'\273');
  for ( h_count = 0; h_count <= h; ++h_count )
    { locate(x + h_count + 1, y );
      printf("%c",'\272');
      for ( w_count = 0; w_count <= w; ++w_count )  printf(" ");
      printf("%c",'\272');
    }

  locate(x + h + 1, y);
  printf("%c",'\310');
  for ( w_count = 0; w_count <= w; ++w_count )  printf("%c",'\315');
  printf("%c",'\274');

}


/* ******************************************************************
 * Function name : box_3 ( print box type 3 on the screen )
 *
 * Description   : This function instructs the PC to print a
 *                  wide line box on the screen. Note that its
 *                  location on the screen and its size are defined
 *                  by input parameters.
 *
 * Variables      : x - This parameter refers to the horizontal row on
 *                      which the top line of the box will be placed.
 *                      Consider this "x" to be the "x" part of an
 *                      "x,y" axis.
 *
 *                  y - This parameter refers to the vertical row on
 *                      which the left line of the box will be placed.
 *                      Consider this "y" to be the "y" part of an
 *                      "x,y" axis.
 *
 *                  w - This parameter refers to the width of the box
 *                      being displayed in the form of horizontal print
 *                      positions.
 *
 *                  h - This parameter refers to the height of the box
 *                      being printed in the form of vertical print lines.
 *
 * Example       : box_3(4,5,6,7);         creates a double line box that
 *                                           in 6 rows wide and 7 lines high
 *                                           with the top left corner being
 *                                           at screen location (4,5)
 *
 * Rules         : The parameters passed must physically allow the box
 *                 being created to fit on the screen. For example, on 
 *                 an 80 character screen, a box starting in column 60 can
 *                 not be 30 characters wide.
 *
 * Calling ex.   : int x_axis, y_axis, wide, high;
 *                 x_axis = 4;
 *                 y_axis = 5;
 *                 wide = 6;
 *                 high = 7;
 *                 box_3(x_axis, y_axis, wide, high);
 *
 *                 box_3(x_axis, y_axis, 5, 10);
 *
 */

box_3(x,y,w,h)
 int x,y,w,h;

{ int w_count, h_count, left, right;

  locate(x,y);
  printf("%c",'\333');
  for ( w_count = 0; w_count <= w; ++w_count )  printf("\333");
  printf("%c",'\333');
  for ( h_count = 0; h_count <= h; ++h_count )
    { locate(x + h_count + 1, y );
      printf("%c",'\333');
      for ( w_count = 0; w_count <= w; ++w_count )  printf(" ");
      printf("%c",'\333');
    }

  locate(x + h + 1, y);
  printf("%c",'\333');
  for ( w_count = 0; w_count <= w; ++w_count )  printf("%c",'\333');
  printf("%c",'\333');

}

/* ******************************************************************
 * Function name : e_line()  ( erase line )  
 *
 * Description   : This function instructs the PC to blank out
 *                  all characters residing on the line where the
 *                  cursor is currently placed. When wishing to clear
 *                  various lines on the screen, use this function in
 *                  conjunction with the "locate()" function which
 *                  can locate the cursor at any specified screen 
 *                  location.
 *
 * Variables      : none - However the string of \'escape ( octal 33 )
 *                         followed by "[k" is passed to the screen
 *                         instructing the line clear
 *
 * Example       : e_line();               instructs the computer to
 *                                          clear the specified line
 *
 * Rules         : The function is not passed any variables and is called
 *                 after the cursor has been placed on the appropriate
 *                 line at the time the line should be deleted.
 *
 * Calling ex.   : e_line();
 *
 */

e_line()
{ printf("%c[k",'\33');
}


/* ******************************************************************
 * Function name : e_screen()  ( erase screen )
 *
 * Description   : This function instructs the PC to blank out
 *                  all characters on the screen and place the cursor
 *                  in screen location (1,1).
 *
 * Variables      : none - However the string of \'escape ( octal 33 )
 *                         followed by "[J" is passed to the screen
 *                         instructing the line clear
 *
 * Example       : e_screen();             instructs the computer to
 *                                          clear the screen
 *
 * Rules         : The function is not passed any variables and is called
 *                 at the time the screen should be cleared.
 *
 * Special note  : The are some screens that will place the cursor in
 *                 a location other than (1,1). In this case, if desired,
 *                 you may place the cursor in location (1,1) by calling
 *                 the "locate()" function using parameters (1,1).
 *
 * Calling ex.   : e_screen();
 *
 */

e_screen()
{ printf("%c[2J",'\33');
}

/* ******************************************************************
 * Function name : hi_intensity ( screen high intensity on )
 *
 * Description   : This function instructs the PC to brighten all
 *                  characters printed on the screen until the
 *                  "low_intensity" function is called.  Note, that
 *                  even after the "low_intensity" function is called,
 *                  the characters printed with high intensity will still
 *                  be bright.  The only way to lower thir intensity is
 *                  to reprint them with the low intensity or remove them
 *                  from the screen by a clear screen or natural screen
 *                  scrolling.
 *
 * Variables      : none - However the string of ESCAPE ( octal 33 )
 *                         followed by "[1m" is sent to the screen  
 *                         instructing the screen to enter high
 *                         intensity mode.
 *
 * Example       : hi_intensity();              instructs the computer
 *                                              to turn high intensity
 *
 * Rules         : The function is not passed any variables and is called
 *                 prior to printing the characters you wish to
 *                 print brightly.
 *
 *                 High intensity mode will stay on until it is
 *                 turned off by calling the "low_intensity()" function.
 *
 * Calling ex.   : hi_intensity();
 *
 */

hi_intensity()
{ printf("%c[1m",'\33');
}

/* ******************************************************************
 * Function name : locate (locate the cursor )
 *
 * Description   : This function instructs the PC move the cursor
 *                  to a specified location on the screen.
 *
 * Variables      : x - This parameter refers to the horizontal row on
 *                      which the cursor will be placed.
 *                      Consider this "x" to be the "x" part of an
 *                      "x,y" axis.
 *
 *                  y - This parameter refers to the vertical row on
 *                      which the cursor will be placed.
 *                      Consider this "y" to be the "y" part of an
 *                      "x,y" axis.
 *
 * Example       : locate(5,10);            instructs the computer
 *                                           to move the cursor to
 *                                           locate 5,10.
 *
 * Rules         : The passed parameters must be a valid screen location.
 *                 The "x" value must be between 1 and 24 and the "y" value
 *                 must be between 1 and 80.
 *
 * Calling ex.   : int x_axis, y_axis;
 *                 x_axis = 5;
 *                 y_axis = 10;
 *                 locate (x,y);
 *
 *                 locate(20,5);
 *
 */

locate(x, y)
    int x, y;
{ printf("%c[%d;%dH", '\33', x, y);
}


/* ******************************************************************
 * Function name : low_intensity ( hi_intensity mode off )
 *
 * Description   : This function instructs the PC to turn high
 *                  intensity mode off.  This function works in
 *                  conjunction with "hi_intensity()". "hi_intensity"
 *                  causes all characters printed to the screen to
 *                  print brightly. This function turns this high
 *                  intensity mode off.  Note, that this function 
 *                  will not stop previously printed characters
 *                  from being bright, only stop future characters.
 *
 * Variables      : none - However the string of ESCAPE ( octal 33 )
 *                         followed by "[0m" is sent to the screen  
 *                         instructing the screen to blink.
 *
 * Example       : low_intensity();        instructs the computer
 *                                          to turn high intensity off
 *
 * Rules         : The function is not passed any variables and is called
 *                 to negate the "hi_intensity" function.
 *
 *                 High intensity mode will stay off until it is
 *                 turned on by calling the "hi_intensity()" function.
 *
 * Caution       : The string used to turn intensity mode off ( escape [0m )
 *                 is also used to turn off other functions such as 
 *                 "blink_on()" and "rv_on()". Therefore, after turning
 *                  the high intensity off, if applicable, you may
 *                  have to turn blink or reverse video back on.
 *
 * Calling ex.   : low_intensity();
 *
 */

low_intensity()
{ printf("%c[0m",'\33');
}

/* ******************************************************************
 * Function name : rv_off ( screen reverse video mode off )
 *
 * Description   : This function instructs the PC to turn reverse
 *                  video mode off.  This function works in
 *                  conjunction with "rv_on()". "rv_on"
 *                  causes all characters printed to the screen to
 *                  print in reverse screen color. This function
 *                  turns this reverse video mode off.  Note, that
 *                  this function will not stop previously printed
 *                  characters from being in reverse, only stop
 *                  future characters.
 *
 * Variables      : none - However the string of ESCAPE ( octal 33 )
 *                         followed by "[0m" is sent to the screen  
 *                         instructing the screen to blink.
 *
 * Example       : rv_off();               instructs the computer
 *                                          to turn reverse video off
 *
 * Rules         : The function is not passed any variables and is called
 *                 to negate the "rv_on" function.
 *
 *                 High video mode will stay off until it is
 *                 turned on by calling the "rv_on()" function.
 *
 * Caution       : The string used to turn video mode off ( escape [0m )
 *                 is also used to turn off other functions such as 
 *                 "blink_on()" and "hi_intensity()". Therefore,
 *                 after turning the reverse video off, if applicable,
 *                 you may have to turn blink or high intensity back on.
 *
 * Calling ex.   : rv_off();
 *
 */

rv_off()
{ printf("%c[0m",'\33');
}


/* ******************************************************************
 * Function name : rv_on ( screen reverse video on )
 *
 * Description   : This function instructs the PC to reverse all 
 *                  characters printed on the screen until the 
 *                  "rv_off" function is called.  Note, that
 *                  even after the "rv_off" function is called,
 *                  the characters printed with reverse video will still
 *                  be reversed.  The only way to change their video is
 *                  to reprint them with the regular video or remove them
 *                  from the screen by a clear screen or natural screen
 *                  scrolling.
 *
 * Variables      : none - However the string of ESCAPE ( octal 33 )
 *                         followed by "[7m" is sent to the screen  
 *                         instructing the screen to enter reverse
 *                         video mode.
 *
 * Example       : rv_on();                    instructs the computer
 *                                              to turn reverse video
 *
 * Rules         : The function is not passed any variables and is called
 *                 prior to printing the characters you wish to
 *                 print in reverse video.
 *
 *                 Reverse video mode will stay on until it is
 *                 turned off by calling the "rv_off()" function.
 *
 * Calling ex.   : rv_on();
 *
 */

rv_on()
{ printf("%c[7m",'\33');
}


/* ******************************************************************
 * Function name : srn_off()  ( turn screen reverse video off )
 *
 * Description   : This function instructs the PC to blank out
 *                  all characters on the screen, turn reverse video
 *                  off and place the cursor in screen location (1,1).
 *
 * Variables      : none - However this function calls two other video
 *                         functions: "rv_off()" to turn reverse video
 *                         off and "e_screen()" to clear the screen.
 *
 * Example       : srn_off();             instructs the computer to
 *                                         clear the screen and turn
 *                                         reverse video off
 *
 * Rules         : The function is not passed any variables and is called
 *                 at the time the screen should be cleared and reverse
 *                 video turned off.
 *
 * Special note  : The are some screens that will place the cursor in
 *                 a location other than (1,1). In this case, if desired,
 *                 you may place the cursor in location (1,1) by calling
 *                 the "locate()" function using parameters (1,1).
 *
 * Calling ex.   : srn_off();
 *
 */

srn_off()
{ rv_off();
  e_screen();
}


/* ******************************************************************
 * Function name : srn_on()  ( turn screen reverse video on )
 *
 * Description   : This function instructs the PC to blank out
 *                  all characters on the screen, turn reverse video
 *                  on and place the cursor in screen location (1,1).
 *
 * Variables      : none - However this function calls two other video
 *                         functions: "rv_on()" to turn reverse video
 *                         on and "e_screen()" to clear the screen.
 *
 * Example       : srn_on();             instructs the computer to
 *                                         clear the screen and turn
 *                                         reverse video on
 *
 * Rules         : The function is not passed any variables and is called
 *                 at the time the screen should be cleared and reverse
 *                 video turned on.
 *
 * Special note  : The are some screens that will place the cursor in
 *                 a location other than (1,1). In this case, if desired,
 *                 you may place the cursor in location (1,1) by calling
 *                 the "locate()" function using parameters (1,1).
 *
 * Calling ex.   : srn_on();
 *
 */

srn_on()
{ rv_on();
  e_screen();
}
