/**/
/****************************************************/
/*                                                  */
/*                     Scan-Fu.C                    */
/* Copyright 1991 J.W. Hance -  All rights Reserved */
/*                                                  */
/****************************************************/

/*

    Esentially, this is a mini-mail packer in that it
    will scan your NetMail directory and PACK an FTSC
    packet from all the suitable messages it finds.

*/


/**/

#define ALLOCSIZE 32768

#include <stdio.h>
#include <dos.h>
#include <time.h>
#include <stdlib.h>
#include <string.h>

#include "fred.h"
#include "version.h"

/* Special stuff for compiling under TurboC or TurboC++ */
#ifdef __TURBOC__
#include <dir.h>
#define Findnext findnext
#define Findfirst findfirst
#endif

/* Special stuff for compiling under Zortech C++ */
#ifdef __ZTC__
#include "fixdir.c"
#define strcmpi strcmpl
#endif

/* struct nodeaddr { int zone, net, node, point; };
int VerifyNode(char *address, struct nodeaddr *node);
*/

/* Structure to hold the message on disk in MSG format */
/* This could be different for other mail systems      */
/* but this is the format specified in FTSC standards  */
struct MSG {
	     char From[36];
	     char To[36];
	     char Subject[72];
	     char Date[20];
	     int TimesRead;
	     int DestNode;
	     int OriginNode;
	     int Cost;
	     int OriginNet;
	     int DestNet;
	     char Fill[8];
	     int ReplyTo;
	     int Attribute;
	     int NextReply;
	   };

/* Defines for Attribute Word */
#define MsgPrivate 0x0001
#define MsgCrash   0x0002
#define MsgSent    0x0008
#define MsgKill    0x0080
#define MsgLocal   0x0100
#define MsgHold    0x0200


/* ------------------------------------------------ */
/* ------------------------------------------------ */

void main(int argc, char *argv[])
{

 time_t t;
 struct ffblk ffblk;
 int done, for_me, result, i;
 FILE *in, *cffp, *pktfp, *log;
 char *temp_path, *buffer, *where;
 char *cpt, who[32], ext[5];
 char pathname[256], sub_path[256], temp_pathname[256];
 char tmpname[256], filename[256], outbound_path[256];
 char *logfile;
 int config_file_flag, path_flag, delete_flag, ct;
 unsigned int address_flag, count;
 int delete_files, first;
 char *config_file, *path, *delete, *address, *from_address;
 char *f1, *f2;
 struct PktHeader *PktHeader;
 struct MsgHeader *MsgHeader;
 struct MSG MSG;
 struct nodeaddr nodeaddr, from_nodeaddr;
 int killsent, nozone, zone_flag = 0;

 fprintf(stderr,"* Scan-Fu Version %s ", SCVERSION);
 fprintf(stderr,"Built %s at time %s *\n", __DATE__, __TIME__);
 fprintf(stderr,"* Copyright 1991 J.W. Hance - All rights reserved *\n");

 if(argc != 5){
  fprintf(stderr, "\nUsage: Scan-Fu -c <config> -d <DELETE|NODELETE>\n");
  fprintf(stderr, "Example: Scan-Fu -c config.fil -d DELETE\n");
  fprintf(stderr, "where:    config.fil is the configuration file\n");
  fprintf(stderr, "          DELETE will delete processed messages\n");
  fprintf(stderr, "          NODELETE will save processed messages\n");
  exit(-1);
 }

 /**/
 /* Sift out command line options */
 /**/
  delete_flag = config_file_flag = path_flag = address_flag = 0;
  first = 0;
  strcpy(ext, ".HUT");

  /* Look for the config file name */
  for(ct = 1; ct < argc; ct++){
   if(strcmpi(argv[ct], "-c") == 0){
    if(config_file_flag){
     fprintf(stderr, "ERROR: Redundant '-c' switches\n");
     exit(-1);
    }

    else{
     config_file = malloc(strlen(argv[++ct])+1);
     strcpy(config_file, argv[ct]);
     config_file_flag++;
    }
   }

   /* Look for the -d flag */
   else if(strcmpi(argv[ct], "-d") == 0){
    if(delete_flag){
     fprintf(stderr, "ERROR: Redundant '-d' switches\n");
     exit(-1);
    }
    else{
     delete = malloc(strlen(argv[++ct])+1);
     strcpy(delete, argv[ct]);
     delete_flag++;
     if(strcmpi(delete, "DELETE") == 0)
      delete_files = 1;
     else
      delete_files = 0;
    }
   }

  } /* End For */

 /* Done processing command line options */

 /**/
 /* Now process the configuration file */
 /**/

 /* Try to open config file */
 cffp = fopen(config_file, "r");
 if(cffp == NULL){
  fprintf(stderr, "ERROR: Can't open config file: %s\n", config_file);
  exit(-1);
 }

 buffer = malloc(256);

 f1 = malloc(128);
 f2 = malloc(128);
 temp_path = NULL;

 while(fgets(buffer, 256, cffp) != NULL){

  /* See if it is a comment */
  if(*buffer == '#') continue;

  /* Otherwise look for interesting config statements */
  sscanf(buffer, "%s %s", f1, f2);

  /* Find where the temp files go */
  if(strcmpi(f1, "fdmail") == 0){
   temp_path = malloc(strlen(f2)+1);
   strcpy(temp_path, f2);
   continue;
  }

  if(strcmpi(f1, "extofhold") == 0){
   if(strlen(f2) > 4)
    fprintf(stderr, "ExtOfHold '%s' is too long (4 chars MAX)\n", f2);
   else{
    strcpy(ext, f2);
    fprintf(stderr, "--> ExtOfHold is set to '%s'\n", ext);
   }
  }

  if(strcmpi(f1, "scanaddr") == 0){
   address = malloc(strlen(f2)+1);
   strcpy(address, f2);
   fprintf(stderr, "-- ScanAddr is %s.\n", address);
   if(VerifyNode(address, &nodeaddr) == -1){
    fprintf(stderr, "ERROR: %s is an invalid address.\n", address);
    exit(-1);
   }
  }

  if(strcmpi(f1, "scandir") == 0){
   path = malloc(strlen(f2)+1);
   strcpy(path, f2);
   fprintf(stderr, "-- ScanDir is %s.\n", path);
  }

  /* Get the logfile */
  if(strcmpi(f1, "logfile") == 0){
   logfile = malloc(strlen(f2)+1);
   strcpy(logfile, f2);

   /* Try to open the logfile */
   log = fopen(logfile, "a");
   if(log == NULL){
    fprintf(stderr, "ERROR: Can't open LOGFILE %s.\n", logfile);
    exit(4);
   }
   else{
    t = time(NULL);
    fprintf(log, "\n#SCAN-FU START @TM=%s\n", ctime(&t));
   }
   continue;
  }

  if(strcmpi(f1, "outbound") == 0){
   sprintf(outbound_path, "%ld", time(NULL));
   strcpy(tmpname, f2);
   strcat(tmpname, outbound_path+1);
  }

  if(strcmpi(f1, "who") == 0){
   strcpy(who, f2);
   fprintf(stderr, "-- Scanning for %s or UUCP.\n", who);
  }


  /* Find the FROM_NODE line */
  if(strcmpi(f1, "from_node") == 0){
   from_address = malloc(strlen(f2)+1);
   strcpy(from_address, f2);
   if(VerifyNode(from_address, &from_nodeaddr) == -1){
    fprintf(stderr, "ERROR: %s is an invalid address.\n", from_address);
    exit(-1);
   }
   continue;
  }

 }

 fclose(cffp);


 /* Done Reading Configuration file */

 strcpy(pathname, path);

 strcpy(sub_path, pathname);

 strcat(tmpname, ext);
 fprintf(stderr, "--> Packet name will be '%s'\n", tmpname);
 fprintf(log, "+PKTNAME %s\n", tmpname);

 strcat(pathname, "*.Msg");

 /* pathname has a filename like d:\fd\mail\*.msg */
 fprintf(stderr, "-- Looking for %s.\n", pathname);
 fprintf(log, "+SCNAME %n", pathname);

 done = Findfirst(pathname, &ffblk, 0);

 /* Allocate space to hold message, Pkt and Msg Headers */
 MsgHeader = malloc(sizeof(struct MsgHeader));
 PktHeader = malloc(sizeof(struct PktHeader));
 cpt = malloc(ALLOCSIZE);

 if(MsgHeader == NULL || PktHeader == NULL || cpt == NULL){
  fprintf(stderr, "ERROR: Couldn't malloc() enough memory.\n");
  fprintf(log, "!MALLOC()\n");
  exit(-1);
 }

 while(done == 0){
  for_me = 0;

  /* Look at the file that was found */
  strcpy(filename, sub_path);

  strcat(filename, ffblk.ff_name);

  in = fopen(filename, "r+b");

  Read_It:
  count = fread(&MSG, sizeof(MSG), 1, in);
  if(count != 1){
   fprintf(stderr, "ERROR: Can't read file %s.\n", filename);
   fprintf(log, "!fread()/%s\n", filename);
   exit(-1);
  } /* EndIf */

  if((MSG.DestNet  == nodeaddr.net ) &&
     (MSG.DestNode == nodeaddr.node)){

   for_me = 1;

   if((MSG.Attribute & MsgSent)){
    for_me = 0;
   }
  }
  else{
   fclose(in);
  }

  /* Make sure that it is address to (who) or UUCP */
  if(for_me){
   if(strcmpi(who, MSG.To) && strcmpi(MSG.To, "uucp")){
    for_me = 0;
    fclose(in);
   }
   else{
    if(!first){
     fprintf(stderr, "-- Found  ");
     fprintf(log, "+FND ");
    }
    else{
     fprintf(stderr, "-- Adding ");
     fprintf(log, "+ADD ");
    }

    fprintf(stderr, "%s -> %d/%d - %s\n", filename,
					   MSG.DestNet,
					   MSG.DestNode, MSG.To);

    fprintf(log, "<%s> <%d/%d> <%s>\n", filename, MSG.DestNet, MSG.DestNode,
					MSG.To);

    if((MSG.Attribute & MsgKill)) killsent = 1;
    else killsent = 0;

   }
  }

  /* Read the message if for me */
  if(for_me){
   count = fread(cpt, sizeof(char), ALLOCSIZE, in);

   if(((where = strstr(cpt, "ToUucp:")) != NULL) ||
      ((where = strstr(cpt, "touucp:")) != NULL) ||
      ((where = strstr(cpt, "TOUUCP:")) != NULL) ||
      ((where = strstr(cpt, "Touucp:")) != NULL) ||
      ((where = strstr(cpt, "To:"))     != NULL) ||
      ((where = strstr(cpt, "TO:"))     != NULL) ||
      ((where = strstr(cpt, "to:"))     != NULL) ||
      ((where = strstr(cpt, "TOUucp:")) != NULL)){
    *where = toupper(*where);
    if(!first){
     first++;
     /* Open output file and write Packet Header */
     fprintf(stderr, "-- Opening %s as output packet.\n", tmpname);
     pktfp = fopen(tmpname, "wb");
     PktHeader->OriginNode = from_nodeaddr.node;
     PktHeader->DestNode   = nodeaddr.node;
     /* Don't care what the date is since Fred-Fu ignores it anyway */
     PktHeader->Year       = 0;
     PktHeader->Month      = 0;
     PktHeader->Day        = 0;
     PktHeader->Hour       = 0;
     PktHeader->Minute     = 0;
     PktHeader->Second     = 0;
     PktHeader->Baud       = 0;
     PktHeader->Version    = 2;
     PktHeader->OriginNet  = from_nodeaddr.net;
     PktHeader->DestNet    = nodeaddr.net;
     PktHeader->ProdSer    = 0;
     PktHeader->OriginZone = from_nodeaddr.zone;
     PktHeader->DestZone   = nodeaddr.zone;

     for(i=0; i<8; i++){
      PktHeader->Password[i] = 0;
      PktHeader->Fill[i]     = 0;
     }

     for(i=8; i<20; i++)
      PktHeader->Fill[i]     = 0;

     fwrite(PktHeader, sizeof(struct PktHeader), 1, pktfp);
    }

    /* Now write out the MsgHeader */
    MsgHeader->Version     = 2;
    MsgHeader->OriginNode  = MSG.OriginNode;
    MsgHeader->DestNode    = MSG.DestNode;
    MsgHeader->OriginNet   = MSG.OriginNet;
    MsgHeader->DestNet     = MSG.DestNet;
    MsgHeader->Attribute   = 0;
    MsgHeader->Cost        = 0;
    fwrite(MsgHeader, sizeof(struct MsgHeader), 1, pktfp);

    /* Do the header lines */
    fprintf(pktfp, "%s%c", MSG.Date,    0);
    fprintf(pktfp, "%s%c", MSG.To,      0);
    fprintf(pktfp, "%s%c", MSG.From,    0);
    fprintf(pktfp, "%s%c", MSG.Subject, 0);

    /* And finally the message */
    fwrite(cpt, sizeof(char), count, pktfp);

    /* NULL terminate the message */
    fprintf(pktfp, "%c", 0);

    /* Delete packet file if delete is specified */
    if(delete_files && killsent){
     fclose(in);
     remove(filename);
    }
    else if(delete_files && !killsent){
     rewind(in);
     MSG.Attribute |= MsgSent;
     fwrite(&MSG, sizeof(MSG), 1, in);
     fclose(in);
    }
    else{
     fclose(in);
    }
   }
  }


  done = Findnext(&ffblk);
 } /* EndWhile */

 /* Write the final 0000 to packet */
 if(first){
  fprintf(pktfp, "%c%c", 0,0);
  fclose(pktfp);
  fprintf(log, "\n$SCAN-FU STOP @TM=%s CODE=%d\n", ctime(&t), 1);
  exit(1);
 }
 else{
  fprintf(log, "\n$SCAN-FU STOP @TM=%s CODE=%d\n", ctime(&t), 0);
  exit(0);
 }

}

/**/
/* Function Calls */
/**/
/* Verify and separate the fields in the NODE address */
/**/

int VerifyNode(char *string, struct nodeaddr *n)
{
 if(sscanf(string, "%d:%d/%d.%d", &n->zone, &n->net, &n->node, &n->point) != 4)
  return -1;
 else
 return 0;
}

