/****************************************************************************
*   sfdos.h - standart dos routines.                                        *
*                                               T.  Sandberg                *
*                                               02/23/88                    *
*                                                                           *
****************************************************************************/

#define F_DELETED       0
#define F_FOUND         0
#define F_NOT_FOUND    -1
#define F_PROTECTED    -2
#define F_ATTR_ON       1
#define F_ATTR_OFF     -1




struct sfdir {
              char  fnameext[12];      /* file name with extension */
              char  fname[9];          /* file name only */
              char  fext[4];           /* extension only */
              char  fdate[9];          /* file date m/d/y */
              long  fldate;            /* file date in long inter format y/m/d */
              char  ftime[6];          /* time */
              long  fltime;            /* long integer time hhmmss */
     unsigned long  fsize;    /* file size bytes */
             };


int sfdos_dir_first_entry(char *,struct ffblk *,struct sfdir *,int);
/****************************************************************************
* sfdos_dir_first_entry   retuns dir_ent of path, with file attrib          *
*                         where attrib are                                  *
*                         FA_RDONLY = read only attribue                    *
*                         FA_HIDDEN = Hidden file                           *
*                         FA_SYSTEM = System file                           *
*                         FA_DIREC  = Directory                             *
*                         FA_ARCH   = Archive   (normal attribute           *
*                                                                           *
*    ffblk variable is passed to keep several paths going at once.          *
*          variable is for internal use only.                               *
****************************************************************************/




int sfdos_dir_next_entry(struct ffblk *,struct sfdir *,int);
/****************************************************************************
* sfdos_dir_next_entry   retuns next dir_ent of path, with file attrib      *
*                        started in sfdos_dir_first_entry                   *
*                                                                           *
*   ffblk variable is passed to keep several paths going at once.           *
*         variable is for internal use only must be same as call for first  *
*                                                                           *
****************************************************************************/





void sfdos_getenv(char *env_var,char *env_value);
/*****************************************************************************
*  sfdos_getinv  get inviroment variable inv_var and returns value inv_value *
*                note inviroment variable are set with the dos set command   *
*                they are in from inv_var=inv_value                          *
*                an exmple is   LINK=c:\masm                                 *
*                where "LINK" is the inv_var and "c:\masm" is the inv_value  *
*                                                                            *
*                note : a null string will be returned if enviroment variable*
*                       is not found.                                        *
*****************************************************************************/





void sfdos_getdfree(char d,struct dfree *dspace,unsigned long *total,
                     unsigned long *total_available,int *percent_free);
/****************************************************************************
*  sfdos_getdfree  - get free space information on disk d:                  *
*                                                                           *
*   parms are                                                               *
*               d        =  disk drive letter "a" - "z" or "A" - "Z"        *
*                                                                           *
*               dspace   =  Detail on disk drive.                           *
*                                                                           *
*               total    =  Total disk space used and availble.             *
*                                                                           *
*               available =  Total disk space free (not in use)             *
*                                                                           *
*               percent_free = Percentage of free space                     *
****************************************************************************/




void sfdos_get_volid(char drive,char *vol);
/******************************************************************************
* sfdos_get_volid   retrive volid id for drive return result in volname       *
******************************************************************************/


int sfdos_delete_file(char *path);
/*****************************************************************************
* sfdos_delete_file   - delete file specified in path                        *
*                       returns F_DELETED = file deleted                     *
*                               F_NOT_FOUND                                  *
*                               F_PROCTECED    file procted can not delete   *
*****************************************************************************/


int sfdos_get_file_attr(char *path,int *attr_r,int *attr_h,int *attr_s,
                        int *attr_a);
/****************************************************************************
*  sfdos_get_file_attr - check file *path, and check to see if read only,   *
*                        if hidden, if system and if archive.               *
*                        return TRUE or FALSE for                           *
*                        each attribute.                                    *
*                        returns                                            *
*                        F_FOUND or F_NOT_FOUND                             *
****************************************************************************/


int sfdos_set_file_attr(char *path,int attr_r,int attr_h,int attr_s,
                                   int attr_a);
/****************************************************************************
*  sfdos_set_file_attr - set file *path, and change  the three catagory,    *
*                        read only, hidden, sytem.                          *
*                        set F_ATTR_ON OR F_ATTR_OFF for                    *
*                        each attribute.                                    *
*                        returns                                            *
*                        F_FOUND or F_NOT_FOUND                             *
****************************************************************************/