/*****************************************************************************/
/*                                SMARTLNK.C                                 */
/*                                                                           */
/*  This do-nothing program illustrates the effectiveness of the TopSpeed    */
/*  SmartLinker.  When this program is compiled with TopSpeed C the          */
/*  resulting executable is much smaller than when compiled using competing  */
/*  compilers since only the code necessary to execute the printf() is       */
/*  linked in.  Traditional linkers will link in the entire standard C       */
/*  library.  As a comparison, defining the macro LIB_SIZE will ensure that  */
/*  all functions can be called so that the TopSpeed linker will also link   */
/*  the entire standard C library.  This provides a rough comparison of the  */
/*  size of the runtime libraries.                                           */
/*                                                                           */
/*  PSW                                                                      */
/*****************************************************************************/

#include <assert.h>
#include <ctype.h>
#include <errno.h>
#include <float.h>
#include <limits.h>
#include <locale.h>
#include <math.h>
#include <setjmp.h>
#include <signal.h>
#include <stdarg.h>
#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#ifdef __ZTC__
 #define NZC(s)
#else
 #define NZC(s) s
#endif

void assert_h(void)
{
    assert(0 == 0);
}

void ctype_h(void)
{   int     i;

    i       = isalnum(i);
    i       = isalpha(i);
    i       = iscntrl(i);
    i       = isdigit(i);
    i       = isgraph(i);
    i       = islower(i);
    i       = isprint(i);
    i       = ispunct(i);
    i       = isspace(i);
    i       = isupper(i);
    i       = isxdigit(i);
    i       = tolower(i);
    i       = toupper(i);
}

void errno_h(void)
{   int     i;

    errno   = i = EDOM;
    errno   = i = ERANGE;
}

void float_h(void)
{   int         i;
    float       f;
    double      d;
    long double ld;

    i       = FLT_RADIX;
    i       = FLT_ROUNDS;

    i       = FLT_MANT_DIG;
    i       = DBL_MANT_DIG;
    i       = LDBL_MANT_DIG;

    i       = FLT_DIG;
    i       = DBL_DIG;
    i       = LDBL_DIG;

    i       = FLT_MIN_EXP;
    i       = DBL_MIN_EXP;
    i       = LDBL_MIN_EXP;

    i       = FLT_MIN_10_EXP;
    i       = DBL_MIN_10_EXP;
    i       = LDBL_MIN_10_EXP;

    i       = FLT_MAX_EXP;
    i       = DBL_MAX_EXP;
    i       = LDBL_MAX_EXP;

    i       = FLT_MAX_10_EXP;
    i       = DBL_MAX_10_EXP;
    i       = LDBL_MAX_10_EXP;

    f       = FLT_MAX;
NZC(d       = DBL_MAX);
NZC(ld      = LDBL_MAX);

    f       = FLT_EPSILON;
    d       = DBL_EPSILON;
    ld      = LDBL_EPSILON;

    f       = FLT_MIN;
    d       = DBL_MIN;
    ld      = LDBL_MIN;
}

void limits_h(void)
{   char            c;
    signed char     sc;
    unsigned char   uc;
    int             i;
    unsigned int    ui;
    short           sh;
    unsigned short  ush;
    long            l;
    unsigned long   ul;

    i       = CHAR_BIT;
    sc      = SCHAR_MIN;
    sc      = SCHAR_MAX;
    uc      = UCHAR_MAX;
    c       = CHAR_MIN;
    c       = CHAR_MAX;

    i       = MB_LEN_MAX;

    sh      = SHRT_MIN;
    sh      = SHRT_MAX;
    ush     = USHRT_MAX;

    i       = INT_MIN;
    i       = INT_MAX;
    ui      = UINT_MAX;

    l       = LONG_MIN;
    l       = LONG_MAX;
    ul      = ULONG_MAX;

}

void locale_h(void)
{   int             i;
    char            c;
    char           *cp;
    const char     *ccp;
    struct lconv    lc;
    struct lconv   *lcp;

    lc.decimal_point        = cp;
    lc.thousands_sep        = cp;
NZC(lc.grouping             = cp);
    lc.int_curr_symbol      = cp;
    lc.currency_symbol      = cp;
    lc.mon_decimal_point    = cp;
    lc.mon_thousands_sep    = cp;
NZC(lc.mon_grouping         = cp);
NZC(lc.positive_sign        = cp);
    lc.negative_sign        = cp;
NZC(lc.int_frac_digits      = c);
    lc.frac_digits          = c;
    lc.p_cs_precedes        = c;
    lc.p_sep_by_space       = c;
    lc.n_cs_precedes        = c;
    lc.n_sep_by_space       = c;
NZC(lc.p_sign_posn          = c);
    lc.n_sign_posn          = c;

    i       = LC_ALL;
    i       = LC_COLLATE;
    i       = LC_CTYPE;
    i       = LC_MONETARY;
    i       = LC_NUMERIC;
    i       = LC_TIME;

    cp      = setlocale(i = LC_ALL, ccp);   /* 4.4.1 */
    lcp     = localeconv();                 /* 4.4.2 */
}

void math_h(void)
{   int         i;
    int        *ip;
    double      d;
    double     *dp;

    d       = HUGE_VAL;
    d       = acos(d);               /* 4.5.2.1 */
    d       = asin(d);               /* 4.5.2.2 */
    d       = atan(d);               /* 4.5.2.3 */
    d       = atan2(d, d);          /* 4.5.2.4 */
    d       = cos(d);                /* 4.5.2.5 */
    d       = sin(d);                /* 4.5.2.6 */
    d       = tan(d);                /* 4.5.2.7 */
    d       = cosh(d);               /* 4.5.3.1 */
    d       = sinh(d);               /* 4.5.3.2 */
    d       = tanh(d);               /* 4.5.3.3 */
    d       = exp(d);                /* 4.5.4.1 */
    d       = frexp(d, ip);          /* 4.5.4.2 */
    d       = ldexp(d, i);            /* 4.5.4.3 */
    d       = log(d);                /* 4.5.4.4 */
    d       = log10(d);              /* 4.5.4.5 */
    d       = modf(d, dp);         /* 4.5.4.6 */
    d       = pow(d, d);            /* 4.5.5.1 */
    d       = sqrt(d);               /* 4.5.5.2 */
    d       = ceil(d);               /* 4.5.6.1 */
    d       = fabs(d);               /* 4.5.6.2 */
    d       = floor(d);              /* 4.5.6.3 */
    d       = fmod(d, d);           /* 4.5.6.4 */
}

void setjmp_h(void)
{   int     i;
    jmp_buf jb;

    i       = setjmp(jb);               /* 4.6.1.1 */
              longjmp(jb, i);             /* 4.6.2.1 */
}

void signal_h(void)
{   volatile sig_atomic_t   sat;
    int                     i;
    void                  (*pfi)(int);

    pfi     = SIG_DFL;
    pfi     = SIG_ERR;
    pfi     = SIG_IGN;

    i       = SIGABRT;
    i       = SIGFPE;
    i       = SIGILL;
    i       = SIGINT;
    i       = SIGSEGV;
    i       = SIGTERM;

    pfi     = signal(i = SIGABRT, pfi = SIG_DFL);         /* 4.7.1.1 */
    i       = raise(SIGABRT);                        /* 4.7.2.1 */
}

void stdarg_h(int i, ...)
{   va_list vl;

              va_start(vl, i);
    i       = va_arg(vl, int);
              va_end(vl);
}

void stddef_h(void)
{   ptrdiff_t   pd;
    size_t      size;
    wchar_t     wc;

    struct s { int i; } s = {offsetof(struct s, i)};
}

void stdio_h(void)
{   static char buf[BUFSIZ];
    char       *cp;
    int         i;
    long        l;
    void       *vp;
    size_t      size;
    fpos_t      fpos;
    FILE       *fp;
    va_list     vl;

    vp      = NULL;
    fp      = stdin;
    fp      = stdout;
    fp      = stderr;

    i       = _IOFBF;
    i       = _IONBF;
    i       = _IOLBF;

    i       = EOF;

    i       = FOPEN_MAX;
    i       = FILENAME_MAX;

    i       = L_tmpnam;

    i       = SEEK_CUR;
    i       = SEEK_END;
    i       = SEEK_SET;

    i       = TMP_MAX;

    i       = remove(cp);                                 /* 4.9.4.1 */
    i       = rename(cp, cp);                               /* 4.9.4.2 */
    fp      = tmpfile();                                        /* 4.9.4.3 */
    cp      = tmpnam(cp);                                       /* 4.9.4.4 */
    i       = fclose(fp);
    i       = fflush(fp);
    fp      = fopen(cp, cp);                                /* 4.9.5.3 */
    fp      = freopen(cp, cp, fp);                          /* 4.9.5.4 */
              setbuf(fp, cp);                                /* 4.9.5.5 */
    i       = setvbuf(fp, buf, i = _IOFBF, size = BUFSIZ);                 /* 4.9.5.6 */
    i       = fprintf(fp, cp);                              /* 4.9.6.1 */
    i       = fscanf(fp, cp);                               /* 4.9.6.2 */
    i       = printf(cp);                                   /* 4.9.6.3 */
    i       = scanf(cp);                                    /* 4.9.6.4 */
    i       = sprintf(cp, cp);                              /* 4.9.6.5 */
    i       = sscanf(cp, cp);                               /* 4.9.6.6 */


    i       = vfprintf(fp, cp, vl);                         /* 4.9.6.7 */
    i       = vprintf(cp, vl);                              /* 4.9.6.8 */
    i       = vsprintf(cp, cp, vl);                         /* 4.9.6.9 */

    i       = fgetc(fp);                                          /* 4.9.7.1 */
    cp      = fgets(cp, i, fp);                             /* 4.9.7.2 */
    i       = fputc(i, fp);                                    /* 4.9.7.3 */
    i       = fputs(cp, fp);                                /* 4.9.7.4 */
    i       = getc(fp);                                           /* 4.9.7.5 */
    i       = getchar();                                         /* 4.9.7.6 */
    cp      = gets(cp);                                         /* 4.9.7.7 */
    i       = putc(i, fp);                                     /* 4.9.7.8 */
    i       = putchar(i);                                          /* 4.9.7.9 */
    i       = puts(cp);                                     /* 4.9.7.10 */
    i       = ungetc(i, fp);                                   /* 4.9.7.11 */

    size    = fread(vp, size, size, fp);                    /* 4.9.8.1 */
    size    = fwrite(vp, size, size, fp);                   /* 4.9.8.2 */

    i       = fgetpos(fp, &fpos);                              /* 4.9.9.1 */
    i       = fseek(fp, l, i);                              /* 4.9.9.2 */
    i       = fsetpos(fp, &fpos);                           /* 4.9.9.3 */
    l       = ftell(fp);                                         /* 4.9.9.4 */
	      rewind(fp);                                         /* 4.9.9.5 */

	      clearerr(fp);                                       /* 4.9.10.1 */
    i       = feof(fp);                                           /* 4.9.10.2 */
    i       = ferror(fp);                                         /* 4.9.10.3 */
	      perror(cp);                                 /* 4.9.10.4 */
}

void stdlib_h(void)
{   wchar_t         wc;
    div_t           dt;
    ldiv_t          ldt;
    void          (*pfv)(void);
    int           (*cf)(const void *, const void *);
    int             i;
    unsigned int    ui;
    long            l;
    unsigned long   ul;
    double          d;
    size_t          size;
    char           *cp;
    const char     *ccp;
    void           *vp;
    const void     *cvp;
    wchar_t        *wcp;
    const wchar_t  *cwcp;

    i       = EXIT_FAILURE;
    i       = EXIT_SUCCESS;

    i       = RAND_MAX;

    i       = MB_CUR_MAX;

    d       = atof(cp);                   /* 4.10.1.1 */
    i       = atoi(cp);                   /* 4.10.1.2 */
    l       = atol(cp);                 /* 4.10.1.3 */
    d       = strtod(cp, &cp);           /* 4.10.1.4 */
    l       = strtol(cp, &cp, i);       /* 4.10.1.5 */
    ul      = strtoul(cp, &cp, i);      /* 4.10.1.6 */

    i       = rand();                        /* 4.10.2.1 */
              srand(ui);                   /* 4.10.2.2 */

    vp      = calloc(size, size);                /* 4.10.3.1 */
              free(vp);                      /* 4.10.3.2 */
    vp      = malloc(size);                    /* 4.10.3.3 */
    vp      = realloc(vp, size);              /* 4.10.3.4 */

              abort();                       /* 4.10.4.1 */
    i       = atexit(pfv);               /* 4.10.4.2 */
              exit(i);                     /* 4.10.4.3 */

    cp      = getenv(cp);                /* 4.10.4.4 */
    i       = system(ccp);                 /* 4.10.4.5 */

    vp      = bsearch(cvp, cvp, size, size, cf);      /* 4.10.5.1 */
              qsort(vp, size, size, cf);      /* 4.10.5.2 */

    i       = abs(i);                      /* 4.10.6.1 */
    dt      = div(i, i);                    /* 4.10.6.2 */
    l       = labs(l);                  /* 4.10.6.3 */
    ldt     = ldiv(l, l);             /* 4.10.6.4 */

    i       = mblen(ccp, size);              /* 4.10.7.1 */
    i       = mbtowc(wcp, ccp, size);      /* 4.10.7.2 */
    i       = wctomb(cp, wc);               /* 4.10.7.3 */

    size    = mbstowcs(wcp, ccp, size);        /* 4.10.8.1 */
    size    = wcstombs(cp, cwcp, size);        /* 4.10.8.2 */
}

void string_h(void)
{   int         i;
    void       *vp;
    const void *cvp;
    char       *cp;
    const char *ccp;
    size_t      size;

    vp      = memcpy(vp, cvp, size);       /* 4.11.2.1 */
    vp      = memmove(vp, cvp, size);      /* 4.11.2.2 */
    cp      = strcpy(cp, ccp);           /* 4.11.2.3 */
    cp      = strncpy(cp, ccp, size);      /* 4.11.2.4 */

    cp      = strcat(cp, ccp);           /* 4.11.3.1 */
    cp      = strncat(cp, ccp, size);      /* 4.11.3.2 */

    i       = memcmp(cvp, cvp, size);     /* 4.11.4.1 */
    i       = strcmp(ccp, ccp);         /* 4.11.4.2 */
NZC(i       = strcoll(ccp, ccp));       /* 4.11.4.3 */
    i       = strncmp(ccp, ccp, size);    /* 4.11.4.4 */
NZC(size    = strxfrm(cp, ccp, size));      /* 4.11.4.5 */

    vp      = memchr(cvp, i, size);          /* 4.11.5.1 */
    cp      = strchr(ccp, i);              /* 4.11.5.2 */
    size    = strcspn(ccp, ccp);	/* 4.11.5.3 */
    cp      = strpbrk(ccp, ccp);       /* 4.11.5.4 */
    cp      = strrchr(ccp, i);             /* 4.11.5.5 */
    size    = strspn(ccp, ccp);         /* 4.11.5.6 */
    cp      = strstr(ccp, ccp);        /* 4.11.5.7 */
    cp      = strtok(cp, ccp);           /* 4.11.5.8 */

    vp      = memset(vp, i, size);             /* 4.11.6.1 */
    cp      = strerror(i);                    /* 4.11.6.2 */
    size    = strlen(ccp);                 /* 4.11.6.3 */
}

void time_h(void)
{   int                 i;
    double              d;
    char               *cp;
    const char         *ccp;
    size_t              size;
    clock_t             c;
    time_t              t;
    time_t             *tp;
    const time_t       *ctp;
    struct tm           tms;
    struct tm          *tmsp;
    const struct tm    *ctmsp;

    i   = CLOCKS_PER_SEC;

    tms.tm_sec  = i;
    tms.tm_min  = i;
    tms.tm_hour = i;
    tms.tm_mday = i;
    tms.tm_mon  = i;
    tms.tm_year = i;
    tms.tm_wday = i;
    tms.tm_yday = i;
    tms.tm_isdst= i;

    c       = clock();                       /* 4.12.2.1 */
    d       = difftime(t, t);               /* 4.12.2.2 */
    t       = mktime(tmsp);                    /* 4.12.2.3 */
    t       = time(tp);                     /* 4.12.2.4 */

    cp      = asctime(ctmsp);               /* 4.12.3.1 */
    cp      = ctime(ctp);                /* 4.12.3.2 */
    tmsp    = gmtime(ctp);             /* 4.12.3.3 */
    tmsp    = localtime(ctp);              /* 4.12.3.4 */
    size    = strftime(cp, size, ccp, ctmsp);            /* 4.12.3.5 */
}

void call_all_library_functions(void)
{
    assert_h();
    ctype_h();
    errno_h();
    float_h();
    limits_h();
    locale_h();
    math_h();
    setjmp_h();
    signal_h();
    stdarg_h(1);
    stddef_h();
    stdio_h();
    stdlib_h();
    string_h();
    time_h();
}

int main(void)
{
#ifdef LIB_SIZE
    call_all_library_functions();
    printf("ANSI C Library Size Test.\n");
#else
    printf("SmartLink Test.\n");
#endif
    return 0;
}
