// SRCHDLG.CXX - Search Dialogs object members

// Include Tier1 objects used in this module

#define INCL_TIER_TYPES                 // needed
#define INCL_TIER_T1OBJECT              // needed
#define INCL_TIER_WINDOW                // needed
   #define INCL_TIER_WINDOWMSGS         // needed
   #define INCL_TIER_DIALOG             // needed
      #define INCL_TIER_CONTROLWINDOW   // needed  
         #define INCL_TIER_BUTTON       // needed
         #define INCL_TIER_EDIT         // needed
#define INCL_TIER_MODALDLG              // needed  
#define INCL_TIER_MEMORY                // needed
#define INCL_TIER_MSGBOX                // needed



// Module includes .......................................................
#ifdef _TURBOL
extern "C"        {
#define __MSC
#endif // _TURBOL

#ifdef _GLOCK20L
extern "C"        {
#endif // _GLOCK20L

#ifdef _ZTC201L
extern "Pascal"   {
#endif // _ZTC201L

#include <stdio.h>       // sprintf(), fprintf()

#ifdef _GLOCK20L
                  }
#endif // _GLOCK20L

#ifdef _ZTC201L
                  }
#endif // _ZTC201L

#ifdef _TURBOL
                  }
#endif // _TURBOL


#include <tier1.hxx>     // This gets all needed includes for a Tier2 app.
#include "defines.h"     // TRACE app. common defines (cursors)
#include "srchdlg.hxx"   // Object definition associated with this module.  
                         



//////////////////////////////////////////////////////////////////////////////
// SearchDlg member functions (Search Edit or DblClick)
//////////////////////////////////////////////////////////////////////////////
SearchDlg :: FAR SearchDlg( Window FAR *lpParentWindow, int nResID, LPSTR lpszSearchText, int nMaxTextLen, BOOL bCaseSensitive, BOOL bSearchForward ) : (lpParentWindow)
   {
      LocalMemory    mem;
      LPSTR          lpszTemp;

   // Init IVAR's
   nMaxLen  = nMaxTextLen;
   bCase    = bCaseSensitive;
   bForward = bSearchForward;

   mem.Allocate( nMaxLen+1 );
   if( !(lpszTemp = (LPSTR)mem.Lock()) )
      {
      MsgBox().Show( "Unable to allocate search string buffer", "Search", MB_OK | MB_ICONEXCLAMATION );
      return;
      }

   lstrcpy( lpszTemp, lpszSearchText );
   mem.Unlock();

   hSrchText = mem.GetHandle();

   if( !Create( nResID ) )
      MsgBox().Show( "Unable to create Search dialog", "Search", MB_OK | MB_ICONEXCLAMATION );
      
   }                                  




// Destructor
SearchDlg :: ~SearchDlg()
   {
      LocalMemory    mem;

   if( hSrchText )
      {
      mem.SetHandle( hSrchText );
      mem.Free();
      }
   }





// Search Dialog message handling function.
BOOL SearchDlg :: FAR DlgMessageHandler( HWND hDlg, unsigned iMessage, WORD wParam, LONG lParam )
   {

   switch( iMessage )
      {
      case  WM_INITDIALOG:
            {
            LocalMemory  mem;
            LPSTR        lpszTemp;
            
            mem.SetHandle( hSrchText );
            lpszTemp = (LPSTR)mem.Lock();
            EditWindow( this, IDD_SEARCHTEXT, FALSE ).SetText( lpszTemp );
            mem.Unlock();

            if( bCase )
               ButtonWindow( this, IDD_CASE, FALSE ).Check();


            if( bForward )               
               ButtonWindow( this, IDD_DIRFORWARD, FALSE ).Check();
            else ButtonWindow( this, IDD_DIRBACKWARD, FALSE ).Check();
            }
      break;

      case  WM_CLOSE:
            End( FALSE );
      break;

      case  WM_COMMAND:
            switch( wParam )
               {
               case  IDOK:
                     {
                     LocalMemory  mem;
                     LPSTR        lpszTemp;
            
                     mem.SetHandle( hSrchText );
                     lpszTemp = (LPSTR)mem.Lock();
                     EditWindow( this, IDD_SEARCHTEXT, FALSE ).GetText( lpszTemp, nMaxLen );
                     mem.Unlock();

                     bCase    = ButtonWindow( this, IDD_CASE, FALSE ).IsChecked();
                     bForward = ButtonWindow( this, IDD_DIRFORWARD, FALSE ).IsChecked();

                     End( TRUE );
                     }
               break;

               case  IDCANCEL:
                     End( FALSE );
               break;
               }
      break;

      default:
         return( DlgDefMessageHandler(hDlg,iMessage,wParam,lParam) );
      }

   return( TRUE  );
   }





int SearchDlg :: FAR GetSearchText( LPSTR lpszSearchStr, int nMaxChars )
   {
      LocalMemory    mem;
      int            i;
      LPSTR          lpszTemp;

   mem.SetHandle( hSrchText );
   lpszTemp = (LPSTR)mem.Lock();
   
   for( i = 0; i < nMaxChars; i++ )
      {
      lpszSearchStr[i] = lpszTemp[i];
      if( lpszTemp[i] == '\0' )
         {
         mem.Unlock();
         return( i-1 );
         }
      }

   lpszSearchStr[i+1] = '\0';
   mem.Unlock();

   return( i );
   }

