/**/
/* Function Calls for FRED-UF */
/**/

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <alloc.h>
#include <string.h>
#include <io.h>
#include <fcntl.h>
#include <time.h>
#include <dos.h>
#include <mem.h>

#include "version.h"
#include "fred.h"

extern int Debug;

/**/
/* Output statistics of various newsgroups to log file */
/**/
#ifdef __COMPACT__ || __LARGE__
void DumpFidosToLog(struct tofidos far *ptr, FILE *fp)
{
 struct tofidos far *p;
#endif

#ifdef __SMALL__ || __MEDIUM__
void DumpFidosToLog(struct tofidos *ptr, FILE *fp)
{
 struct tofidos *p;
#endif

#ifdef __TINY__ || __HUGE__
#error Cannot COMPILE for TINY or HUGE
#endif

 p = ptr;

 fprintf(fp, "* Individual NewsGroup Summary *\n");
 do{
  if(p->count != 0){
   fprintf(fp, "%5d messages mapped from %s to %s\n",
	   p->count, p->uucpname, p->fidoname);
  }
  p = p->next;
 } while(p != NULL);
}


/**/
/* Determine what AREA is in the current buffer */
/**/

char * pascal FindArea(char *cptr, int count, struct Str *Str)
{
 char *pt, *ept, *tpt;
 int i;

 i = 0;

 /* Find the beginning of the field */
 pt = FindHeader(cptr, count, "AREA:");

 /* Account for any spaces between AREA: and the name */
 while((*pt == SPACE) && (++i < 10))
  pt++;

 if(i == 9){
  fprintf(stderr, "ERROR - no area tag found after AREA: field\n");
  exit(2);
 }

 if(pt != NULL){
  /* Now get the end of the field */
  ept = memchr(pt, '\r', count);
  if(ept == NULL){
   fprintf(stderr, "ERROR: Can't find EndOfLine in FindArea()\n");
   exit(2);
  EndIf

 EndIf
 else{
  /* AREA: not found - must be NetMail */
  return NULL;
 EndElse

 /*
 if(Debug)
  fprintf(stderr,"\nAREA is: ");
  fwrite(pt, sizeof(char), ept - pt, stderr);
  fprintf(stderr, "\n");
 */

 Str->cpt = pt;
 Str->cnt = ept - pt;

 return pt;

}

/**/
/* Determine is a ToUucp: is in the current buffer */
/**/

char * pascal FindToUucp(char *cptr, int count, struct Str *Str)
{
 char *pt, *ept, *tpt;

 /* Find the beginning of the field */
 if((pt = FindHeader(cptr, count, "ToUucp:")) == NULL)
  if((pt = FindHeader(cptr, count, "touucp:")) == NULL)
   if((pt = FindHeader(cptr, count, "To:")) == NULL)
    pt = FindHeader(cptr, count, "to:");

 if(pt != NULL){
  /* ignore any leading blanks */
  if(*pt == SPACE){
   while(*pt == SPACE){
    pt++;
   }
  }

  /* Now get the end of the field */
  ept = memchr(pt, '\r', count);
  if(ept == NULL){
   fprintf(stderr, "ERROR: Can't find EndOfLine in FindToUucp()\n");
   exit(2);
  EndIf
  else{
   /* Strip off any trailing spaces */
   if(*(ept-1) == SPACE){
    ept--;
    while(*ept == SPACE){
     ept--;
    EndWhile
    *(++ept) = '\r';
   EndIf
  EndElse

 EndIf
 else{
  /* ToUucp: not found - must be NetMail without ToUucp: */
  return NULL;
 EndElse

 if(Debug){
  fprintf(stderr,"\nToUucp is: ");
  fwrite(pt, sizeof(char), ept - pt, stderr);
  fprintf(stderr, "\n");
 }

 Str->cpt = pt;
 Str->cnt = ept - pt;

 /* See if there are any SPACES in the string */
 tpt = memchr(pt, SPACE, ept - pt);
 if(tpt != NULL){
  Str->cnt = tpt - pt;
 }

 return pt;

}


/**/
/* Find the start of the field after the requested header */
/**/

char * pascal FindHeader(char *xptr, int count, char *findme)
{
 int flag;
 char *pt, *cptr;

 flag = 0;
 cptr = xptr;
 while((pt = (char *)memchr(cptr, *findme, count)) != NULL){
  if(memicmp(pt, findme, strlen(findme)) == 0){
   /* Found it */
   flag++;
   break;
  EndIf
  else{
   count -= (pt - cptr + 1);
   cptr = ++pt;
  EndElse
 EndWhile

 if(flag){
  /* Now get to the character after the ':' */
  /* pt points to first char of string now */
  pt += strlen(findme);
  return pt;
 EndIf

 return NULL;

}

 /**/
 /* Print startup message */
 /**/
void FredFuStartup(void)
{
 fprintf(stderr,"* Fred-Fu Version %s ", FUVERSION);
 fprintf(stderr,"Built %s at time %s *\n", __DATE__, __TIME__);
 fprintf(stderr,"* Copyright 1990, 1991 J. W. Hance - ");
 fprintf(stderr,"All Rights Reserved *\n");
}

/**/
/* List usage */
/**/
void FredFuUsage(void)
{
  fprintf(stderr, "\nERROR: Improper commandline usage.\n");
  fprintf(stderr, "usage: fred-fu -c config <packet >batch\n");
  fprintf(stderr, "where: config is the configuration file.\n");
  fprintf(stderr, "       batch is the batched news file to build.\n");
  fprintf(stderr, "       packet is the UnArced FidoNet mail packet.\n");
  exit(2);
}

unsigned int GetSequenceNumber(void)
{

 FILE *seqfp;
 unsigned int sequence;

    /* Open the sequence file, read and increment the value */
    if((seqfp = fopen("./sequence", "r")) != NULL){
     if(fscanf(seqfp, "%d", &sequence) == 1){
      fclose(seqfp);
      if((seqfp = fopen("./sequence", "w")) != NULL){

       if(sequence == 10000) sequence = 0;

       fprintf(seqfp, "%d", sequence+1);
       fclose(seqfp);
      EndIf
      else{
       fprintf(stderr, "ERROR: Can't open sequence file for write.\n");
       exit(2);
      EndElse
     EndIf
     else{
      fprintf(stderr, "ERROR: Couldn't get value from sequence file.\n");
      exit(2);
     EndElse
    EndIf
    else{
     fprintf(stderr, "ERROR: Can't open sequence file.\n");
     exit(2);
    EndElse

    return sequence;
}

void WriteCmdXQTFiles(char *spoolpath,
		      char *address, char *user, char *system,
		      char *seq)
{
 FILE *fp;
 char *filename;


 filename = AllocateString(strlen(spoolpath)+strlen(address)+12);

 strcpy(filename, spoolpath);
 strcat(filename, address);
 strcat(filename, "/");
 strcat(filename, "FR");
 strcat(filename, seq);
 strcat(filename, ".CMD");

 fp = OpenFile(filename, "w");
 fprintf(fp, "S FR%s.DAT D.%s%s uucp - FR%s.DAT 0666\n", seq,
	      system, seq, seq);

 fprintf(fp, "S FR%s.XQT X.%s%s uucp - FR%s.XQT 0666\n", seq,
	      system, seq, seq);
 fclose(fp);

 strcpy(filename, spoolpath);
 strcat(filename, address);
 strcat(filename, "/");
 strcat(filename, "FR");
 strcat(filename, seq);
 strcat(filename, ".XQT");

 fp = OpenFile(filename, "wb");

 fprintf(fp, "U uucp %s\n", system);
 fprintf(fp, "Z\n");
 fprintf(fp, "F D.%s%s\n", system, seq);
 fprintf(fp, "I D.%s%s\n", system, seq);
 fprintf(fp, "C rmail %s\n", user);

 fclose(fp);


 free(filename);

}

/**/
/* Write out the header for a uucp mail message */
/**/
FILE * WriteUucpMessageHeader(char *spoolpath,
			    char *address, char *user, char *full_user,
			    char *system, char *from,
			    char *seq, char *who, char *ascii_date,
			    char *message_date, int lost,
			    int zone, int net, int node,
			    char *domain, char *category, int sendertype,
			    struct nodeaddr *onode)
{

 FILE *fp;
 char *filename, *uufrom;
 int i;

 filename = AllocateString(strlen(spoolpath)+strlen(address)+12);
 uufrom   = AllocateString(strlen(from));

 strcpy(filename, spoolpath);
 strcpy(uufrom, from);

 /* Turn any spaces in uufrom to "." */
 for(i=0; i<strlen(uufrom); i++){
  if(*(uufrom+i) == SPACE)
   *(uufrom+i) = PERIOD;

 EndFor

 if(!lost){
  strcat(filename, address);
  strcat(filename, "/");
 EndIf

 strcat(filename, "FR");
 strcat(filename, seq);
 strcat(filename, ".DAT");

 fp = OpenFile(filename, "wb");

 fprintf(fp, "From %s %s local at %s\n",
	      who, ascii_date, system);
 fprintf(fp, "Received: by %s (%s/%s)\n", system, who, FUVERSION);
 fprintf(fp, "        via %s; %s\n", who, ascii_date);
 fprintf(fp, "        for %s\n", user);

 if(sendertype)
  if(strcmpi(category, "(blank)") == 0)
   fprintf(fp, "From: %s@%s.%s\n", uufrom, system, domain, from);
  else
   fprintf(fp, "From: %s@%s.%s.%s\n", uufrom, system, domain, category, from);
 else
  if(strcmpi(category, "(blank)") == 0)
   fprintf(fp, "From: %s@f%d.n%d.z%d.%s (%s)\n",
		    uufrom, node, net, zone, domain, from);
  else
   fprintf(fp, "From: %s@f%d.n%d.z%d.%s.%s (%s)\n",
		    uufrom, node, net, zone, domain, category, from);

 if(sendertype)
  if(strcmpi(category, "(blank)") == 0)
   fprintf(fp, "Sender: %s@%s.%s\n", who, system, domain);
  else
   fprintf(fp, "Sender: %s@%s.%s.%s\n", who, system, domain, category);
 else
  if(strcmpi(category, "(blank)") == 0)
   fprintf(fp, "Sender: %s@p%d.f%d.n%d.z%d.%s\n", who, onode->point,
		onode->node, onode->net, onode->zone, domain);
  else
   fprintf(fp, "Sender: %s@p%d.f%d.n%d.z%d.%s.%s\n", who, onode->point,
		onode->node, onode->net, onode->zone, domain, category);

 fprintf(fp, "Date: %s\n", message_date);
 fprintf(fp, "Message-ID: <%s.%ld@%s>\n", who, time(NULL), system);
 fprintf(fp, "To: %s (%s)\n\n", user, full_user);

 free(filename);
 free(uufrom);

 return fp;

}

/**/
/* Match the given username with one from tousers to get a uucp name map */
/**/
struct tousers *FindUucpUserName(struct tousers *fptr, char *name)
{
    int flag;
    struct tousers *ptr;

    ptr = fptr;
    flag = 0;

    do{
       if(strcmp(name, ptr->fidoname) == 0){
	flag++;
	break;
       EndIf

       /* Point to the next record */
       ptr = ptr->next;

       } while(ptr != NULL);

       if(flag)
	return ptr;
       else
	return NULL;

}

/**/
/* Get the FidoNet address of the origin node from the * Origin line */
/**/

/* Returns: -1 - ERROR in address
	     0 - Net/Node
	     1 - Net/Node.Point
	     2 - Zone:Net/Node
	     3 - Zone:Net/Node.Point
	     */

int GetAddressFromOrigin(char *buf, int count,
			 int *zone, int *net, int *node, int *point)
{
 int i;
 char *rparen, *lparen, *where;
 int is_colon, is_slash, is_point;

 is_colon = is_slash = is_point = 0;

 /* Look backwards for the ')' */
 for(i=count; i>0; i--){
  if(*(buf+i) == RPAREN)
   break;
 EndFor

 rparen = buf+i;

 for(; i>0; i--){
  if(*(buf+i) == LPAREN)
   break;
 EndFor

 lparen = buf+i;

 /* If lparen >= rparen then there must be an error */
 if(lparen >= rparen)
  return (-1);

 /* Find the what is in the address */
 if(memchr(lparen, COLON,  rparen-lparen) != NULL) is_colon++;
 if(memchr(lparen, SLASH,  rparen-lparen) != NULL) is_slash++;
 if(memchr(lparen, PERIOD, rparen-lparen) != NULL) is_point++;

 if(!is_point) *point = -1;
 if(!is_colon) *zone  = -1;

 /* If no '/' then it must be wrong */
 if(!is_slash) return (-1);

 if(is_colon && is_point){
  sscanf(lparen+1, "%d:%d/%d.%d", zone, net, node, point);
  return(3);
 EndIf
 else if(is_colon){
  sscanf(lparen+1, "%d:%d/%d", zone, net, node);
  return(2);
 EndIf
 else if(is_point){
  sscanf(lparen+1, "%d/%d.%d", net, node, point);
  return(1);
 EndIf
 else{
  sscanf(lparen+1, "%d/%d", net, node);
  return(0);
 EndElse

}

/* Filter out any SoftCR (0x8d) characters in the specified buffer */
/* by changing them to LF */

void FilterSoftCR(char *buffer, unsigned int count)
{

 char *point, *oldpoint;

 oldpoint = point = buffer;

 while(1){
  if((point = memchr(point, 0x8d, count)) == NULL)
   break;
  else{
   if((*(point+1) != LF) && (point < (oldpoint+count)))
    *(point) = LF;
   else
    *(point) = SPACE;
   count -= (point - oldpoint);
   oldpoint = point;
  EndElse
 EndWhile
}

/* Filter out any ^A (0x01) characters in the specified buffer */
/* by changing them to SPACE (0x20) */

void FilterCTRLA(char *buffer, unsigned int count)
{

 char *point, *oldpoint;

 oldpoint = point = buffer;

 while(1){
  if((point = memchr(point, 0x01, count)) == NULL)
   break;
  else{
   *(point) = SPACE;
   count -= (point - oldpoint);
   oldpoint = point;
   fprintf(stderr, "-- NOTE: Stray CTRLA Filtered...\n");
  EndElse
 EndWhile
}

/* Parse a uucp pathname for the first system name */
int ParsePath(char *path, char *Uuser, char *Uusystem)
{
 int type;
 char *Expt, *Atpt, *Pcpt, *Ccpt, *Tmp;

 /* Determine the type of address and return type
   -1 - Unknown
    0 - Smart Mailer Needed
    1 - sys1!sys2!sys3!user   -> Uu=sys2!sys3!user Us=sys1
    2 - user@sys1             -> Uu=user           Us=sys1
    3 - user%sys2@sys1        -> Uu=user@sys2      Us=sys1
    4 - @sys1:user@sys2       -> Uu=user@sys2      Us=sys1

    if Us would be returned with a "." in it, we need to send the
    message to a smart mailer.  In that case, we set Uu to the
    original path, Us is don't care, and return a code of 1.
 */

 /* First see if it has and "!", if so, no % or @ is allowed */
 if((Expt = strchr(path, '!')) != NULL){
  if((strchr(path, '@') != NULL) || (strchr(path, '%') != NULL)){
   return -1;
  }
  else{
   /* It's OK, go ahead and process */
   strncpy(Uusystem, path, (Expt - path));
   *(Uusystem + (Expt - path)) = NULL;

   if(strchr(Uusystem, '.') != NULL){
    strcpy(Uuser, path);
    return 0;
   }
   else{
    strcpy(Uuser, (Expt + 1));
    return 1; /* ! */
   }
  }
 }

 /* Now look for a type 3 which has a % in it */
 /* It MUST also have an @ for this type      */
 if((Pcpt = strchr(path, '%')) != NULL){
  if((Atpt = strchr(path, '@')) == NULL)
   return -1;
  else{
   strcpy(Uusystem, (Atpt + 1));
   if(strchr(Uusystem, '.') != NULL){
    strcpy(Uuser, path);
    return 0;
   }
   else{
    strncpy(Uuser, path, (Atpt - path));
    *(Uuser + (Pcpt - path)) = '@';
    *(Uuser + (Atpt - path)) = NULL;
    return 3;
   }
  }
 }

 /* A type 4 must have a : and start with an @ */
 if((Ccpt = strchr(path, ':')) != NULL){
  if(*path != '@')
   return -1;
  else{
   strncpy(Uusystem, path+1, (Ccpt - path)-1);
   *(Uusystem + (Ccpt - path)) = NULL;
   if(strchr(Uusystem, '.') != NULL){
    strcpy(Uuser, path);
    return 0;
   }
   else{
    strcpy(Uuser, (Ccpt + 1));
    return 4;
   }
  }
 }

 /* Finally a type 2 with an @ */
 if((Atpt = strchr(path, '@')) != NULL){
  strcpy(Uusystem, (Atpt + 1));
  if(strchr(Uusystem, '.') != NULL){
   strcpy(Uuser, path);
   return 0;
  }
  else{
   strncpy(Uuser, path, (Atpt - path));
   *(Uuser + (Atpt - path)) = NULL;
   return 2;
  }
 }

 return -1;
}

/* Insert line breaks wherever necessary */
void DoLineBreak(int BreakAt, char *Buffer, int BufferCount)
{
    char *ptr;
    int i;

    /* If the line is short enough, just forget it, OK? */
    if(BufferCount <= BreakAt) return;

    /* Look for any of the possible line break characters */
    ptr = memchr(Buffer, LF, BreakAt);
    if(ptr == NULL)
     ptr = memchr(Buffer, CR, BreakAt);
    if(ptr == NULL)
     ptr = memchr(Buffer, SOFTCR, BreakAt);

    /* See if there were none */
    if(ptr == NULL){
     /* Find the last SPACE and turn it into an LF */
     for(ptr = Buffer+BreakAt; (*ptr != SPACE) && (ptr >= Buffer); ptr--) ;
      if(ptr > Buffer) *ptr = LF;
      else{
       fprintf(stderr, "==> Couldn't BREAK line as specified.\n");
       return;
      }
    }

    /* See if any more */
    ++ptr;
    BufferCount -= ptr - Buffer;
    DoLineBreak(BreakAt, ptr, BufferCount);
    return;
}
