/**/
/* Function Calls for FRED-UF */
/**/

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <alloc.h>
#include <mem.h>
#include <io.h>
#include <fcntl.h>
#include <time.h>
#include <dos.h>

#include "version.h"
#include "fred.h"

extern int Debug;
extern int quiet;

/**/
/* Output statistics of various newsgroups to log file */
/**/

#ifdef __COMPACT__ || __LARGE__
void DumpFidosToLog(struct tofidos far *ptr, FILE *fp)
{
 struct tofidos far *p;
#endif

#ifdef __SMALL__ || __MEDIUM__
void DumpFidosToLog(struct tofidos *ptr, FILE *fp)
{
 struct tofidos *p;
#endif

#ifdef __TINY__ || __HUGE__
#error Cannot COMPILE for TINY or HUGE
#endif


 p = ptr;

 fprintf(fp, "+SUMMARY\n");
 do{
  if(p->count != 0){
   fprintf(fp, "+MAP <%5d> <%s> <%s>\n",
	   p->count, p->uucpname, p->fidoname);
  }
  p = p->next;
 } while(p != NULL);
}

/**/
/* Get the whole Path: string */
/**/

char * pascal FindPath(char *cptr, int count, struct Str *Str)
{
 static char *pt;
 char *ept, *tpt;

 pt = FindHeader(cptr, count, "Path:");
 if(pt != NULL){

  /* Now get the end of the line */
  ept = memchr(pt, '\n', count);
  if(ept == NULL){
   fprintf(stderr, "ERROR: Can't find EndOfLine in FindPath()\n");
   return NULL;
  EndIf
 EndIf
 else{
  fprintf(stderr, "ERROR: Path: not found in this message\n");
  return NULL;
 EndElse

  Str->cpt = pt;
  Str->cnt = ept - pt;

  return pt;

}

/**/
/* Get the whole Message-ID: string */
/**/

char * pascal FindMessageID(char *cptr, int count, struct Str *Str)
{
 static char *pt;
 char *ept, *tpt;

 pt = FindHeader(cptr, count, "Message-ID:");
 if(pt != NULL){

  /* Now get the end of the line */
  ept = memchr(pt, '\n', count);
  if(ept == NULL){
   fprintf(stderr, "ERROR: Can't find EndOfLine in FindMessageID()\n");
   return NULL;
  EndIf
 EndIf
 else{
  fprintf(stderr, "ERROR: Message-ID: not found in this message.\n");
  return NULL;
 EndElse

  Str->cpt = pt;
  Str->cnt = ept - pt;

  return pt;
}

/**/
/* Get the Organization: string */
/**/

char * pascal FindOrganization(char *cptr, int count, struct Str *Str)
{
 static char *pt;
 char *ept, *tpt;

 pt = FindHeader(cptr, count, "Organization:");
 if(pt != NULL){

  /* Now get the end of the line */
  ept = memchr(pt, '\n', count);
  if(ept == NULL){
   fprintf(stderr, "ERROR: Can't find EndOfLine in FindOrganization()\n");
   return NULL;
  EndIf
 EndIf
 else{
  return NULL;
 EndElse

  Str->cpt = pt;
  Str->cnt = ept - pt;

  return pt;
}

/**/
/* Get the Reply_To: string */
/**/

char * pascal FindReplyTo(char *cptr, int count, struct Str *Str)
{
 static char *pt;
 char *ept, *tpt;

 pt = FindHeader(cptr, count, "Reply-To:");
 if(pt != NULL){

  /* Now get the end of the line */
  ept = memchr(pt, '\n', count);
  if(ept == NULL){
   fprintf(stderr, "ERROR: Can't find EndOfLine in FindReplyTo()\n");
   return NULL;
  EndIf
 EndIf
 else{
  return NULL;
 EndElse

  Str->cpt = pt;
  Str->cnt = ept - pt;

  return pt;
}

/**/
/* Determine what newsgroups are in the current buffer */
/**/

/* if there are more than one separated by commas,   */
/* only return the first one */
struct mgroups * pascal FindNewsGroup(char *cptr, int count, struct Str *Str)
{
 char *pt, *ept, *tpt, *xpt;
 static struct mgroups Mgroups;
 char buffer[256];
 int i;

 Mgroups.howmany = 0;

 /* Find the beginning of the field */
 pt = FindHeader(cptr, count, "NewsGroups:");

 if(pt != NULL){
  /* Now get the end of the field */
  ept = memchr(pt, '\n', count);
  if(ept == NULL){
   fprintf(stderr, "ERROR: Can't find EndOfLine in FindNewsGroup()\n");
   return NULL;
  EndIf

  /* Added to support multiple newsgroups */

  /* Copy to a buffer */
  xpt = memcpy(buffer, pt, ept-pt);
  *(xpt + (ept-pt)) = NULL;

  if((xpt = strtok(buffer, ",")) != NULL){
   strcpy(Mgroups.g[0], xpt);
   strcpy(Mgroups.firstgroup, xpt);
   Mgroups.howmany++;

   i = 1;
   while(i<10){
    if((xpt = strtok(NULL, ",")) != NULL){
     strcpy(Mgroups.g[i++], xpt);
     Mgroups.howmany++;
    EndIf
    else
     break;
   EndWhile
  }

  /***************************************/

  /* Look for any ',' */
  tpt = memchr(pt, COMMA, ept - pt);

  if(tpt != NULL){
   ept = tpt;
  EndIf
 EndIf
 else{
  fprintf(stderr, "ERROR: 'Newsgroups:' not found in this message.\n");
  return NULL;
 EndElse

 /*
 if(Debug)
  fprintf(stderr,"\nGroupname is: ");
  fwrite(pt, sizeof(char), ept - pt, stderr);
  fprintf(stderr, "\n");
 */

 Str->cpt = pt;
 Str->cnt = ept - pt;

 return &Mgroups;

}

/**/
/* Determine what is the SUBJECT in the current buffer */
/**/

char * pascal FindSubject(char *cptr, int count, struct Str *Str)
{
 static char *pt;
 char *ept, *tpt;

 pt = FindHeader(cptr, count, "Subject:");

 if(pt != NULL){
  /* Now get the end of the field */
  ept = memchr(pt, '\n', count);
  if(ept == NULL){
   fprintf(stderr, "ERROR: Can't find EndOfLine in FindSubject()\n");
   return NULL;
  EndIf

 EndIf
 else{
  fprintf(stderr, "ERROR: 'Subject:' not found in this message.\n");
  return NULL;
 EndElse

 /* Truncate if Subject is too long */
 if((ept - pt) > 71)
  ept = pt + 71;

 /*
 if(Debug)
  fprintf(stderr,"\nSubject is: ");
  fwrite(pt, sizeof(char), ept - pt, stderr);
  fprintf(stderr, "\n");
 */

 Str->cpt = pt;
 Str->cnt = ept - pt;

 return pt;

}

/**/
/* Get the whole From: string */
/**/

char * pascal FindWholeFrom(char *cptr, int count, struct Str *Str)
{
 static char *pt;
 char *ept, *tpt;

 pt = FindHeader(cptr, count, "From:");
 if(pt != NULL){

  /* Now get the end of the line */
  ept = memchr(pt, '\n', count);
  if(ept == NULL){
   fprintf(stderr, "ERROR: Can't find EndOfLine in FindWholeFrom()\n");
   return NULL;
  EndIf
 EndIf
 else{
  fprintf(stderr, "ERROR: From: not found in this message.\n");
  return NULL;
 EndElse

  Str->cpt = pt;
  Str->cnt = ept - pt;

  return pt;
}

/**/
/* Determine who message in the current buffer is from */
/**/

char * pascal FindFrom(char *cptr, int count, struct Str *Str)
{
 int flag1;
 static char *pt;
 char *ept, *tpt;

 pt = FindHeader(cptr, count, "From:");
 if(pt != NULL){

  /* Now get the end of the line */
  ept = memchr(pt, '\n', count);
  if(ept == NULL){
   fprintf(stderr, "ERROR: Can't find EndOfLine in FindFrom()\n");
   return NULL;
  EndIf

  /* Look for a '(' */
  flag1 = 0;
  tpt = memchr(pt, LPAREN, ept - pt);
  if(tpt != NULL){
   pt = ++tpt;
   /* And find the ')' */
   tpt = memchr(pt, RPAREN, ept - pt);
   if(tpt != NULL){
    flag1++;
    ept = tpt;
   EndIf
  EndIf


  /* Not found - return NULL */
  if(flag1 == 0)
   pt = NULL;

 EndIf
 else{
  fprintf(stderr, "WARNING: 'From:' not found in this message.\n");
  pt = NULL;
 EndElse

 /* Truncate if too long */
 if((ept - pt) >35)
  ept = pt + 35;

 /*
 if(Debug)
  if(pt != NULL){
   fprintf(stderr, "\nFrom is: ");
   fwrite(pt, sizeof(char), ept - pt, stderr);
   fprintf(stderr, "\n");
  EndIf
  else{
   fprintf(stderr, "From is: NULL\n");
  EndElse
 */

 Str->cpt = pt;
 Str->cnt = ept - pt;

 return pt;

}

/**/
/* Get the whole date string for current message */
/**/

char * pascal FindWholeDate(char *cptr, int count, struct Str *Str)
{
 int i, j, k, flag, flag1;
 static char *pt;
 char *ept, *dt, *tpt;

 pt = FindHeader(cptr, count, "Date:");

 if(pt != NULL){

  /* Now get the end of the field */
  ept = memchr(pt, '\n', count);
  if(ept == NULL){
   fprintf(stderr, "ERROR: Can't find EndOfLine in FindWholeDate()\n");
   return NULL;
  EndIf
 EndIf
 else{
  fprintf(stderr, "ERROR: Date: not found in this message.\n");
  return NULL;
 EndElse

  Str->cpt = pt;
  Str->cnt = ept - pt;

  return pt;
}


/**/
/* Determine the date of the current message */
/**/

char * pascal FindDate(char *cptr, int count, struct Str *Str)
{
 int i, j, k, flag, flag1;
 static char *pt, *dt;
 static char GoodDate[32], HoldDate[32];
 char *ept, *tpt;

 pt = FindHeader(cptr, count, "Date:");

 if(pt != NULL){

  /* Now get the end of the field */
  ept = memchr(pt, '\n', count);
  if(ept == NULL){
   fprintf(stderr, "ERROR: Can't find EndOfLine in FindDate()\n");
   return NULL;
  EndIf

  /* Look for the first Digit */
  for(i=0; i<32; i++){
   if(isdigit(*(pt+i))){
    dt = pt + i;
    break;
   EndIf
  EndFor

  tpt = memchr(dt, COLON, ept - dt);
  if(tpt != NULL){
   ept = tpt + 6;
  EndIf

 EndIf
 else{
  fprintf(stderr, "WARNING: 'Date:' not found in this message.\n");
  return NULL;
 EndElse

 if(( ept - dt) > 22)
  return NULL;

 strncpy(HoldDate, dt, ept-dt);
 HoldDate[ept-dt] = '\0';

 /* Get the day into GoodDate */
 tpt = strtok(HoldDate, " ");
 if(atoi(tpt) == 0 || atoi(tpt) > 31) return NULL;
 strcpy(GoodDate, tpt);
 strcat(GoodDate, " ");

 /* Get the Month */
 tpt = strtok(NULL, " ");
 if(strlen(tpt) != 3) return NULL;
 strcat(GoodDate, tpt);
 strcat(GoodDate, " ");

 /* Get the year */
 tpt = strtok(NULL, " ");
 if(atoi(tpt) > 1900)
  tpt += 2;
 if(strlen(tpt) != 2) return NULL;
 strcat(GoodDate, tpt);
 strcat(GoodDate, " ");

 /* Get the time */
 tpt = strtok(NULL, " ");
 if(strlen(tpt) != 8) return NULL;
 strcat(GoodDate, tpt);

 Str->cpt = GoodDate;
 Str->cnt = strlen(GoodDate);

 return GoodDate;

}

/**/
/* Find the start of message body by looking for first LFLF */
/**/

char * pascal FindMessageStart(char *cptr, int count)
{
 static char *pt;

 while((pt = (char *)memchr(cptr, LF, count)) != NULL){

  if(*(pt + 1) == LF){
   pt += 2;
   return pt;
  EndIf
  else{
   count -= (pt - cptr + 1);
   cptr = ++pt;
  EndElse

 EndWhile

 return NULL;
}

/**/
/* Find the start of the field after the requested header */
/**/

char * pascal FindHeader(char *cptr, int count, char *findme)
{
 int flag;
 static char *pt;

 flag = 0;
 while((pt = (char *)memchr(cptr, *findme, count)) != NULL){
  if(memicmp(pt, findme, strlen(findme)) == 0){
   /* Found it */
   flag++;
   break;
  EndIf
  else{
   count -= (pt - cptr + 1);
   cptr = ++pt;
  EndElse
 EndWhile

 if(flag){
  /* Now get to the character after the ':' */
  /* pt points to first char of string now */
  pt++ ;
  while(!isspace(*pt)){
   pt++;
  EndWhile
  pt++;
 EndIf
 else
  pt = NULL;

 return pt;

}

void FredUfStartup(void)
{
 fprintf(stderr,"* Fred-Uf Version %s ", UFVERSION);
 fprintf(stderr,"Built %s at time %s *\n", __DATE__, __TIME__);
 fprintf(stderr,"* Copyright 1990, 1991 J. W. Hance - ");
 fprintf(stderr,"All Rights Reserved *\n");
}

void FredUfUsage(void)
{
  fprintf(stderr, "\nERROR: Improper commandline usage.\n");
  fprintf(stderr, "usage: fred-uf -c config [-quiet] <batch >packet\n");
  fprintf(stderr, "where: config is the configuration file.\n");
  fprintf(stderr, "       -quiet supresses a lot of messages.\n");
  fprintf(stderr, "       batch is the DECOMPRESSED batched news file.\n");
  fprintf(stderr, "       packet is the FidoNet mail packet to build.\n");
  exit(2);
}

/**/
/* Output the message */
/**/

int pascal DoMessage(char *cpt,
          unsigned int count,
	      FILE *logfp,
	      unsigned int headers,
	      int *total_processed,
	      int *total_copied,
	      int *total_dumped,
	      char *ascii_time,
	      char *origin,
	      char *origin_node,
	      char *domain,
	      struct nodeaddr *fnode,
	      struct nodeaddr *tnode,

#ifdef __COMPACT__ || __LARGE__
	      struct tofidos far *firstfidosptr,
#endif
#ifdef __SMALL__ || __MEDIUM__
	      struct tofidos *firstfidosptr,
#endif
	      struct MsgHeader *MsgHeader,
	      unsigned long length)

{
 char *tmp, *from;
 char *start_of_message, *end_of_message;
 int flag, header_length, i;
 static char dumped_group[128];

#ifdef __COMPACT__ || __LARGE__
	      struct tofidos far *tofidosptr;
          struct mgroups far *mgptr;
#endif
#ifdef __SMALL__ || __MEDIUM__
	      struct tofidos *tofidosptr;
          struct mgroups *mgptr;
#endif

   struct Str Str;

   start_of_message = FindMessageStart(cpt, count);

   if(start_of_message == NULL){
    /* Need to do error recovery here !!! */
    fprintf(stderr, "WARNING: Message body seems to be corrupt\n");
    fprintf(logfp, "!FindMessageStart()/CORRUPT?\n");
    fprintf(stderr, "Atempting to continue...\n");
    return(-1);
   EndIf

   header_length = start_of_message - cpt;
   end_of_message = cpt + count;

   /* No news group is an error */

   if((mgptr = FindNewsGroup(cpt, header_length, &Str)) == NULL)
    return (-1);

   /* Now find the proper EchoMail area to send it to from tofidos */
   tofidosptr = firstfidosptr;
   flag = 0;

   i = 0;

   do{
    if(strcmp(mgptr->g[i], tofidosptr->uucpname) == 0){
     flag++;

     /* Write the message out with the right AREA */

     /* Write the message header */
     fwrite(MsgHeader, sizeof(struct MsgHeader), 1, stdout);

     /* Switch the file to text mode */
     fflush(stdout); /* Flush it out first */
     setmode(fileno(stdout), O_TEXT); /* Set it to Text mode */

     /* Write the DATE: field */
     /* Use the current message date if it can be found */
     /* Otherwise, use the current system date */

     if(FindDate(cpt, header_length, &Str) == NULL)
      fprintf(stdout, "%s%c", ascii_time, 0);
     else{
      fwrite(Str.cpt, sizeof(char), Str.cnt, stdout);
      fputc(0, stdout);
     EndElse

     /* Write the TO: field */
     fputs("All", stdout);
     fputc(0, stdout);

     /* Write the FROM: field */

     from = FindFrom(cpt, header_length, &Str);
     if(from == NULL){
      fputs("-Unknown-", stdout);
      fputc(0, stdout);
     EndIf
     else{
      fwrite(Str.cpt, sizeof(char), Str.cnt, stdout);
      fputc(0, stdout);
     EndElse

     /* Write the SUBJECT: Field */

     if(FindSubject(cpt, header_length, &Str) == NULL){
      fprintf(stdout, " ");
     EndIf
     else{
      fwrite(Str.cpt, sizeof(char), Str.cnt, stdout);
     EndElse
     fputc(0, stdout);

     /* Put in the AREA tag */
     fputs("AREA:", stdout);
     fputs(tofidosptr->fidoname, stdout);
     fputc('\r', stdout);

     /* Put out a FSC-0044 PTH line */
     fprintf(stdout, "%cPTH %s@%s\r", 0x01, origin_node, domain);

     /* Optionally write out the UseNet headers */
     if(headers){
      WriteUseNetHeaders(headers, cpt, header_length, &Str, i, mgptr);
     EndIf

     /* Write out the message body */

     fwrite(start_of_message, sizeof(char),
	     end_of_message - start_of_message, stdout);

     /* If it is a long message, then copy out the rest of the text */
     if(length){

      /* Print a warning just in case */
       fprintf(stderr, "\r** WARNING - BIG (%8ld byte) Message was output in packet **\n",
			 length);
       fprintf(logfp, "+BIG <%8ld>\n",
			 length);

      length -= count;
      while(length){
       count = fread(cpt, sizeof(char),
		     (length > ALLOC_SIZE) ? ALLOC_SIZE : length, stdin);

      /* Make sure we read as many bytes as we asked for */
      if(count != ((length > ALLOC_SIZE) ? ALLOC_SIZE : length)){
       fprintf(stderr, "ERROR: reading stdin - message CORRUPT?\n");
      EndIf

      fwrite(cpt, sizeof(char), count, stdout);
       length -= count;

      EndWhile
     EndIf

     fprintf(stdout, "\n--- Fred-Uf %s\n", UFVERSION);
     fprintf(stdout, " * Origin: %s (%s)\n", origin, origin_node);
     fprintf(stdout, "%cPID: %s\n", 0x01, PIDVERSION);
     fprintf(stdout, "SEEN-BY: %d/%d %d/%d\n\n%c", tnode->net, tnode->node,
           fnode->net, fnode->node, 0);

     /* Switch back to binary mode on file */
     fflush(stdout);
     setmode(fileno(stdout), O_BINARY);

     if(quiet == 0){
      strcpy(dumped_group, tofidosptr->uucpname);
       fprintf(stderr, "%5d : Message COPIED (%s)\n",
              *total_processed, dumped_group);
     EndIf
     *total_copied += 1;
     tofidosptr->count++;
     break;
    EndIf

    else{

     /* Point to the next record */
     tofidosptr = tofidosptr->next;
     if((tofidosptr == NULL) && (i < mgptr->howmany-1)){
      i++;
      tofidosptr = firstfidosptr;
     EndIf


    EndElse

   } while(tofidosptr != NULL);

   if(flag == 0){
    if(quiet == 0){
     strncpy(dumped_group, Str.cpt, Str.cnt);
     *(dumped_group + Str.cnt) = NULL;
     fprintf(stderr, "%5d : Message DUMPED (%s)\n",
             *total_processed, dumped_group);
    EndIf
    *total_dumped += 1;
   EndIf

   return 0;
}


/**/
/* Output the message in SPLIT mode */
/**/

int pascal DoMessageSplit(char *cpt,
          unsigned int count,
	      FILE *logfp,
	      unsigned int headers,
	      int *total_processed,
	      int *total_copied,
	      int *total_dumped,
	      char *ascii_time,
	      char *origin,
	      char *origin_node,
	      char *domain,
	      struct nodeaddr *fnode,
	      struct nodeaddr *tnode,

#ifdef __COMPACT__ || __LARGE__
	      struct tofidos far *firstfidosptr,
#endif
#ifdef __SMALL__ || __MEDIUM__
	      struct tofidos *firstfidosptr,
#endif
	      struct MsgHeader *MsgHeader,
	      unsigned int split_count,
	      unsigned int of_split_count)

{
 char *tmp, *from;
 static char *remember_date, *remember_from, *remember_area;
 static char *remember_subject;
 static int remember_date_cnt, remember_from_cnt, remember_area_cnt;
 static int remember_subject_cnt;
 static int remember_i;
 static int split_number;
 static int da, fa, aa, sa;
 char *start_of_message, *end_of_message;
 int flag, header_length, i;
 static char dumped_group[128];

#ifdef __COMPACT__ || __LARGE__
	      struct tofidos far *tofidosptr;
#endif
#ifdef __SMALL__ || __MEDIUM__
	      struct tofidos *tofidosptr;
#endif

 struct Str Str;
 static struct mgroups *mgptr;

   if(split_count == 1){
    da = fa = aa = sa = 0;
    start_of_message = FindMessageStart(cpt, count);
    split_number = *total_processed;
   EndIf
   else
    start_of_message = cpt;

   if(start_of_message == NULL){
    /* Need to do error recovery here !!! */
    fprintf(stderr, "ERROR: Message body seems to be corrupt\n");
    fprintf(logfp, "!FindMessageStart()/CORRUPT?\n");
    fprintf(stderr, "Attempting to continue...\n");
    return(-1);
   EndIf

   header_length = start_of_message - cpt;
   end_of_message = cpt + count;

   if(split_count == 1){
    if((mgptr = FindNewsGroup(cpt, header_length, &Str)) == NULL)
     return (-1);
    remember_area = AllocateString(Str.cnt);
    aa++;
    memcpy(remember_area, Str.cpt, Str.cnt);
    remember_area_cnt = Str.cnt;
   EndIf
   else{
    Str.cnt = remember_area_cnt;
    Str.cpt = remember_area;
   EndIf

   /* Now find the proper EchoMail area to send it to from tofidos */
   tofidosptr = firstfidosptr;
   flag = 0;

   if(split_count == 1)
    i = 0;
   else
    i = remember_i;

   do{
    if(strcmp(mgptr->g[i], tofidosptr->uucpname) == 0){
     flag++;
     remember_i = i;

     /* Write the message out with the right AREA */

     /* Write the message header */
     fwrite(MsgHeader, sizeof(struct MsgHeader), 1, stdout);

     /* Switch the file to text mode */
     fflush(stdout); /* Flush it out first */
     setmode(fileno(stdout), O_TEXT); /* Set it to Text mode */

     /* Write the DATE: field */
     /* Use the current message date if it can be found */
     /* Otherwise, use the current system date */
     if(split_count == 1){
      if(FindDate(cpt, header_length, &Str) == NULL){
       fprintf(stdout, "%s%c", ascii_time, 0);
       remember_date = AllocateString(strlen(ascii_time));
       da++;
       memcpy(remember_date, ascii_time, strlen(ascii_time));
       remember_date_cnt = strlen(ascii_time);
      EndIf
      else{
       fwrite(Str.cpt, sizeof(char), Str.cnt, stdout);
       fputc(0, stdout);
       remember_date = AllocateString(Str.cnt);
       da++;
       memcpy(remember_date, Str.cpt, Str.cnt);
       remember_date_cnt = Str.cnt;
      EndElse
     EndIf
     else{
      fwrite(remember_date, sizeof(char), remember_date_cnt, stdout);
      fputc(0, stdout);
     EndElse

     /* Write the TO: field */
     fputs("All", stdout);
     fputc(0, stdout);

     /* Write the FROM: field */
     if(split_count == 1){
      from = FindFrom(cpt, header_length, &Str);
      if(from == NULL){
       fputs("Unknown", stdout);
       fputc(0, stdout);
       remember_from = AllocateString(strlen("Unknown"));
       fa++;
       memcpy(remember_from, "Unknown", strlen("Unknown"));
       remember_from_cnt = strlen("Unknown");
      EndIf
      else{
       fwrite(Str.cpt, sizeof(char), Str.cnt, stdout);
       fputc(0, stdout);
       remember_from = AllocateString(Str.cnt);
       fa++;
       memcpy(remember_from, Str.cpt, Str.cnt);
       remember_from_cnt = Str.cnt;
      EndElse
     EndIf
     else{
      fwrite(remember_from, sizeof(char), remember_from_cnt, stdout);
      fputc(0, stdout);
     EndElse

     /* Write the SUBJECT: Field */
     if(split_count == 1){
      if(FindSubject(cpt, header_length, &Str) == NULL){
       fprintf(stdout, "Unknown");
      EndIf
      else{
       fwrite(Str.cpt, sizeof(char), Str.cnt, stdout);
      EndElse
      fputc(0, stdout);
      remember_subject = AllocateString(Str.cnt);
      sa++;
      memcpy(remember_subject, Str.cpt, Str.cnt);
      remember_subject_cnt = Str.cnt;
     EndIf
     else{
      if(remember_subject_cnt > 66) remember_subject_cnt = 66;

      fprintf(stdout, "%02d/", split_count);
      fwrite(remember_subject, sizeof(char),
	     remember_subject_cnt, stdout);
      fputc(0, stdout);
     EndElse

     /* Put in the AREA tag */
     fputs("AREA:", stdout);
     fputs(tofidosptr->fidoname, stdout);
     fputc('\r', stdout);

     /* Put out a FSC-0047 SPLIT line */
     fprintf(stdout, "%cSPLIT: %s @%d/%d     %5d %02d/%02d +++++++++++\r",
		      0x01, ascii_time, tnode->net, tnode->node,
		      split_number, split_count, of_split_count);
     /* Put out a FSC-0044 PTH line */
     fprintf(stdout, "%cPTH %s@%s\r", 0x01, origin_node, domain);

     /* Optionally write out the UseNet headers */
     if(headers & (split_count == 1)){
      WriteUseNetHeaders(headers, cpt, header_length, &Str, i, mgptr);
     EndIf

     /* Write out the message body */
     fwrite(start_of_message, sizeof(char),
	     end_of_message - start_of_message, stdout);

     /* Put in a blank line except on the last part */
     if(split_count != of_split_count)
      fprintf(stdout, "\n");

     fprintf(stdout, "\n--- Fred-Uf %s\n", UFVERSION);
     fprintf(stdout, " * Origin: %s (%s)\n", origin, origin_node);
     fprintf(stdout, "%cPID: %s\n", 0x01, PIDVERSION);
     fprintf(stdout, "SEEN-BY: %d/%d %d/%d\n\n%c", tnode->net, tnode->node,
           fnode->net, fnode->node, 0);

     /* Switch back to binary mode on file */
     fflush(stdout);
     setmode(fileno(stdout), O_BINARY);

     if(quiet == 0){
      fprintf(stderr, "%5d : Message SPLIT  (%s)  \n",
               *total_processed, tofidosptr->uucpname);
     EndIf

     *total_copied += 1;
     tofidosptr->count++;
     break;
    EndIf

    else{

     /* Point to the next record */
     tofidosptr = tofidosptr->next;
     if((tofidosptr == NULL) && (i < mgptr->howmany-1)){
      i++;
      tofidosptr = firstfidosptr;
     EndIf


    EndElse

   } while(tofidosptr != NULL);

   if(flag == 0){
    strncpy(dumped_group, Str.cpt, Str.cnt);
    *(dumped_group + Str.cnt) = NULL;
    fprintf(stderr, "%5d : Message DUMPED (%s) \n",
             *total_processed, dumped_group);
    *total_dumped += 1;
   EndIf

   if(split_count == of_split_count){
    if(aa) free(remember_area);
    if(da) free(remember_date);
    if(fa) free(remember_from);
    if(sa) free(remember_subject);
   EndIf

   return 0;
}


/**/
/* Figure out which headers to output and which to hide */
/**/

unsigned int WhichHeaders(char *s)
{
 static unsigned int tmp, factor;
 int i;

 if(strlen(s) != 16){
  fprintf(stderr, "ERROR: In 'HEADERS YES <code>' -  <code> must specify exactly 16 fields.\n");
  exit(2);
 EndIf

 tmp = 0;
 factor = 1;

 for(i=15; i>=0; i--){
  if(*(s+i) == '1'){
   tmp += factor;
  EndIf
  factor *= 2;
 EndFor

 return tmp;
}

/**/
/* Write out the UseNet Headers to output */
/**/

void pascal WriteUseNetHeaders(unsigned int headers, char *cpt,
		   int header_length, struct Str *Str,
                   int i, struct mgroups *mgptr)
{
			 
  if(headers & 0x0020){
   if(FindWholeFrom(cpt, header_length, Str) != NULL){
	if(headers & 0x2000) fputc(0x01, stdout);
     fputs("From: ", stdout);
    fwrite(Str->cpt, sizeof(char), Str->cnt, stdout);
	fputc('\r', stdout);
   EndIf
  EndIf

  if(headers & 0x0010){
   if(FindPath(cpt, header_length, Str) != NULL){
	if(headers & 0x1000) fputc(0x01, stdout);
     fputs("Path: ", stdout);
	fwrite(Str->cpt, sizeof(char), Str->cnt, stdout);
	fputc('\r', stdout);
   EndIf
  EndIf

  if(headers & 0x0008){
   if(mgptr != NULL){
	if(headers & 0x0800) fputc(0x01, stdout);
     fputs("Newsgroups: ", stdout);
/*    fwrite(Str->cpt, sizeof(char), Str->cnt, stdout); */
    fprintf(stdout, "%s", mgptr->g[i]);
	fputc('\r', stdout);
   EndIf
  EndIf

  if(headers & 0x0004){
   if(FindSubject(cpt, header_length, Str) != NULL){
	if(headers & 0x0400) fputc(0x01, stdout);
     fputs("Subject: ", stdout);
	fwrite(Str->cpt, sizeof(char), Str->cnt, stdout);
	fputc('\r', stdout);
   EndIf
  EndIf

  if(headers & 0x0040){
   if(FindOrganization(cpt, header_length, Str) != NULL){
	if(headers & 0x4000) fputc(0x01, stdout);
     fputs("Organization: ", stdout);
	fwrite(Str->cpt, sizeof(char), Str->cnt, stdout);
	fputc('\r', stdout);
   EndIf
  EndIf

  if(headers & 0x0080){
   if(FindReplyTo(cpt, header_length, Str) != NULL){
	if(headers & 0x8000) fputc(0x01, stdout);
     fputs("Reply-To: ", stdout);
	fwrite(Str->cpt, sizeof(char), Str->cnt, stdout);
	fputc('\r', stdout);
   EndIf
  EndIf

  if(headers & 0x0002){
   if(FindMessageID(cpt, header_length, Str) != NULL){
	if(headers & 0x0200) fputc(0x01, stdout);
     fputs("Message-ID: ", stdout);
	fwrite(Str->cpt, sizeof(char), Str->cnt, stdout);
	fputc('\r', stdout);
   EndIf
  EndIf

  if(headers & 0x0001){
   if(FindWholeDate(cpt, header_length, Str) != NULL){
	if(headers & 0x0100) fputc(0x01, stdout);
     fputs("Date: ", stdout);
	fwrite(Str->cpt, sizeof(char), Str->cnt, stdout);
	fputc('\r', stdout);
   EndIf
  EndIf

  /* Put out an extra CR to separate headers from message body */
  /* if any of them are output */
  if(headers & 0x00ff)
   fputc('\r', stdout);
}
