/*****************************************************************
 *                                                               *
 * VolSubs.c = Put/Get volume name from a disk                   *
 *                                                               *
 * written for Lattice-C, modifications required for other C     *
 * compilers.                                                    *
 * SUPPORTED UNDER DOS 3.x only...                               *
 *                                                               *
 * Copyright (c) 1987      Repstad Computer Consultants          *
 *                         RFD #1, Box 3720                      *
 *                         Sheldon, VT 05483                     *
 *                                                               *
 * Author can be reached via: Black Creek BBS (802) 933-5133     *
 *                            Address mail to SYSOP.             *
 *                                                               *
 *****************************************************************/

#include <dos.h>
#include <string.h>

#define XFCB_PREFIX	0xff	                  /* prefix for extended FCBs   */
#define LABEL_ATTRIB	0x08	                  /* label attribute bit        */

#define CLOSE_FILE	0x10	                  /* bdos fn # to close a file  */
#define CREATE_FILE	0x16	                  /* bdos fn # to create a file */
#define RENAME_FILE	0x17	                  /* bdos fn # to rename a file */

char xfcb[48];                               /* space for extended fcb     */

struct FILEINFO info;                        /* File information structure */
                                             /* see dos.h for details      */
 
/*****************************************************************
 *                                                               *
 * short putvol(volnam,drive) - Puts a volume name on a diskette *
 *                                                               *
 * returned values:  0 = success                                 *
 *                   1 = NULL volume name                        *
 *                   2 = bad drive letter                        *
 *                   3 = unable to create volume                 *
 *                                                               *
 *                                                               *
 *****************************************************************/

short putvol(volnam,drive)

char *drive;                                 /* input drive letter         */
char *volnam;                                /* input volume name          */
{
   char *c;                                  /* temp string pointer        */
	int d = 0;                                /* drive number               */
	int i;                                    /* utility counter            */

   if (*volnam == '\0')
      return 1;                              /* null volume name!          */

   if (*drive >= 'A' && *drive <= 'Z')       /* determine drive #          */
      d = *drive - 'A' + 1;
   else if (*drive >= 'a' && *drive <= 'z')
      d = *drive - 'a' + 1;	
   else 
      return 2;                              /* invalid drive # return     */

	xfcb[0] = XFCB_PREFIX;	                  /* says this is an extd fcb   */
	xfcb[6] = LABEL_ATTRIB;	                  /* says this is the vol label */
	xfcb[7] = d;		                        /* put the drive specifier in */
                                             /* the fcb                    */


	for(i=1;i<12;i++) xfcb[7+i]='?';          /* wildcard matches any label */

	for(c=volnam, i=0;i<11;i++) 	            /* fill the second part of the*/
   {                                         /* fcb with the new label name*/
		if(*c != '\0') xfcb[0x18+i] = *c++;	   
		else xfcb[0x18+i] = ' ';               /* pad new name with blanks   */
	}

/***************************************************************************
 *                                                                         *
 * first attempt to rename the existing volume name, if that fails (i.e.   *
 * no volume on the disk) try to create a new file with a volume           *
 * attribute. If that fails return and error. For more info on how this    *
 * works see the book "Advanced MSDOS", by Ray Duncan, Microsoft Press,    *
 * pp156-159.                                                              *
 *                                                                         *
 ***************************************************************************/

	if(bdos(RENAME_FILE,xfcb) & 0xff) 	      /* try to RENAME.            */
   {                                         
                                             /* RENAME failed...           */
		for(i=0; i<11; i++)	                  /* move the new name into     */
			xfcb[8+i] = xfcb[0x18+i]; 	         /* the normal filename spot   */

		if(bdos(CREATE_FILE,xfcb) & 0xff)      /* ok rename failed, create   */
         return 3;                           /* oops! couldn't create it   */
		else
         bdos(CLOSE_FILE,xfcb);              /* all ok...close and return  */
	}
   return 0;
}

/*****************************************************************
 *                                                               *
 * short getvol(volnam,drive) - gets volume name from a diskette *
 *                                                               *
 * returned values: 0 - Volume name found                        *
 *                  1 - bad drive letter                         *
 *                  2 - no volume name found                     *
 *                                                               *
 *                                                               *
 *****************************************************************/

short getvol(volnam,drive)

char *drive;                                 /* input drive letter         */
char *volnam;                                /* returned volume name       */

{
   int attr;                                 /* file attribute byte        */


   struct FILEINFO *finfo;                   /* File information structure */
                                             /* see dos.h for details      */
   static char *fname = "A:*.*\0";           /* search file name           */


                                             

   if ((*drive >= 'A' && *drive <= 'Z') || (*drive >= 'a' && *drive <= 'z'))
      *fname = *drive;                       /* get drive letter           */
   else
      return 1;                              /* bad drive letter           */


   attr = LABEL_ATTRIB;                      /* set attr=volume only       */

   finfo = &info;
   if (dfind(finfo,fname,attr))              /* go find the volume         */
   {
      *volnam = '\0';                        /* return NULL volnam         */
      return 2;                              /* vol not found              */
   }
                                             
   volnam = strcpy(volnam,finfo->name);      /* got vol return it          */
   return 0;                                 /* return success             */

}

              3 = unable to create volume                 