#ident "@(#)/usr/lane/bin/xdmp.c v1.0 by lane watts"

#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <time.h>
#include "true.h"

#define BFSIZE 512

#define LPP 60

extern int errno;

int ercd;
int pgnate;
int siarg;
int	lns,	pg,		pg1;

char	pgmname[81];
char	timestr[32];
char	hexstr[50];
char	charstr[18];

time_t	tm;

struct	dumpspec {
	char flnm[81];
	long st;
	long ln;
	long nd;
};

extern char *optarg;
extern int optind;

void prtusg();						/* print usage message					*/
struct dumpspec *getspecs();		/* get specs for file to dump			*/
void dmpfl();						/* produce dump of one file				*/
void bldstr();						/* build hex and ascii strings			*/
void prthdr();						/* print header							*/

main(argc,argv)
int argc;
char *argv[];
{
	int i, c;

	ercd = 0; pg1 = TRUE; siarg = 1;
	memset(pgmname,'\0',sizeof(pgmname));
	strncpy(pgmname,argv[0],sizeof(pgmname)-1);

	pgnate = FALSE;
	while ((c = getopt(argc,argv,"h?p")) != -1) {
		switch (c) {
		case 'p':	pgnate = TRUE;
					break;
		case 'h':
		case '?':
		default:	prtusg();
					siarg = 0;
					break;
		}
	}

	if ((argc - optind) == 0)
		argc += siarg;

	for (i=optind; i<argc; i++) {
		dmpfl(getspecs(argv[i]));
	}

    exit(ercd);
}

void prtusg()						/* print usage message					*/
{
	printf("\n");
	printf("%s (rev 2.2) - produce a listing of a file in hexadecimal and ascii\n",pgmname);
	printf("usage:\n");
	printf("    %s [options] filename,s,c,e [filename,s,c,e ...]\n",pgmname);
	printf("where options are:\n");
	printf(" -h        = display this list of parameters\n");
	printf(" -?        = display this list of parameters\n");
	printf(" -p        = paginate output\n");
	printf("and:\n");
	printf("  filename = name of the file to be listed\n");
	printf("  s        = byte position within the file to start the listing\n");
	printf("  c        = number of bytes to list\n");
	printf("             if e is specified, c is ignored.\n");
	printf("  e        = byte position within the file to end the listing\n");
	printf("the listing will always go to the standard output file (stdout).\n");
	printf("stdin may be used by leaving the file name blank.\n");
	printf("s, c, and e may be omitted and will default as follows:\n");
	printf("if s is omitted, it will default to 0\n");
	printf("if c is omitted, it will default to end-start\n");
	printf("if e is omitted, it will default to start+count\n");
	printf("if c and e are omitted, it will default to the end of the file\n");
	printf("\n");
	return;
}

struct dumpspec *getspecs(spstr)	/* get specs for file to dump			*/
char	*spstr;
{
    int 	i;
	char	*ps1,	*ps2;
	static struct dumpspec ds;

	memset(&ds,'\0',sizeof(ds));
	ps1 = spstr;
	for (i=0; ((i<4) && (ps1 != (char *)0)); i++) {
		ps2 = strchr(ps1,',');
		if (ps2 != (char *)0) {
			*ps2 = '\0';
			ps2++;
		}
		switch (i) {
		case 0:	strcpy(ds.flnm,ps1);
				break;
		case 1: ds.st = atol(ps1);
				break;
		case 2: ds.ln = atol(ps1);
				break;
		case 3: ds.nd = atol(ps1);
				break;
		}
		ps1 = ps2;
	}
	if ((ds.ln == 0) && (ds.nd != 0))
		ds.ln = (ds.nd+1) - ds.st;
	if ((ds.nd == 0) && (ds.ln != 0))
		ds.nd = (ds.st + ds.ln) - 1;

	return(&ds);
}

void dmpfl(d)					/* produce dump of one file				*/
struct dumpspec *d;
{
	int		c;
    int 	i,	j,	k;
	int		bp,		lp,		cp;
	int		cnt,	tcnt;

	char	inbuf[BFSIZE];
	char	*pc;
	FILE	*fp;

	lns = LPP+1; pg = 0;
	tm = time(0);
	strncpy(timestr,ctime(&tm),sizeof(timestr));
	pc = strchr(timestr,'\n');				/* replace \n w/ null */
	if (pc != (char *)0)
		*pc = '\0';

	if ((int)strlen(d->flnm) <= 0) {
		fp = stdin;
		strcpy(d->flnm,"(stdin)");
	} else {
		if ((fp = fopen(d->flnm,"r")) == (FILE *)0) {
			sprintf(inbuf,"error opening file %s",d->flnm);
			perror(inbuf);
			printf("\n\n");
			ercd = errno;
			return;
		}
	}

	if (fseek(fp,d->st,0) != 0) {
		sprintf(inbuf,"error reading byte %d of %s\007",d->st,d->flnm);
		perror(inbuf);
		printf("\n\n");
		ercd = errno;
		return;
	}

	k = d->st % 16;
	tcnt = d->st-k;
	cnt = -1; bp = 0;
    do {
		if ((d->ln > 0) && (tcnt > d->nd))
			break;
		memset(hexstr,' ',49); hexstr[49] = '\0';
		memset(charstr,' ',16); charstr[16] = '\0';
		for (j=k; j<16; j++) {
			if (bp >= cnt) {
				cnt = fread(inbuf,sizeof(*inbuf),BFSIZE,fp);
				bp = 0;
				if (cnt <= 0)
					break;
			}
			if ((d->ln > 0) && ((tcnt+j) > d->nd))
				break;
			bldstr(j,inbuf[bp++]);
		}
		lns++;
		if (lns > LPP) {
			if (pgnate || (pg == 0))
				prthdr(d);
		}
		printf("%6.6d |%s |%s|\n",tcnt,hexstr,charstr);
		tcnt += 16;
		k = 0;
		if ((d->ln > 0) && (tcnt > d->nd))
			break;
    } while (cnt > 0);

	if (!pgnate)
		printf("\n\n\n");
	fclose(fp);
	return;
}

void bldstr(pos,chr)				/* build hex and ascii strings			*/
int		pos;
char	chr;
{
	int			offset;
	unsigned	uns;
	char		tmp[20];
	char		nohi;

	offset = (pos<8) ? (pos*3)+1 : (pos*3)+2;
	uns = (unsigned)(chr & 255);
	sprintf(tmp,"%2.2x",uns);
	memcpy(&hexstr[offset],tmp,2);
	nohi = (unsigned)(uns & (unsigned)127);
	charstr[pos] = (nohi>=32 && nohi<127) ? nohi : '.';
}

void prthdr(d)						/* print header							*/
struct dumpspec *d;
{
	if ((!pg1) && pgnate)
		printf("\f");
	else
		(pg1 = FALSE);
	pg++; lns = 0;

	printf("%s         xdmp r2.2 - hex/ascii dump          page: %d\n",timestr,pg);
	printf("file: %s  start: %d  count: %d  end: %d\n\n",d->flnm,d->st,d->ln,d->nd);
	return;
}
